@@ -3,14 +3,12 @@
 // GkClient.cxx
 //
 // Copyright (c) Citron Network Inc. 2001-2003
+// Copyright (c) 2002-2010, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
-// initial version: 02/27/2002
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -201,6 +199,7 @@ void NATClient::SendInfo(int state)
 	information.SetCallState(Q931::CallStates(state));
 	information.Encode(buf);
 	if (socket) {
+		PrintQ931(5, "Send to ", socket->GetName(), &information, NULL);
 		socket->TransmitData(buf);
 	}
 }
@@ -361,14 +360,19 @@ public:
   }
 
   const STUNmessageHeader * operator->() const { return (STUNmessageHeader *)theArray; }
-  
+
+// ignore overflow warning when comparing length
+#if (!_WIN32) && (GCC_VERSION >= 40400)
+#pragma GCC diagnostic ignored "-Wstrict-overflow"
+#endif
+
   STUNattribute * GetFirstAttribute() { 
 
     int length = ((STUNmessageHeader *)theArray)->msgLength;
     if (theArray == NULL || length < (int) sizeof(STUNmessageHeader))
       return NULL;
 
-    STUNattribute * attr = (STUNattribute *)(theArray+sizeof(STUNmessageHeader)); 
+    STUNattribute * attr = (STUNattribute *)(theArray+sizeof(STUNmessageHeader));
     STUNattribute * ptr = attr;
 
     if (attr->length > GetSize() || attr->type >= STUNattribute::MaxValidCode)
@@ -383,7 +387,7 @@ public:
     if (length != 0)
       return NULL;
 
-    return attr; 
+    return attr;
   }
 
   bool Validate()
@@ -563,19 +567,26 @@ private:
 
     GkClient *                    m_client;
 	NatTypes                      m_nattype;
-	bool                             m_shutdown;
-	PMutex                         m_portCreateMutex;
+	bool                          m_shutdown;
+	PMutex                        m_portCreateMutex;
+    int                           m_socketsForPairing;
+    int                           m_pollRetries;
 
 };
 
 STUNClient::STUNClient(GkClient * _client, const H323TransportAddress & addr)
-:  m_client(_client), m_nattype(UnknownNat), m_shutdown(false)
+:  m_client(_client), m_nattype(UnknownNat), m_shutdown(false),
+   m_socketsForPairing(4), m_pollRetries(3)
 {
 
 	PIPSocket::Address ip;
 	WORD port;
 	addr.GetIpAndPort(ip,port);
+#ifdef hasNewSTUN
+    m_serverAddress = PIPSocketAddressAndPort(ip, port);
+#else
 	SetServer(ip, port);
+#endif
 
 	STUNportRange ports;
 	ports.LoadConfig("Proxy", "RTPPortRange", "1024-65535");
@@ -611,7 +622,7 @@ void STUNClient::Exec()
 
 		// Wait 500 ms until the RCF has been processed before running tests 
 		// to prevent blocking.
-		mute.Wait(500); 
+		mute.Wait(500);
 	
 		// Get a valid NAT type....
 	    // We do the test 3 times as often the first test gives odd results
@@ -631,7 +642,7 @@ void STUNClient::Exec()
 		// Keep this job (thread) open so that creating STUN ports does not hold up 
 		// the processing of other calls
 		while (!m_shutdown) {
-			PProcess::Sleep(100);
+			PThread::Sleep(100);
 		}
 
 	ReadUnlock unlockConfig(ConfigReloadMutex);
@@ -646,6 +657,43 @@ void STUNClient::OnDetectedNAT(int nattype)
     m_client->H46023_TypeDetected(nattype);
 }
 
+#ifdef hasNewSTUN
+bool STUNClient::OpenSocketA(UDPSocket & socket, PortInfo & portInfo, const PIPSocket::Address & binding)
+{
+  if (!m_serverAddress.IsValid()) {
+    PTRACE(1, "STUN\tServer port not set.");
+    return false;
+  }
+
+  if (portInfo.basePort == 0) {
+    if (!socket.Listen(binding, 1)) {
+      PTRACE(3, "STUN\tCannot bind port to " << m_interface);
+      return false;
+    }
+  }  
+  else {
+    WORD startPort = portInfo.currentPort;
+    PTRACE(3, "STUN\tUsing ports " << portInfo.basePort << " through " << portInfo.maxPort << " starting at " << startPort);
+    for (;;) {
+      bool status = socket.Listen(binding, 1, portInfo.currentPort);
+      PWaitAndSignal mutex(portInfo.mutex);
+      portInfo.currentPort++;
+      if (portInfo.currentPort > portInfo.maxPort)
+        portInfo.currentPort = portInfo.basePort;
+      if (status)
+        break;
+      if (portInfo.currentPort == startPort) {
+        PTRACE(3, "STUN\tListen failed on " << m_interface << ":" << portInfo.currentPort);
+        return false;
+      }
+    } 
+  }
+
+  socket.SetSendAddress(m_serverAddress.GetAddress(), m_serverAddress.GetPort());
+
+  return true;
+}
+#else
 bool STUNClient::OpenSocketA(UDPSocket & socket, PortInfo & portInfo, const PIPSocket::Address & binding)
 {
   if (serverPort == 0) {
@@ -679,6 +727,7 @@ bool STUNClient::OpenSocketA(UDPSocket & socket, PortInfo & portInfo, const PIPS
          << portInfo.currentPort << '-' << portInfo.maxPort);
   return false;
 }
+#endif
 
 bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const PIPSocket::Address & binding)
 {
@@ -700,7 +749,7 @@ bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const P
   PList<STUNmessage> request;
   PList<STUNmessage> response;
 
-  for (i = 0; i < numSocketsForPairing; i++)
+  for (i = 0; i < m_socketsForPairing; i++)
   {
     PINDEX idx = stunSocket.Append(new STUNsocket);
     if (!OpenSocketA(stunSocket[idx], pairedPortInfo, binding)) {
@@ -714,16 +763,16 @@ bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const P
     response.Append(new STUNmessage);
   }
 
-  for (i = 0; i < numSocketsForPairing; i++)
+  for (i = 0; i < m_socketsForPairing; i++)
   {
-    if (!response[i].Poll(stunSocket[i], request[i], pollRetries))
+    if (!response[i].Poll(stunSocket[i], request[i], m_pollRetries))
     {
       PTRACE(1, "STUN\tServer unexpectedly went offline." << GetServer());
       return false;
     }
   }
 
-  for (i = 0; i < numSocketsForPairing; i++)
+  for (i = 0; i < m_socketsForPairing; i++)
   {
     STUNmappedAddress * mappedAddress = (STUNmappedAddress *)response[i].FindAttribute(STUNattribute::MAPPED_ADDRESS);
     if (mappedAddress == NULL)
@@ -736,9 +785,9 @@ bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const P
     stunSocket[i].externalIP = mappedAddress->GetIP();
   }
 
-  for (i = 0; i < numSocketsForPairing; i++)
+  for (i = 0; i < m_socketsForPairing; i++)
   {
-    for (PINDEX j = 0; j < numSocketsForPairing; j++)
+    for (PINDEX j = 0; j < m_socketsForPairing; j++)
     {
       if ((stunSocket[i].GetPort()&1) == 0 && (stunSocket[i].GetPort()+1) == stunSocket[j].GetPort()) {
         stunSocket[i].SetSendAddress(0, 0);
@@ -918,7 +967,7 @@ GkClient::GkClient()
 	m_handlers[3] = new GkClientHandler(this, &GkClient::OnIRQ, H225_RasMessage::e_infoRequest);
 
 #ifdef OpenH323Factory
-	m_password = PString();
+	m_password = PString::Empty();
     m_h235Authenticators = new H235Authenticators();
     PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
     PFactory<H235Authenticator>::KeyList_T::const_iterator r;
@@ -989,7 +1038,7 @@ void GkClient::OnReload()
 	PStringList gkHost;
 #if P_DNS
 	PStringList str;
-    if (PDNS::LookupSRV(gk,"_h323rs._udp.",str)) {
+    if (gk != "no" && PDNS::LookupSRV(gk, "_h323rs._udp.", str)) {
 		PTRACE(5, "EP\t" << str.GetSize() << " h323rs SRV Records found" );
 		  for (PINDEX i = 0; i < str.GetSize(); i++) {
 			PCaselessString newhost = str[i].Right(str[i].GetLength()-5);
@@ -1008,7 +1057,7 @@ void GkClient::OnReload()
 	
 	if (gk == "no") {
 		m_timer = 0;
-		m_rrjReason = PString();
+		m_rrjReason = PString::Empty();
 		return;
 	} else if (gk == "auto") {
 		m_gkaddr = INADDR_BROADCAST;
@@ -1340,7 +1389,7 @@ bool GkClient::SendARQ(Routing::SetupRequest & setup_obj, bool answer, int natof
 			RewriteE164(arq.m_destinationInfo, true);
 	}
 	arq.m_answerCall = answer;
-	// workaround for bandwidth, as OpenH323 library :p
+	// workaround for bandwidth
 	arq.m_bandWidth = 1280;
 
 #ifdef HAS_H46023
@@ -1390,7 +1439,7 @@ bool GkClient::SendARQ(Routing::FacilityRequest & facility_obj)
 		arq.m_destinationInfo = facility.m_alternativeAliasAddress;
 	}
 	arq.m_answerCall = false;
-	// workaround for bandwidth, as OpenH323 library :p
+	// workaround for bandwidth
 	arq.m_bandWidth = 1280;
 
 	return OnSendingARQ(arq, facility_obj) && WaitForACF(arq, request, &facility_obj);
@@ -1499,7 +1548,7 @@ bool GkClient::RewriteE164(
 bool GkClient::Discovery()
 {
 	m_loaddr = m_gkaddr;
-	m_gatekeeperId = PString();
+	m_gatekeeperId = PString::Empty();
 
 	if (!m_discoverParent)
 		return true;
@@ -1594,7 +1643,7 @@ bool GkClient::GetAltGK()
 		// increase our resent time to avoid flooding the parent
 		m_timer = (m_resend *= 2) * 1000;
 		if (m_resend >= m_gkfailtime) {
-			// TODO: not thread-safed
+			// TODO: not thread-safe ?
 			Toolkit::Instance()->GetRouteTable()->InitTable();
 			m_resend = m_gkfailtime;
 		}
@@ -1693,6 +1742,19 @@ bool GkClient::WaitForACF(H225_AdmissionRequest &arq, RasRequester & request, Ro
 				H225_AdmissionConfirm & acf = (*ras)->m_recvRAS;
 				Route route("parent", acf.m_destCallSignalAddress);
 				route.m_flags |= Route::e_toParent;
+				// check if destination has changed
+				if (acf.HasOptionalField(H225_AdmissionConfirm::e_destinationInfo)) {
+					// signal change of destination if caller supports canMapAlias
+					if (arq.HasOptionalField(H225_AdmissionRequest::e_canMapAlias)
+						&& arq.m_canMapAlias
+						&& acf.m_destinationInfo.GetSize() > 0 ) {
+						robj->SetFlag(Routing::RoutingRequest::e_aliasesChanged);
+						Routing::AdmissionRequest * orig_request = dynamic_cast<Routing::AdmissionRequest *>(robj);
+						if (orig_request) {
+							orig_request->GetRequest().m_destinationInfo = acf.m_destinationInfo;
+						}
+					}
+				}
 				robj->AddRoute(route);
 #ifdef HAS_H46023
 			  if (Toolkit::Instance()->IsH46023Enabled()) {
@@ -1826,7 +1888,7 @@ bool GkClient::H46023_TypeNotify(int & nattype)
 void GkClient::HandleP2P_ACF(callptr m_call, H460_FeatureStd * feat)
 {
     if (feat->Contains(Std24_NATInstruct)) {
-		unsigned NATinst = feat->Value(Std24_NATInstruct); 
+		unsigned NATinst = feat->Value(Std24_NATInstruct);
 
         PTRACE(4,"GKC\tH46024 strategy for call set to " << NATinst);
 
