@@ -2,13 +2,25 @@
 //
 // GkStatus.cxx
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2011, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
+#include "config.h"
+
+#ifdef HAS_LIBSSH
+#ifdef _WIN32
+#pragma comment(lib, LIBSSH_LIB)
+#endif
+#include "libssh/libssh.h"
+#include "libssh/server.h"
+#endif
+
 #include <ptlib.h>
 #include <ptlib/sockets.h>
 #include <ptclib/telnet.h>
@@ -60,7 +72,7 @@ public:
 
 	bool NeedEcho() const { return m_needEcho; }
 
-private:
+protected:
 	enum State {
 		StateNormal,
 		StateCarriageReturn,
@@ -97,7 +109,7 @@ public:
 		
 	virtual ~StatusClient() {}
 
-	bool ReadCommand(
+	virtual bool ReadCommand(
 		/// command that has been read (if ReadCommand succeeded)
 		PString& cmd,
 		/// should the command be echoed (also NeedEcho() has to be true)
@@ -113,7 +125,7 @@ public:
 		@return
 		true if the message has been sent (or ignored because of trace level).
 	*/
-	bool WriteString(
+	virtual bool WriteString(
 		/// string to be sent through the socket
 		const PString& msg, 
 		/// output trace level assigned to the message
@@ -133,7 +145,7 @@ public:
 		@return
 		true if the client has been granted access, false to reject the client.
 	*/
-	bool Authenticate();
+	virtual bool Authenticate();
 	
 	/** Executes the given command as a new Job (in a separate Worker thread).
 	*/
@@ -170,12 +182,12 @@ public:
 	bool IsBusy()
 	{ 
 		PWaitAndSignal lock(m_cmutex);
-		return m_numExecutingCommands > 0; 
+		return m_numExecutingCommands > 0;
 	}
 
 	const PString& GetUser() const { return m_user; }
 	
-private:
+protected:
 	// override from class ServerSocket
 	virtual void Dispatch();
 
@@ -203,7 +215,7 @@ private:
 		@return
 		true if the use has been authenticated.
 	*/
-	bool AuthenticateUser();
+	virtual bool AuthenticateUser();
 
 	/** @return
 		The decrypted password associated with the specified login.
@@ -294,6 +306,318 @@ private:
         bool m_isFilteringActive;
 };
 
+#ifdef HAS_LIBSSH
+
+// SSH version of the staus client
+class SSHStatusClient : public StatusClient {
+
+public:
+	SSHStatusClient(int instanceNo);
+	virtual ~SSHStatusClient();
+
+#ifdef LARGE_FDSET
+	virtual bool Accept(YaTCPSocket &);
+#else
+	virtual PBoolean Accept(PSocket &);
+#endif
+
+	/** Check the client with all configured status authentication rules.
+		Ask for username/password, if required.
+		
+		@return
+		true if the client has been granted access, false to reject the client.
+	*/
+	virtual bool Authenticate();
+
+	/** Authenticate this status interface client through all authentication
+		rules and return the final result.
+		
+		@return
+		true if the use has been authenticated.
+	*/
+	virtual bool AuthenticateUser();
+
+	virtual bool ReadCommand(
+		/// command that has been read (if ReadCommand succeeded)
+		PString& cmd,
+		/// should the command be echoed (also NeedEcho() has to be true)
+		bool echo = true,
+		/// timeout (ms) for read operation
+		int readTimeout = 0
+		);
+
+	virtual bool WriteData(const BYTE * msg, int len);
+
+protected:
+	ssh_bind sshbind;
+	ssh_session session;
+    ssh_message message;
+    ssh_channel chan;
+};
+
+SSHStatusClient::SSHStatusClient(int instanceNo) : StatusClient(instanceNo)
+{
+	chan = 0;
+	m_needEcho = true;
+}
+
+SSHStatusClient::~SSHStatusClient()
+{
+    ssh_disconnect(session);
+	if (sshbind) {
+		ssh_bind_set_fd(sshbind, -1);		// make sure StatusListener is not closed
+		ssh_bind_free(sshbind);
+	}
+	ssh_finalize();
+}
+
+#ifdef LARGE_FDSET
+bool SSHStatusClient::Accept(YaTCPSocket & socket)
+#else
+PBoolean SSHStatusClient::Accept(PSocket & socket)
+#endif
+{
+	sshbind = ssh_bind_new();
+	session = ssh_new();
+
+	//ssh_bind_options_set(sshbind, SSH_BIND_OPTIONS_LOG_VERBOSITY_STR, "3");	// only enable for debugging
+	ssh_bind_options_set(sshbind, SSH_BIND_OPTIONS_BANNER, "SSH-2.0-GnuGk");	// TODO: doesn't seem to work
+
+	bool keyAvailable = false;
+	PString dsakey = GkConfig()->GetString(authsec, "DSAKey", "/etc/ssh/ssh_host_dsa_key");
+	if (PFile::Exists(dsakey) && PFile::Access(dsakey, PFile::ReadOnly)) {
+		keyAvailable = true;
+		ssh_bind_options_set(sshbind, SSH_BIND_OPTIONS_DSAKEY, (const char *)dsakey);
+		PTRACE(3, "Setting DSA key to " << dsakey);
+	}
+	PString rsakey = GkConfig()->GetString(authsec, "RSAKey", "/etc/ssh/ssh_host_rsa_key");
+	if (PFile::Exists(rsakey) && PFile::Access(rsakey, PFile::ReadOnly)) {
+		keyAvailable = true;
+		ssh_bind_options_set(sshbind, SSH_BIND_OPTIONS_RSAKEY, (const char *)rsakey);
+		PTRACE(3, "Setting RSA key to " << rsakey);
+	}
+	if (!keyAvailable) {
+		PTRACE(1, "No DSA or RSA key file found");
+		socket.Close();
+		return false;
+	}
+
+    if(ssh_init() < 0) {
+		PTRACE(1, "ssh_init() failed");
+		return false;
+    }
+	ssh_bind_set_fd(sshbind, socket.GetHandle());
+    if(ssh_bind_accept(sshbind, session) == SSH_ERROR) {
+		PTRACE(1, "ssh_bind_accept() failed: " << ssh_get_error(sshbind));
+		return false;
+    }
+
+    if(ssh_handle_key_exchange(session)) {
+		PTRACE(1, "ssh_handle_key_exchange failed: " << ssh_get_error(session));
+		return false;
+	}
+	
+	// set handle for SocketReader
+	os_handle = ssh_get_fd(session);
+
+	return true;
+}
+
+bool SSHStatusClient::Authenticate()
+{
+	// TODO: check the auth rules and allow any password if client is covered by explicit IP auth
+	const time_t now = time(NULL);
+	const int loginTimeout = GkConfig()->GetInteger(authsec, "LoginTimeout", 120);
+    bool auth = false;
+    int retries = 0;
+    do {
+        message = ssh_message_get(session);
+        if (!message) {
+			PTRACE(1, "ssh read error: " << ssh_get_error(session));
+            break;
+		}
+        switch(ssh_message_type(message)) {
+            case SSH_REQUEST_AUTH:
+                switch(ssh_message_subtype(message)) {
+                    case SSH_AUTH_METHOD_PASSWORD:
+						retries++;
+                        if (AuthenticateUser()) {
+							auth = true;
+							ssh_message_auth_reply_success(message,0);
+							break;
+						}
+						// fall through: not authenticated, send default message
+                    case SSH_AUTH_METHOD_NONE:
+                    default:
+                        ssh_message_auth_set_methods(message, SSH_AUTH_METHOD_PASSWORD);
+                        ssh_message_reply_default(message);
+                        break;
+                }
+                break;
+            default:
+                ssh_message_reply_default(message);
+        }
+        ssh_message_free(message);
+    } while (!auth && (retries < 3) && ((time(NULL) - now) < loginTimeout));
+
+    if (!auth) {
+        return false;
+    }
+
+	// open channel
+    do {
+        message = ssh_message_get(session);
+        if (message) {
+            switch(ssh_message_type(message)) {
+                case SSH_REQUEST_CHANNEL_OPEN:
+                    if (ssh_message_subtype(message) == SSH_CHANNEL_SESSION) {
+                        chan = ssh_message_channel_request_open_reply_accept(message);
+                        break;
+                    }
+	                // fall through intended
+                default:
+	                ssh_message_reply_default(message);
+            }
+            ssh_message_free(message);
+        }
+    } while(message && !chan);
+    if (!chan) {
+        PTRACE(1, "Error establishing SSH channel: " << ssh_get_error(session));
+        return false;
+    }
+
+	// handle requests
+	bool shell = false;
+    do {
+        message = ssh_message_get(session);
+        if (message) {
+            switch(ssh_message_type(message)) {
+                case SSH_REQUEST_CHANNEL:
+                    switch (ssh_message_subtype(message)) {
+						case SSH_CHANNEL_REQUEST_UNKNOWN:
+							// Putty X11 forwarding, deny to avoid hang
+			                ssh_message_reply_default(message);	// deny
+			                break;
+						case SSH_CHANNEL_REQUEST_PTY:
+		                    ssh_message_channel_request_reply_success(message);
+			                break;
+						case SSH_CHANNEL_REQUEST_SHELL:
+							shell = true;
+		                    ssh_message_channel_request_reply_success(message);
+			                break;
+						case SSH_CHANNEL_X11:
+			                ssh_message_reply_default(message);	// deny X11 forwarding
+			                break;
+						default:
+							PTRACE(3, "Unhandled SSH_REQUEST_CHANNEL message " << ssh_message_type(message) << " subtype " << ssh_message_subtype(message));
+							break;
+					}
+					break;
+				default:
+					PTRACE(3, "Unhandled SSH message " << ssh_message_type(message) << " subtype " << ssh_message_subtype(message));
+				// ignore the rest for now
+            }
+            ssh_message_free(message);
+        }
+    } while(message && !shell);
+	if (!message)
+		return false;
+
+    return true;
+}
+
+bool SSHStatusClient::AuthenticateUser()
+{
+	const int delay = GkConfig()->GetInteger(authsec, "DelayReject", 0);
+
+	PString login = ssh_message_auth_user(message);
+	PString password = ssh_message_auth_password(message);
+
+	// PTRACE(1, "User " << login << " wants to authenticate with password " << password);
+
+	const PString storedPassword = GetPassword(login);
+	if (storedPassword.IsEmpty())
+		PTRACE(5, "STATUS\tCould not find password in the config for user " << login);
+	else if (!password && password == storedPassword) {
+		m_user = login;
+		return true;
+	} else
+		PTRACE(5, "STATUS\tPassword mismatch for user " << login);
+
+	PProcess::Sleep(delay * 1000);
+
+	return false;
+}
+
+bool SSHStatusClient::ReadCommand(PString& cmd, bool echo, int readTimeout)
+{
+    char buf[2048];
+    int i = 0;
+    do {
+        i = ssh_channel_read(chan, buf, sizeof(buf), 0);
+		for (int c = 0; c < i; c++) {
+			char byte;
+			char ch = buf[c];
+			switch (ch)
+			{
+				case '\r':
+				case '\n':
+					cmd = m_currentCmd;
+					m_lastCmd = cmd;
+					m_currentCmd = PString();
+					if (echo && NeedEcho())
+						ssh_channel_write(chan, (void *)"\r\n", 2);
+					return true;
+				case '\b':
+				case 0x7f:	// backspace with ssh
+					if (m_currentCmd.GetLength()) {
+						m_currentCmd = m_currentCmd.Left(m_currentCmd.GetLength() - 1);
+						byte = char(ch);
+						if (echo && NeedEcho()) {
+							ssh_channel_write(chan, (void *)"\b \b", 3);
+						}
+					}
+					break;
+				case '\x04':	// Ctrl-D
+					cmd = "exit";
+					m_lastCmd = cmd;
+					m_currentCmd = PString::Empty();
+					return true;
+				default:
+					byte = char(ch);
+					m_currentCmd += byte;
+					cmd = m_currentCmd.Right(3);
+					// Note: this only works if the telnet client doesn't buffer characters
+					// Windows does, my Linux telnet doesn't
+					if (cmd == "\x1b\x5b\x41" || cmd == "\x5b\x41") { // Up Arrow
+						if (echo && NeedEcho()) {
+							for(PINDEX i = 0; i < m_currentCmd.GetLength() - 3; i ++)
+								Write("\b", 1);
+							WriteString(m_lastCmd);
+						}
+						m_currentCmd = m_lastCmd;
+					} else if (cmd.Find('\x1b') == P_MAX_INDEX)
+						if (echo && NeedEcho())
+							ssh_channel_write(chan, &buf[c], 1);
+					break;
+				}
+			}
+    } while (i > 0);
+
+	return true;
+}
+
+bool SSHStatusClient::WriteData(const BYTE * msg, int len)
+{
+	if (!chan)
+		return false;
+
+	int written = ssh_channel_write(chan, msg, len);
+	ssh_blocking_flush(session, 5000);	// wait 5 sec. max
+	return (written == len);
+}
+
+#endif // HAS_LIBSSH
 
 TelnetSocket::TelnetSocket() : m_needEcho(false), m_state(StateNormal)
 {
@@ -688,6 +1012,7 @@ void GkStatus::OnStart()
 	m_commands["fv"] = e_FindVerbose;
 	m_commands["disconnectip"] = e_DisconnectIp;
 	m_commands["disconnectcall"] = e_DisconnectCall;
+	m_commands["disconnectcallid"] = e_DisconnectCallId;
 	m_commands["disconnectalias"] = e_DisconnectAlias;
 	m_commands["disconnectendpoint"] = e_DisconnectEndpoint;
 	m_commands["disconnectsession"] = e_DisconnectSession;
@@ -696,6 +1021,7 @@ void GkStatus::OnStart()
 	m_commands["unregisterip"] = e_UnregisterIp;
 	m_commands["unregisteralias"] = e_UnregisterAlias;
 	m_commands["transfercall"] = e_TransferCall;
+	m_commands["reroutecall"] = e_RerouteCall;
 	m_commands["makecall"] = e_MakeCall;
 	m_commands["yell"] = e_Yell;
 	m_commands["who"] = e_Who;
@@ -739,6 +1065,7 @@ void GkStatus::OnStart()
 	m_commands["getacctinfo"] = e_GetAcctInfo;
 	m_commands["gci"] = e_GetAcctInfo;
 	m_commands["resetcallcounters"] = e_ResetCallCounters;
+	m_commands["printendpointqos"] = e_PrintEndpointQoS;
 }
 
 void GkStatus::ReadSocket(
@@ -747,8 +1074,9 @@ void GkStatus::ReadSocket(
 {
 	PString cmd;
 	StatusClient* client = static_cast<StatusClient*>(clientSocket);
-	if (client->ReadCommand(cmd) && !cmd)
+	if (client->ReadCommand(cmd) && !cmd.IsEmpty()) {
 		client->OnCommand(cmd);
+	}
 }
 
 void GkStatus::CleanUp()
@@ -783,9 +1111,9 @@ StatusClient::StatusClient(
 	m_traceLevel(MAX_STATUS_TRACE_LEVEL),
 	m_isFilteringActive(false)
 {
-        PStringToString filters = GkConfig()->GetAllKeyValues(filteringsec);
-        SetWriteTimeout(10);
-	
+	PStringToString filters = GkConfig()->GetAllKeyValues(filteringsec);
+	SetWriteTimeout(10);
+
 	for (PINDEX i = 0; i < filters.GetSize(); i++) {
 	    PString key = filters.GetKeyAt(i);
 	    PString data = filters.GetDataAt(i);
@@ -819,7 +1147,7 @@ bool StatusClient::ReadCommand(
 			case '\n':
 				cmd = m_currentCmd;
 				m_lastCmd = cmd;
-				m_currentCmd = PString();
+				m_currentCmd = PString::Empty();
 				if (echo && NeedEcho())
 					TransmitData("\r\n");
 				return true;
@@ -837,7 +1165,7 @@ bool StatusClient::ReadCommand(
 			case '\x04':	// Ctrl-D
 				cmd = "exit";
 				m_lastCmd = cmd;
-				m_currentCmd = PString();
+				m_currentCmd = PString::Empty();
 				return true;
 			default:
 				byte = char(ch);
@@ -878,7 +1206,7 @@ bool StatusClient::WriteString(
 	
 	if (!WriteData(msg, msg.GetLength()))
 	    while (CanFlush())
-		Flush();
+			Flush();
 
 	return IsOpen();
 }
@@ -918,7 +1246,7 @@ bool StatusClient::Authenticate()
 			break;
 		} else
 			result = CheckAuthRule(rules(rule_start, rule_end - 1).Trim());
-		logical_or = (rules[rule_end] == '|') ;
+		logical_or = (rules[rule_end] == '|');
 		if ((logical_or && result) || !(logical_or || result))
 			break;
 		rule_start = rule_end + 1;
@@ -959,7 +1287,7 @@ void StatusClient::DoDebug(
 	const PStringArray& args
 	)
 {
-        bool tmp = m_isFilteringActive;
+	bool tmp = m_isFilteringActive;
 	m_isFilteringActive = false;
 
 	if (args.GetSize() <= 1) {
@@ -999,14 +1327,14 @@ void StatusClient::DoDebug(
 					result += "[" + secs[i] + "]\r\n";
 				WriteString(result + ";\r\n");
 			}
-		} else if ((args[1] *= "set") && (args.GetSize()>=5)) {
+		} else if ((args[1] *= "set") && (args.GetSize() >= 5)) {
 			Toolkit::Instance()->SetConfig(1, args[2], args[3], args[4]);
 			WriteString(GkConfig()->GetString(args[2],args[3],"") + "\r\n");
 		} else if (args[1] *= "remove") {
-			if (args.GetSize()>=4) {
+			if (args.GetSize() >= 4) {
 				Toolkit::Instance()->SetConfig(2, args[2], args[3]);
 				WriteString("Remove " + args[3] + " in section " + args[2] + "\r\n");
-			} else if (args.GetSize()>=3) {
+			} else if (args.GetSize() >= 3) {
 				Toolkit::Instance()->SetConfig(3, args[2]);
 				WriteString("Remove section " + args[2] + "\r\n");
 			}
@@ -1102,7 +1430,7 @@ bool StatusClient::AuthenticateUser()
 		} else
 			PTRACE(5, "STATUS\tPassword mismatch for user " << login);
 			
-		PProcess::Sleep(delay * 1000);
+		PThread::Sleep(delay * 1000);
 
 		if ((time(NULL) - now) > loginTimeout)
 			break;
@@ -1117,7 +1445,7 @@ PString StatusClient::GetPassword(
 	) const
 {
 	return !login
-		? Toolkit::Instance()->ReadPassword(authsec, login, true) : PString();
+		? Toolkit::Instance()->ReadPassword(authsec, login, true) : PString::Empty();
 }
 
 void StatusClient::CommandError(const PString & msg)
@@ -1126,6 +1454,11 @@ void StatusClient::CommandError(const PString & msg)
 	PTRACE(2, "STATUS\t" + msg + " from client " << Name());
 }
 
+// ignore warning when comparing to define
+#if (!_WIN32) && (GCC_VERSION >= 40400)
+#pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+
 void StatusClient::ExecCommand(
 	/// the command to be executed
 	PString cmd
@@ -1160,6 +1493,13 @@ void StatusClient::ExecCommand(
 		else
 			CommandError("Syntax Error: DisconnectCall CALL_NUMBER [CALL_NUMBER...]");
 		break;
+	case GkStatus::e_DisconnectCallId:
+		// disconnect call with this call ID
+		if (args.GetSize() == 2)
+			SoftPBX::DisconnectCallId(args[1]);
+		else
+			CommandError("Syntax Error: DisconnectCallId CALL_ID");
+		break;
 	case GkStatus::e_DisconnectEndpoint:
 		// disconnect call on this alias
 		if (args.GetSize() == 2)
@@ -1189,7 +1529,7 @@ void StatusClient::ExecCommand(
 		SoftPBX::PrintAllRegistrations(this, TRUE);
 		break;
 	case GkStatus::e_PrintAllCached:
-		// print list of all cached outer-zone endpoints
+		// print list of all cached out-of-zone endpoints
 		SoftPBX::PrintAllCached(this, (args.GetSize() > 1));
 		break;
 	case GkStatus::e_PrintCurrentCalls:
@@ -1261,8 +1601,18 @@ void StatusClient::ExecCommand(
 	case GkStatus::e_TransferCall:
 		if (args.GetSize() == 3)
 			SoftPBX::TransferCall(args[1], args[2]);
+		else if (args.GetSize() == 4)
+			SoftPBX::TransferCall(args[1], args[2], args[3], "FacilityForward");
+		else if (args.GetSize() == 5)
+			SoftPBX::TransferCall(args[1], args[2], args[3], args[4]);
 		else
-			CommandError("Syntax Error: TransferCall SOURCE DESTINATION");
+			CommandError("Syntax Error: TransferCall SOURCE DESTINATION or TransferCall CALLID CALLER|CALLED DESTINATION [TRANSFER-METHOD]");
+		break;
+	case GkStatus::e_RerouteCall:
+		if (args.GetSize() == 4)
+			SoftPBX::RerouteCall(args[1], args[2], args[3]);
+		else
+			CommandError("Syntax Error: RerouteCall CALLID CALLER|CALLED DESTINATION");
 		break;
 	case GkStatus::e_MakeCall:
 		if (args.GetSize() == 3)
@@ -1328,35 +1678,49 @@ void StatusClient::ExecCommand(
 		if (args.GetSize() == 4) {
 			if (args[1] == "-")
 				args[1] = "";	// "-" is empty agent
-			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], "", args[2], args[3].AsUnsigned(), "");
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], "", args[2], args[3].AsUnsigned(), "", "", "");
 		} else if (args.GetSize() == 5) {
 			if (args[1] == "-")
 				args[1] = "";	// "-" is empty agent
 			args[4].Replace("-", " ", true);
 			args[4] = args[4].Trim();
-			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], "", args[2], args[3].AsUnsigned(), args[4]);
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], "", args[2], args[3].AsUnsigned(), args[4], "", "");
+		} else if (args.GetSize() == 6) {
+			if (args[1] == "-")
+				args[1] = "";	// "-" is empty agent
+			args[4].Replace("-", " ", true);
+			args[4] = args[4].Trim();
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], "", args[2], args[3].AsUnsigned(), args[4], "", args[5]);
 		} else
-			CommandError("Syntax Error: RouteToAlias TARGET_ALIAS CALLING_ENDPOINT_ID CRV [CALLID]");
+			CommandError("Syntax Error: RouteToAlias TARGET_ALIAS CALLING_ENDPOINT_ID CRV [CALLID [CALLER-ID]]");
 		break;
 	case GkStatus::e_RouteToGateway:
 		if (args.GetSize() == 5) {
-			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], args[2], args[3], args[4].AsUnsigned(), "");
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], args[2], args[3], args[4].AsUnsigned(), "", "", "");
 		} else if (args.GetSize() == 6) {
 			args[5].Replace("-", " ", true);
 			args[5] = args[5].Trim();
-			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], args[2], args[3], args[4].AsUnsigned(), args[5]);
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], args[2], args[3], args[4].AsUnsigned(), args[5], "", "");
+		} else if (args.GetSize() == 7) {
+			args[5].Replace("-", " ", true);
+			args[5] = args[5].Trim();
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[1], args[2], args[3], args[4].AsUnsigned(), args[5], "", args[6]);
 		} else
-			CommandError("Syntax Error: RouteToGateway TARGET_ALIAS TARGET_IP CALLING_ENDPOINT_ID CRV [CALLID]");
+			CommandError("Syntax Error: RouteToGateway TARGET_ALIAS TARGET_IP CALLING_ENDPOINT_ID CRV [CALLID [CALLER-ID]]");
 		break;
 	case GkStatus::e_BindAndRouteToGateway:
 		if (args.GetSize() == 6) {
-			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[2], args[3], args[4], args[5].AsUnsigned(), "", args[1]);
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[2], args[3], args[4], args[5].AsUnsigned(), "", args[1], "");
 		} else if (args.GetSize() == 7) {
 			args[6].Replace("-", " ", true);
 			args[6] = args[6].Trim();
-			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[2], args[3], args[4], args[5].AsUnsigned(), args[6], args[1]);
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[2], args[3], args[4], args[5].AsUnsigned(), args[6], args[1], "");
+		} else if (args.GetSize() == 8) {
+			args[6].Replace("-", " ", true);
+			args[6] = args[6].Trim();
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[2], args[3], args[4], args[5].AsUnsigned(), args[6], args[1], args[7]);
 		} else
-			CommandError("Syntax Error: BindAndRouteToGateway BIND_IP TARGET_ALIAS TARGET_IP CALLING_ENDPOINT_ID CRV [CALLID]");
+			CommandError("Syntax Error: BindAndRouteToGateway BIND_IP TARGET_ALIAS TARGET_IP CALLING_ENDPOINT_ID CRV [CALLID [CALLER-ID]]");
 		break;
 	case GkStatus::e_RouteReject:
 		if (args.GetSize() == 3) {
@@ -1376,7 +1740,7 @@ void StatusClient::ExecCommand(
 				m_traceLevel = MAX_STATUS_TRACE_LEVEL;
 			else {
 				unsigned level = args[1].AsUnsigned();
-				if (level >= MIN_STATUS_TRACE_LEVEL 
+				if (level >= MIN_STATUS_TRACE_LEVEL
 					&& level <= MAX_STATUS_TRACE_LEVEL)
 					m_traceLevel = level;
 				else {
@@ -1476,6 +1840,10 @@ void StatusClient::ExecCommand(
 		else
 			CommandError("Syntax Error: GetAcctInfo|gci ACCT_MODULE_NAME");
 		break;
+	case GkStatus::e_PrintEndpointQoS:
+		// print QoS values for all endpoints
+		SoftPBX::PrintEndpointQoS(this);
+		break;
 	default:
 		// commmand not recognized
 		CommandError("Error: Unknown command '" + cmd + "'");
@@ -1589,5 +1957,14 @@ StatusListener::StatusListener(
 ServerSocket *StatusListener::CreateAcceptor() const
 {
 	static int StaticInstanceNo = 0;
-	return new StatusClient(++StaticInstanceNo);
+#ifdef HAS_LIBSSH
+	if (Toolkit::AsBool(GkConfig()->GetString("SSHStatusPort", 0))) {
+		PTRACE(4, "STATUS\tUsing SSH for status port");
+		return new SSHStatusClient(++StaticInstanceNo);
+	} else {
+#endif
+		return new StatusClient(++StaticInstanceNo);
+#ifdef HAS_LIBSSH
+	}
+#endif
 }
