@@ -2,12 +2,12 @@
 //
 // MakeCall.h
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2007-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Jan Willamowius
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -19,7 +19,6 @@
 #include "singleton.h"
 #include "config.h"
 #include <map>
-using namespace std;
 
 class MakeCallEndPoint : public Singleton<MakeCallEndPoint>, public H323EndPoint
 {
@@ -43,9 +42,9 @@ protected:
     PString GetDestination(PString token);
 
     PMutex destinationMutex;
-    std::map<PString, PString> destinations; 
+    std::map<PString, PString> destinations;
 
-	PBoolean useH450Transfer;
+	PCaselessString transferMethod;
 	PBoolean isRegistered;
 };
 
