@@ -3,14 +3,12 @@
 // Neighboring System for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2002-2003
+// Copyright (c) 2004-2010, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
-// initial version: 05/30/2003
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -232,8 +230,10 @@ H225_LocationRequest & Neighbor::BuildLRQ(H225_RasMessage & lrq_ras, WORD seqnum
 	lrq.m_requestSeqNum = seqnum;
 	lrq.m_destinationInfo = dest;
 
-	// Perform outbound per GK rewrite on the destination of the LRQ
-	Toolkit::Instance()->GWRewriteE164(m_id,false,lrq.m_destinationInfo[0]);
+	// perform outbound per GK rewrite on the destination of the LRQ
+	Toolkit::Instance()->GWRewriteE164(m_id, GW_REWRITE_OUT, lrq.m_destinationInfo[0]);
+	if (m_gkid != m_id)
+		Toolkit::Instance()->GWRewriteE164(m_gkid, GW_REWRITE_OUT, lrq.m_destinationInfo[0]);
 
 	lrq.m_replyAddress = m_rasSrv->GetRasAddress(GetIP());
 
@@ -575,7 +575,7 @@ bool OldGK::SetProfile(const PString & id, const PString & args)
 	if (Toolkit::AsBool(GkConfig()->GetString(LRQFeaturesSection, "AlwaysForwardLRQ", "0")))
 		m_forwardto = 1;
 
-	PTRACE(1, "Set neighbor " << m_gkid << '(' << (m_dynamic ? m_name : AsString(m_ip, m_port)) << ')' << (cfg.GetSize() > 1 ? (" for prefix " + cfg[1]) : PString()));
+	PTRACE(1, "Set neighbor " << m_gkid << '(' << (m_dynamic ? m_name : AsString(m_ip, m_port)) << ')' << (cfg.GetSize() > 1 ? (" for prefix " + cfg[1]) : PString::Empty()));
 	return true;
 }
 
@@ -1125,13 +1125,12 @@ bool NeighborList::CheckIP(const PIPSocket::Address & addr) const
 
 PString NeighborList::GetNeighborIdBySigAdr(const H225_TransportAddress & sigAd)
 {
-
 	PIPSocket::Address ipaddr;
 
 	// Get the Neigbor IP address from the transport address
 	if (!GetIPFromTransportAddr(sigAd, ipaddr))
 	{
-		return PString("");
+		return PString::Empty();
 	}
 
 	return GetNeighborIdBySigAdr(ipaddr);
@@ -1139,21 +1138,42 @@ PString NeighborList::GetNeighborIdBySigAdr(const H225_TransportAddress & sigAd)
 
 PString NeighborList::GetNeighborIdBySigAdr(const PIPSocket::Address & sigAd)
 {
-
-	List::iterator findNeighbor;
-
 	// Attempt to find the neigbor in the list
-	findNeighbor = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsFrom), &sigAd));
+	List::iterator findNeighbor = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsFrom), &sigAd));
 
 	if (findNeighbor == m_neighbors.end())
 	{
-		return PString("");
+		return PString::Empty();
 	}
 
 	return (*findNeighbor)->GetId();
-
 }
 
+PString NeighborList::GetNeighborGkIdBySigAdr(const PIPSocket::Address & sigAd)
+{
+	// Attempt to find the neigbor in the list
+	List::iterator findNeighbor = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsFrom), &sigAd));
+
+	if (findNeighbor == m_neighbors.end())
+	{
+		return PString::Empty();
+	}
+
+	return (*findNeighbor)->GetGkId();
+}
+
+PString NeighborList::GetNeighborGkIdBySigAdr(const H225_TransportAddress & sigAd)
+{
+	PIPSocket::Address ipaddr;
+
+	// Get the Neigbor IP address from the transport address
+	if (!GetIPFromTransportAddr(sigAd, ipaddr))
+	{
+		return PString::Empty();
+	}
+
+	return GetNeighborGkIdBySigAdr(ipaddr);
+}
 
 /* Not used currently
 H225_CryptoH323Token BuildAccessToken(const H225_TransportAddress & dest, const PIPSocket::Address & addr)
@@ -1492,8 +1512,7 @@ Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress
 					endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
 					if (ep) {
 						// endpoint found locally
-						Route * route = new Route(ep);
-						route->m_policy = m_name;
+						Route * route = new Route("srv", ep);
 						return route;
 					} else {
 						return NULL;
@@ -1590,8 +1609,7 @@ Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
 					endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
 					if (ep) {
 						// endpoint found locally
-						Route * route = new Route(ep);
-						route->m_policy = m_name;
+						Route * route = new Route("srv", ep);
 						return route;
 					}
 				} else {
@@ -1689,7 +1707,7 @@ bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 		delete route;
 		return true;
 	}
-	return false;  
+	return false;
 }
 
 // usd for LRQs
@@ -1715,7 +1733,7 @@ bool SRVPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 		delete route;
 		return true;
 	}
-	return false;  
+	return false;
 }
 #endif
 
@@ -1806,7 +1824,7 @@ bool RDSPolicy::FindByAliases(
 bool RDSPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_AliasAddress & /* aliases */)
 { 
     PTRACE(4, "ROUTING\tPolicy RDS not supported for LRQ");
-	return false;  
+	return false;
 }
 #endif
 
