@@ -1,21 +1,19 @@
 //////////////////////////////////////////////////////////////////
 //
-// New Neighboring System for GNU Gatekeeper
+// Neighboring System for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2002-2003
+// Copyright (c) 2004-2010, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
-// initial version: 05/30/2003
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 #ifndef NEIGHBOR_H
-#define NEIGHBOR_H "@(#) $Id: Neighbor.h,v 1.24 2010/03/05 16:55:15 willamowius Exp $"
+#define NEIGHBOR_H "@(#) $Id: Neighbor.h,v 1.26 2011/06/15 20:24:50 willamowius Exp $"
 
 #include <list>
 #include <map>
@@ -73,6 +71,7 @@ public:
 	int ForwardLRQ() const { return m_forwardto; }
 	WORD GetDefaultHopCount() const { return m_forwardHopCount; }
 	PString GetId() const { return m_id; }
+	PString GetGkId() const { return m_gkid; }
 	PIPSocket::Address GetIP() const;
 	H225_LocationRequest & BuildLRQ(H225_RasMessage &, WORD, const H225_ArrayOf_AliasAddress &);
 
@@ -140,10 +139,14 @@ public:
 	bool CheckLRQ(RasMsg *) const;
 	bool CheckIP(const PIPSocket::Address &) const;
 
-	// Return the neighbors Id from the list from the signal address.
+	// return the neighbor's ID from the list by signal address
 	PString GetNeighborIdBySigAdr(const H225_TransportAddress & sigAd);
 	PString GetNeighborIdBySigAdr(const PIPSocket::Address & sigAd);
 
+	// return the neighbor's gatekeeper ID from the list by signal address
+	PString GetNeighborGkIdBySigAdr(const H225_TransportAddress & sigAd);
+	PString GetNeighborGkIdBySigAdr(const PIPSocket::Address & sigAd);
+
 	operator List & () { return m_neighbors; }
 	operator const List & () const { return m_neighbors; }
 
