@@ -5,19 +5,17 @@
 // New Multi-threaded RAS Server for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2001-2003
+// Copyright (c) 2000-2010, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
-// initial version: 03/14/2003
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASSRV_H
-#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.55 2010/05/03 12:37:50 shorne Exp $"
+#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.60 2011/03/30 10:47:36 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -49,7 +47,6 @@ class GkAcctLoggerList;
 class GkClient;
 class ProxyHandler;
 class HandlerList;
-class WaitingARQlist;
 
 namespace Neighbors {
 	class NeighborList;
@@ -112,10 +109,12 @@ public:
 	bool SendRas(const H225_RasMessage &, const Address &, WORD, RasListener * = 0);
 	bool SendRas(const H225_RasMessage &, const H225_TransportAddress &, RasListener * = 0);
 	bool SendRas(const H225_RasMessage &, const Address &, WORD, const Address &);
+	bool SendRIP(H225_RequestSeqNum seqNum, unsigned ripDelay, const Address & addr, WORD port);
 
 	bool IsRedirected(unsigned = 0) const;
 	bool IsForwardedMessage(const H225_NonStandardParameter *, const Address &) const;
 	void ForwardRasMsg(H225_RasMessage &);
+	bool ReplyToRasAddress(const NetworkAddress & ip) const;
 
 	bool RemoveCallOnDRQ() const { return bRemoveCallOnDRQ; }
 
@@ -189,27 +188,39 @@ public:
 		return IsForwardedMessage(ras.HasOptionalField(RAS::e_nonStandardData) ? &ras.m_nonStandardData : 0, addr);
 	}
 
-	template<class RAS> void SetAlternateGK(RAS & ras)
+	template<class RAS> void SetAlternateGK(RAS & ras, const NetworkAddress & ip)
 	{
-		if (altGKs->GetSize() > 0) {
+		H225_ArrayOf_AlternateGK alternates = GetAltGKForIP(ip);
+		if (alternates.GetSize() > 0) {
+			// add alternates by IP
 			ras.IncludeOptionalField(RAS::e_alternateGatekeeper);
-			ras.m_alternateGatekeeper = *altGKs;
+			ras.m_alternateGatekeeper = alternates;
+		} else	if (altGKs.GetSize() > 0) {
+			// use global configuration
+			ras.IncludeOptionalField(RAS::e_alternateGatekeeper);
+			ras.m_alternateGatekeeper = altGKs;
 		}
 	}
 
-	template<class RAS> void SetAltGKInfo(RAS & ras)
+	template<class RAS> void SetAltGKInfo(RAS & ras, const NetworkAddress & ip)
 	{
-		if (altGKs->GetSize() > 0) {
+		H225_ArrayOf_AlternateGK alternates = GetAltGKForIP(ip);
+		if (alternates.GetSize() > 0) {
+			// add alternates by IP
 			ras.IncludeOptionalField(RAS::e_altGKInfo);
 			ras.m_altGKInfo.m_altGKisPermanent = (redirectGK == e_permanentRedirect);
-			ras.m_altGKInfo.m_alternateGatekeeper = *altGKs;
+			ras.m_altGKInfo.m_alternateGatekeeper = alternates;
+		} else	if (altGKs.GetSize() > 0) {
+			// use global configuration
+			ras.IncludeOptionalField(RAS::e_altGKInfo);
+			ras.m_altGKInfo.m_altGKisPermanent = (redirectGK == e_permanentRedirect);
+			ras.m_altGKInfo.m_alternateGatekeeper = altGKs;
 		}
 	}
 
 #ifdef h323v6
 	template<class RAS> bool HasAssignedGK(const PString & alias,const PIPSocket::Address & ip, RAS & ras)
 	{
-
         H225_ArrayOf_AlternateGK assignedGK;
 		assignedGK.SetSize(0);
 
@@ -246,6 +257,8 @@ private:
 	virtual void OnStop();
 
 	void GetAlternateGK();
+	H225_ArrayOf_AlternateGK ParseAltGKConfig(const PString & altGkSetting) const;
+	H225_ArrayOf_AlternateGK GetAltGKForIP(const NetworkAddress & ip) const;
 	void ClearAltGKsTable();
 	void HouseKeeping();
 
@@ -289,12 +302,13 @@ private:
 	};
 	std::vector<Address> altGKsAddr, skipAddr;
 	std::vector<WORD> altGKsPort;
-	H225_ArrayOf_AlternateGK *altGKs;
+	H225_ArrayOf_AlternateGK altGKs;
 	PINDEX altGKsSize;
+	std::map<NetworkAddress, H225_ArrayOf_AlternateGK> m_altGkRules;	// alternate GK rules by IP
 	PINDEX epLimit, callLimit;
 	int redirectGK;
 
-	WaitingARQlist *wArqList;
+	std::map<NetworkAddress, bool> m_replyras; // on which network should we use the rasAddress included in GRQ/RRQ/IRQ
 };
 
 #endif // RASSRV_H
