@@ -2,15 +2,17 @@
 //
 // bookkeeping for RAS-Server in H.323 gatekeeper
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2011, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.174.2.1 2010/05/26 13:02:44 willamowius Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.199 2011/07/02 11:51:38 shorne Exp $"
 
 #include <list>
 #include <map>
@@ -24,7 +26,7 @@
 
 
 namespace Routing {
-	struct Route;
+	class Route;
 }
 
 class USocket;
@@ -32,6 +34,9 @@ class CallSignalSocket;
 class RasServer;
 class Q931;
 
+enum CallLeg { Caller, Called };
+enum RerouteState { NoReroute, RerouteInitiated, Rerouting };
+
 // Template of smart pointer
 // The class T must have Lock() & Unlock() methods
 template<class T> class SmartPtr {
@@ -58,14 +63,12 @@ private:
 	T *pt;
 };
 
-#ifdef HAS_H460P
-class H323Contact;
-#endif
+
 class EndpointRec
 {
 public:
-	/** Construct internal/outer zone endpoint from the specified RAS message.
-		RRQ builds an internal zone endpoint, ARQ, ACF and LCF build outer zone
+	/** Construct internal/out-of-zone endpoint from the specified RAS message.
+		RRQ builds an internal zone endpoint, ARQ, ACF and LCF build out-of-zone
 		endpoints.
 	*/
 	EndpointRec(
@@ -101,6 +104,9 @@ public:
 	virtual void AddNumbers(const PString & numbers);
 	virtual void SetEndpointType(const H225_EndpointType &);
     virtual void SetEndpointInfo(const PString & vendor, const PString & version);
+	virtual long GetBandwidth() const { return m_bandwidth; }
+	virtual void SetBandwidth(long bw) { m_bandwidth = bw;  if (m_bandwidth < 0) m_bandwidth = 0; }
+	virtual int GetMaxBandwidth() const { return m_maxBandwidth; }
 
 	virtual void Update(const H225_RasMessage & lightweightRRQ);
 	virtual bool IsGateway() const { return false; }
@@ -257,6 +263,8 @@ public:
 	// cause code translation
 	unsigned TranslateReceivedCause(unsigned cause) const;
 	unsigned TranslateSentCause(unsigned cause) const;
+	// adding a fixed alias to all calls _to_ this endpoint
+	PString GetAdditionalDestinationAlias() const { return m_additionalDestAlias; }
 
 	bool IsH46018Disabled() const { return m_h46018disabled; }
 	void SetUsesH46018(bool uses) { m_usesH46018 = uses; m_nat = uses; }
@@ -316,7 +324,7 @@ protected:
 	/// session number for call credit service control session
 	int m_callCreditSession;
 	/// endpoint call capacity, -1 means no limit
-	int m_capacity;
+	long m_capacity;
 	/// capacity per prefix (regex)
 	list<pair<string, int> > m_prefixCapacities;
 	int m_calledTypeOfNumber, m_callingTypeOfNumber;
@@ -333,13 +341,17 @@ protected:
 	/// cause code translation
 	std::map<unsigned, unsigned> m_receivedCauseMap;
 	std::map<unsigned, unsigned> m_sentCauseMap;
+	/// additional alias to be added to all calls TO this endpoint
+	PString m_additionalDestAlias;
 
 	EPNatTypes m_epnattype;
 	bool m_usesH46023, m_H46024, m_H46024a, m_H46024b, m_natproxy, m_internal, m_remote;
 	bool m_h46018disabled;
 	bool m_usesH46018;
 	bool m_usesH460P;
-
+	
+	long m_bandwidth;	// bandwidth currently occupied by this endpoint
+	long m_maxBandwidth; // maximum bandwidth allowed for this endpoint
 };
 
 typedef EndpointRec::Ptr endptr;
@@ -428,23 +440,46 @@ protected:
 };
 
 
-class OuterZoneEPRec : public EndpointRec {
+class OutOfZoneEPRec : public EndpointRec {
 public:
-	OuterZoneEPRec(const H225_RasMessage & completeRAS, const H225_EndpointIdentifier &);
+	OutOfZoneEPRec(const H225_RasMessage & completeRAS, const H225_EndpointIdentifier &);
 
 	virtual EndpointRec *Unregister() { return this; }
 	virtual EndpointRec *Expired() { return this; }
 };
 
 
-class OuterZoneGWRec : public GatewayRec {
+class OutOfZoneGWRec : public GatewayRec {
 public:
-	OuterZoneGWRec(const H225_RasMessage & completeRAS, const H225_EndpointIdentifier &);
+	OutOfZoneGWRec(const H225_RasMessage & completeRAS, const H225_EndpointIdentifier &);
 
 	virtual EndpointRec *Unregister() { return this; }
 	virtual EndpointRec *Expired() { return this; }
 };
 
+class EPQoS {
+public:
+	EPQoS() { Init(); }
+	EPQoS(PTime lastMsg) { Init(); m_lastMsg = lastMsg; }
+	~EPQoS() { }
+
+	void Init();
+	PString AsString() const;
+
+	void IncrementCalls() { m_numCalls++; }
+	void SetAudioPacketLossPercent(float val) { m_audioPacketLossPercent = val; }
+	void SetAudioJitter(unsigned val) { m_audioJitter = val; }
+	void SetVideoPacketLossPercent(float val) { m_videoPacketLossPercent = val; }
+	void SetVideoJitter(unsigned val) { m_videoJitter = val; }
+
+protected:
+	PTime m_lastMsg;
+	unsigned m_numCalls;
+	float m_audioPacketLossPercent;
+	unsigned long m_audioJitter;
+	float m_videoPacketLossPercent;
+	unsigned long m_videoJitter;
+};
 
 class RegistrationTable : public Singleton<RegistrationTable> {
 public:
@@ -466,7 +501,7 @@ public:
 	bool FindEndpoint(
 		const H225_ArrayOf_AliasAddress &aliases,
 		bool roundRobin,
-		bool searchOuterZone,
+		bool searchOutOfZone,
 		std::list<Routing::Route> &routes
 		);
 
@@ -477,6 +512,7 @@ public:
 	void PrintAllCached(USocket *client, bool verbose=FALSE);
 	void PrintRemoved(USocket *client, bool verbose=FALSE);
 	void PrintPrefixCapacities(USocket *client, PString alias) const;
+	void PrintEndpointQoS(USocket *client); //const;
 
 	PString PrintStatistics() const;
 
@@ -516,7 +552,7 @@ private:
 	void GenerateAlias(H225_ArrayOf_AliasAddress &, const H225_EndpointIdentifier &) const;
 
 	std::list<EndpointRec *> EndpointList;
-	std::list<EndpointRec *> OuterZoneList;
+	std::list<EndpointRec *> OutOfZoneList;
 	std::list<EndpointRec *> RemovedList;
 	int regSize;
 	mutable PReadWriteMutex listLock;
@@ -542,6 +578,15 @@ template<class> class RasPDU;
  class H323TransportAddress;
 #endif
 #endif
+
+#ifdef HAS_H46018
+// direction definitions for H.460.19
+#define H46019_NONE		0	// m_h46019dir = 0 ' No party needs H.460.19, so skip the code
+#define H46019_CALLER	1	// m_h46019dir = 1 ' Caller needs H.460.19
+#define H46019_CALLED	2	// m_h46019dir = 2 ' Called needs H.460.19
+#define H46019_BOTH		(H46019_CALLER + H46019_CALLED)	// m_h46019dir = 3 ' Both need it
+#endif
+
 class CallRec {
 public:
 	/// flag to overwrite proxy settings for the call
@@ -571,7 +616,7 @@ public:
 		/// ARQ with call information
 		const RasPDU<H225_AdmissionRequest>& arq,
 		/// bandwidth occupied by the call
-		int bandwidth,
+		long bandwidth,
 		/// called party's aliases in a string form
 		const PString& destInfo,
 		/// override proxy mode global setting from the config
@@ -612,7 +657,7 @@ public:
 	endptr GetCallingParty() const { return m_Calling; }
 	endptr GetCalledParty() const { return m_Called; }
 	endptr GetForwarder() const { return m_Forwarder; }
-	int GetBandwidth() const { return m_bandwidth; }
+	long GetBandwidth() const { return m_bandwidth; }
 
 	/** @return
 	    A bit mask with NAT flags for calling and called parties. 
@@ -661,9 +706,12 @@ public:
 
 	/** Can Signalling be offloaded
 	  */
-	bool NATSignallingOffload(bool isAnswer);
+	bool NATSignallingOffload(bool isAnswer) const;
 
 #ifdef HAS_H46024B
+    /** GetSignallingSocket */
+    CallSignalSocket * H46024BSignalSocket(bool response);
+
 	/** Initiate Probe */
 	void H46024BInitiate(WORD sessionID, const H323TransportAddress & fwd, const H323TransportAddress & rev);
 
@@ -705,7 +753,9 @@ public:
 	void SetCalling(const endptr & NewCalling);
 	void SetCalled(const endptr & NewCalled);
 	void SetForward(CallSignalSocket *, const H225_TransportAddress &, const endptr &, const PString &, const PString &);
-	void SetBandwidth(int bandwidth) { m_bandwidth = bandwidth; }
+	void RerouteDropCalling();
+	void RerouteDropCalled();
+	void SetBandwidth(long bandwidth) { m_bandwidth = bandwidth; if (m_bandwidth < 0) m_bandwidth = 0; }
 	void SetSocket(CallSignalSocket *, CallSignalSocket *);
 	void SetCallSignalSocketCalling(CallSignalSocket* socket);
 	void SetCallSignalSocketCalled(CallSignalSocket* socket);
@@ -747,7 +797,7 @@ public:
 
 	PString GenerateCDR(
 		/// timestamp formatting string (empty for a default RFC822 format)
-		const PString& timestampFormat = PString()
+		const PString & timestampFormat = PString::Empty()
 		) const;
 	PString PrintOn(bool verbose) const;
 
@@ -800,61 +850,73 @@ public:
 	*/
 
 	PString GetSRC_media_control_IP() const;
-        PString GetDST_media_control_IP() const;
-    
-        void SetSRC_media_control_IP(PString IP);
-        void SetDST_media_control_IP(PString IP);
-    
-        PString GetSRC_media_IP() const; 
-        PString GetDST_media_IP() const; 
-                                             
-        void SetSRC_media_IP(PString IP);
-        void SetDST_media_IP(PString IP);
-    
-        void SetRTCP_SRC_sdes(PString val);
-        void SetRTCP_DST_sdes(PString val);
-    
-        PStringList GetRTCP_SRC_sdes() const;
-        PStringList GetRTCP_DST_sdes() const;
-    
-        bool GetRTCP_SRC_sdes_flag() const;
-        bool GetRTCP_DST_sdes_flag() const;
-	
-        void InitRTCP_report();
-    
-        void SetRTCP_SRC_packet_count(long val);
-        void SetRTCP_DST_packet_count(long val);
-    
-        void SetRTCP_SRC_packet_lost(long val);
-        void SetRTCP_DST_packet_lost(long val);
-    
-        void SetRTCP_SRC_jitter(int val);
-    
-        void SetRTCP_DST_jitter(int val);
+	PString GetDST_media_control_IP() const;
+
+	void SetSRC_media_control_IP(const PString & IP);
+	void SetDST_media_control_IP(const PString & IP);
+
+	PString GetSRC_media_IP() const;
+	PString GetDST_media_IP() const;
+
+	void SetSRC_media_IP(const PString & IP);
+	void SetDST_media_IP(const PString & IP);
+
+	void SetRTCP_sdes(bool isSRC, const PString & val);
+	void SetRTCP_SRC_sdes(const PString & val);
+	void SetRTCP_DST_sdes(const PString & val);
+
+	PStringList GetRTCP_SRC_sdes() const;
+	PStringList GetRTCP_DST_sdes() const;
+
+	bool GetRTCP_SRC_sdes_flag() const;
+	bool GetRTCP_DST_sdes_flag() const;
+
+	void InitRTCP_report();
+
+	// set audio RTCP stats
+	void SetRTCP_SRC_packet_count(long val);
+	void SetRTCP_SRC_packet_lost(long val);
+	void SetRTCP_SRC_jitter(int val);
+	void SetRTCP_DST_packet_count(long val);
+	void SetRTCP_DST_packet_lost(long val);
+	void SetRTCP_DST_jitter(int val);
+
+	// get audio RTCP stats
+	long GetRTCP_SRC_packet_count() const { return m_rtcp_source_packet_count; }
+	long GetRTCP_SRC_packet_lost() const { return m_rtcp_source_packet_lost; }
+	float GetRTCP_SRC_packet_loss_percent() const { return (m_rtcp_source_packet_count == 0) ? 0 : (float)(m_rtcp_source_packet_lost / m_rtcp_source_packet_count); }
+	int GetRTCP_SRC_jitter_max() const { return m_rtcp_source_jitter_max; }
+	int GetRTCP_SRC_jitter_min() const { return m_rtcp_source_jitter_min; }
+	int GetRTCP_SRC_jitter_avg() const { return m_rtcp_source_jitter_avg; }
+	long GetRTCP_DST_packet_count() const { return m_rtcp_destination_packet_count; }
+	long GetRTCP_DST_packet_lost() const { return m_rtcp_destination_packet_lost; }
+	float GetRTCP_DST_packet_loss_percent() const { return (m_rtcp_destination_packet_count == 0) ? 0 : (float)(m_rtcp_destination_packet_lost / m_rtcp_destination_packet_count); }
+	int GetRTCP_DST_jitter_max() const { return m_rtcp_destination_jitter_max; }
+	int GetRTCP_DST_jitter_min() const { return m_rtcp_destination_jitter_min; }
+	int GetRTCP_DST_jitter_avg() const { return m_rtcp_destination_jitter_avg; }
+
+	// set video RTCP stats
+	void SetRTCP_SRC_video_packet_count(long val);
+	void SetRTCP_SRC_video_packet_lost(long val);
+	void SetRTCP_SRC_video_jitter(int val);
+	void SetRTCP_DST_video_packet_count(long val);
+	void SetRTCP_DST_video_packet_lost(long val);
+	void SetRTCP_DST_video_jitter(int val);
+
+	// get video RTCP stats
+	long GetRTCP_SRC_video_packet_count() const { return m_rtcp_source_video_packet_count; }
+	long GetRTCP_SRC_video_packet_lost() const { return m_rtcp_source_video_packet_lost; }
+	float GetRTCP_SRC_video_packet_loss_percent() const { return (m_rtcp_source_video_packet_count == 0) ? 0 : (float)(m_rtcp_source_video_packet_lost / m_rtcp_source_video_packet_count); }
+	int GetRTCP_SRC_video_jitter_max() const { return m_rtcp_source_video_jitter_max; }
+	int GetRTCP_SRC_video_jitter_min() const { return m_rtcp_source_video_jitter_min; }
+	int GetRTCP_SRC_video_jitter_avg() const { return m_rtcp_source_video_jitter_avg; }
+	long GetRTCP_DST_video_packet_count() const { return m_rtcp_destination_video_packet_count; }
+	long GetRTCP_DST_video_packet_lost() const { return m_rtcp_destination_video_packet_lost; }
+	float GetRTCP_DST_video_packet_loss_percent() const { return (m_rtcp_destination_video_packet_count == 0) ? 0 : (float)(m_rtcp_destination_video_packet_lost / m_rtcp_destination_video_packet_count); }
+	int GetRTCP_DST_video_jitter_max() const { return m_rtcp_destination_video_jitter_max; }
+	int GetRTCP_DST_video_jitter_min() const { return m_rtcp_destination_video_jitter_min; }
+	int GetRTCP_DST_video_jitter_avg() const { return m_rtcp_destination_video_jitter_avg; }
 
-        // Get RTCP source packet count
-        long GetRTCP_SRC_packet_count() const;
-        // Get RTCP destination packet count
-        long GetRTCP_DST_packet_count() const;
-    
-        // Get RTCP source packet lost
-        long GetRTCP_SRC_packet_lost() const;
-        // Get RTCP destination packet lost
-        long GetRTCP_DST_packet_lost() const;
-    
-        // Get RTCP source jitter max
-        int GetRTCP_SRC_jitter_max() const;
-        // Get RTCP source jitter min
-        int GetRTCP_SRC_jitter_min() const;
-        // Get RTCP source jitter avg
-        int GetRTCP_SRC_jitter_avg() const;
-    
-        //Get RTCP destinaton jitter max
-        int GetRTCP_DST_jitter_max() const;
-        //Get RTCP destinaton jitter in
-        int GetRTCP_DST_jitter_min() const;
-        //Get RTCP destinaton jitter avg
-        int GetRTCP_DST_jitter_avg() const;
 	time_t GetSetupTime() const;
 
 	/** Set timestamp for a Setup message associated with this call. */
@@ -1081,14 +1143,12 @@ public:
 		);
 
 	/** @return
-	    Fixed destination address for the call (NULL if not set).
+	    Fixed destination address for the call (size 0 if not set).
 	*/
-	H225_AliasAddress* GetRouteToAlias() const;
+	H225_ArrayOf_AliasAddress GetRouteToAlias() const { return m_routeToAlias; }
 	
 	/// Set fixed destination address for the call
-	void SetRouteToAlias(
-		const H225_AliasAddress& alias /// alias to set
-		);
+	void SetRouteToAlias(const H225_ArrayOf_AliasAddress& alias) { m_routeToAlias = alias; }
 
 	// smart pointer for CallRec
 	typedef SmartPtr<CallRec> Ptr;
@@ -1103,7 +1163,7 @@ public:
 	bool MoveToNextRoute();
 
 	bool IsCallInProgress() const;
-	void SetCallInProgress();
+	void SetCallInProgress(bool val=true);
 
 	bool IsH245ResponseReceived() const;
 	void SetH245ResponseReceived();
@@ -1111,6 +1171,7 @@ public:
 	bool IsFastStartResponseReceived() const;
 	void SetFastStartResponseReceived();
 
+	bool IsFailoverActive() const { return m_failoverActive; }
 	bool SingleFailoverCDR() const;
 	int GetNoCallAttempts() const;
 	int GetNoRemainingRoutes() const;
@@ -1127,17 +1188,24 @@ public:
 	PBYTEArray GetRADIUSClass() const;
 	
 	bool IsProceedingSent() const { return m_proceedingSent; }
-	// TODO: also set when sent through a status port command
 	void SetProceedingSent(bool val) { m_proceedingSent = val; }
-	
+
+	void SetRerouteState(RerouteState state) { m_rerouteState = state; }
+	RerouteState GetRerouteState() const { return m_rerouteState; }
+	void SetRerouteDirection(CallLeg dir) { m_rerouteDirection = dir; }
+	CallLeg GetRerouteDirection() const { return m_rerouteDirection; }
+
 	void SetBindHint(const PString & ip) { m_bindHint = ip; }
 	PString GetBindHint() const { return m_bindHint; }
-	
+
+	void SetCallerID(const PString & id) { m_callerID = id; }
+	PString GetCallerID() const { return m_callerID; }
+
 #ifdef HAS_H46018
-	bool H46019Required();
+	bool H46019Required() const;
 	void StoreSetup(SignalingMsg * msg);
 	PBYTEArray RetrieveSetup();
-	int GetH46019Direction();
+	int GetH46019Direction() const;
 #endif
 
 private:
@@ -1171,7 +1239,7 @@ private:
 	/// called party aliases in a string form
 	PString m_destInfo;
 	/// bandwidth occupied by this call (as declared in ARQ)
-	int m_bandwidth;
+	long m_bandwidth;
 
 	PString m_callerAddr, m_callerId;
 	PString m_calleeAddr, m_calleeId;
@@ -1189,36 +1257,49 @@ private:
 	PStringList m_rtcp_source_sdes;
 	bool m_rtcp_source_sdes_flag;
 
-	PStringList m_rtcp_destination_sdes;  
+	PStringList m_rtcp_destination_sdes;
 	bool m_rtcp_destination_sdes_flag;
 
-	// RTCP_source_packet_count
+	// RTCP audio stats
 	long m_rtcp_source_packet_count;
-	// RTCP_destination_packet_count	
 	long m_rtcp_destination_packet_count;
-
-	// RTCP_source_packet_lost
 	long m_rtcp_source_packet_lost;
-	// RTCP_destination_packet_lost
 	long m_rtcp_destination_packet_lost;
 
-	// RTCP_source_jitter
 	int m_rtcp_source_jitter_min;
 	int m_rtcp_source_jitter_max;
 	int m_rtcp_source_jitter_avg;
 
-	// RTCP_destination_jitter
 	int m_rtcp_destination_jitter_min;
 	int m_rtcp_destination_jitter_max;
 	int m_rtcp_destination_jitter_avg;
 
-
 	int m_rtcp_source_jitter_avg_count;
 	long m_rtcp_source_jitter_avg_sum;
 
 	int m_rtcp_destination_jitter_avg_count;
 	long m_rtcp_destination_jitter_avg_sum;
 
+	// RTCP video stats
+	long m_rtcp_source_video_packet_count;
+	long m_rtcp_destination_video_packet_count;
+	long m_rtcp_source_video_packet_lost;
+	long m_rtcp_destination_video_packet_lost;
+
+	int m_rtcp_source_video_jitter_min;
+	int m_rtcp_source_video_jitter_max;
+	int m_rtcp_source_video_jitter_avg;
+
+	int m_rtcp_destination_video_jitter_min;
+	int m_rtcp_destination_video_jitter_max;
+	int m_rtcp_destination_video_jitter_avg;
+
+	int m_rtcp_source_video_jitter_avg_count;
+	long m_rtcp_source_video_jitter_avg_sum;
+
+	int m_rtcp_destination_video_jitter_avg_count;
+	long m_rtcp_destination_video_jitter_avg_sum;
+
 	/// current timeout (or duration limit) for the call
 	time_t m_timeout;
 	/// timestamp for call timeout measuring
@@ -1260,7 +1341,7 @@ private:
 	/// dialed number (called party's number before rewrite)
 	PString m_dialedNumber;
 	/// fixed destination alias
-	H225_AliasAddress* m_routeToAlias;
+	H225_ArrayOf_AliasAddress m_routeToAlias;
 
 	CallSignalSocket *m_callingSocket, *m_calledSocket;
 
@@ -1308,6 +1389,8 @@ private:
 	bool m_callInProgress;
 	bool m_h245ResponseReceived;
 	bool m_fastStartResponseReceived;
+	/// flag if failover is activated for easy access
+	bool m_failoverActive;
 	bool m_singleFailoverCDR;
 	
 	PString m_codec;
@@ -1320,6 +1403,9 @@ private:
 #endif
 	PUInt64 m_clientAuthId;
 	PString m_bindHint;	// outgoing IP or empty
+	RerouteState m_rerouteState;	// is Pause&Reroute transfer in progress ?
+	CallLeg m_rerouteDirection;
+	PString m_callerID;	// forced caller ID or empty
 };
 
 typedef CallRec::Ptr callptr;
@@ -1340,10 +1426,14 @@ public:
 	void Insert(CallRec * NewRec);
 
 	// bandwidth management
-	void SetTotalBandwidth(int bw);
-	bool GetAdmission(int bw);
-	bool GetAdmission(int bw, const callptr &);
-	int GetAvailableBW() const { return m_capacity; }
+	void SetTotalBandwidth(long bw);
+	long GetAvailableBW() const { return m_capacity; }
+	long GetMinimumBandwidthPerCall() const { return m_minimumBandwidthPerCall; }
+	long GetMaximumBandwidthPerCall() const { return m_maximumBandwidthPerCall; }
+	long CheckTotalBandwidth(long bw) const;
+	void UpdateTotalBandwidth(long bw);
+	long CheckEPBandwidth(const endptr & ep, long bw) const;
+	void UpdateEPBandwidth(const endptr & ep, long bw);
 
 	callptr FindCallRec(const H225_CallIdentifier & CallId) const;
 	callptr FindCallRec(const H225_CallReferenceValue & CallRef) const;
@@ -1360,6 +1450,8 @@ public:
 	void RemoveCall(const H225_DisengageRequest & obj_drq, const endptr &);
 	void RemoveCall(const callptr &);
 	void RemoveFailedLeg(const callptr &);
+	void DropCallingParty();
+	void DropCalledParty();
 
 	void PrintCurrentCalls(USocket *client, bool verbose=FALSE) const;
 	PString PrintStatistics() const;
@@ -1369,6 +1461,7 @@ public:
     void QoSReport(const H225_DisengageRequest &, const endptr &, const PASN_OctetString &);
     void QoSReport(const H225_InfoRequestResponse &, const callptr &, const endptr &, const PASN_OctetString &);
 #endif
+	void SupplyEndpointQoS(std::map<PString, EPQoS> & epqos) const;
 
 	void LoadConfig();
 	void UpdatePrefixCapacityCounters();    // after Reload
@@ -1420,7 +1513,9 @@ private:
 	PINDEX m_CallNumber;
 	mutable PReadWriteMutex listLock;
 
-	int m_capacity;
+	long m_capacity;	// total available bandwidth for gatekeeper (-1 = unlimited)
+	long m_minimumBandwidthPerCall;	// don't accept bandwith requests from endpoints lower tan this (eg. for Netmeeting)
+	long m_maximumBandwidthPerCall;	// maximum bandwidth allowed per call (<= 0 means unlimited)
 
 	// statistics
 	unsigned m_CallCount, m_successCall, m_neighborCall, m_parentCall, m_activeCall;
@@ -1529,7 +1624,7 @@ inline void EndpointRec::SetEndpointInfo(const PString & vendor, const PString &
 	m_endpointVendor = new H225_VendorIdentifier();
 
 	m_endpointVendor->IncludeOptionalField(H225_VendorIdentifier::e_productId);
-	   m_endpointVendor->m_productId.SetValue(vendor); 
+	   m_endpointVendor->m_productId.SetValue(vendor);
 	m_endpointVendor->IncludeOptionalField(H225_VendorIdentifier::e_versionId);
 	   m_endpointVendor->m_versionId.SetValue(version);
 }
@@ -1643,7 +1738,8 @@ inline bool EndpointRec::IsUsed() const
 inline bool EndpointRec::IsUpdated(const PTime *now) const
 {
 	PWaitAndSignal lock(m_usedLock);
-	return (!m_timeToLive || (*now - m_updatedTime).GetSeconds() < m_timeToLive);
+	int ttl = GetTimeToLive();
+	return (!ttl || (*now - m_updatedTime).GetSeconds() < ttl);
 }
 
 inline bool EndpointRec::HasNATSocket() const
@@ -1781,56 +1877,6 @@ inline PStringList CallRec::GetRTCP_DST_sdes() const
     return m_rtcp_destination_sdes;
 }
 
-inline long CallRec::GetRTCP_SRC_packet_count() const
-{
-    return m_rtcp_source_packet_count;
-}
-
-inline long CallRec::GetRTCP_DST_packet_count() const
-{
-    return m_rtcp_destination_packet_count;
-}
-
-inline long CallRec::GetRTCP_SRC_packet_lost() const
-{
-    return m_rtcp_source_packet_lost;
-}
-
-inline long CallRec::GetRTCP_DST_packet_lost() const
-{
-    return m_rtcp_destination_packet_lost;
-}
-
-inline int CallRec::GetRTCP_SRC_jitter_max() const
-{
-    return m_rtcp_source_jitter_max;
-}
-
-inline int CallRec::GetRTCP_SRC_jitter_min() const
-{
-    return m_rtcp_source_jitter_min;
-}
-
-inline int CallRec::GetRTCP_SRC_jitter_avg() const
-{
-    return m_rtcp_source_jitter_avg;
-}
-
-inline int CallRec::GetRTCP_DST_jitter_max() const
-{
-    return m_rtcp_destination_jitter_max;
-}
-
-inline int CallRec::GetRTCP_DST_jitter_min() const
-{
-    return m_rtcp_destination_jitter_min;
-}
-
-inline int CallRec::GetRTCP_DST_jitter_avg() const
-{
-    return m_rtcp_destination_jitter_avg;
-}
-
 inline time_t CallRec::GetSetupTime() const
 {
 	return m_setupTime;
@@ -1889,17 +1935,20 @@ class PreliminaryCall
 {
 public:
 	PreliminaryCall(CallSignalSocket * callerSocket, H225_CallIdentifier id, unsigned ref)
-		: m_socket(callerSocket), m_callid(id), m_callref(ref) { }
+		: m_socket(callerSocket), m_callid(id), m_callref(ref), m_proceedingSent(false) { }
 	~PreliminaryCall() { }
 
 	CallSignalSocket * GetCallSignalSocketCalling() const { return m_socket; }
 	H225_CallIdentifier GetCallIdentifier() const { return m_callid; }
 	unsigned GetCallRef() const { return m_callref; }
+	bool IsProceedingSent() const { return m_proceedingSent; }
+	void SetProceedingSent(bool val) { m_proceedingSent = val; }
 
 private:
 	CallSignalSocket * m_socket;
 	H225_CallIdentifier m_callid;
 	unsigned m_callref;
+	bool m_proceedingSent;
 };
 
 // hold data about calls being established, only valid during the routing process before calls are accepted or rejected
