@@ -3,14 +3,12 @@
 // Routing Mechanism for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2003
+// Copyright (c) 2004-2011, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
-// initial version: 06/18/2003
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -20,7 +18,6 @@
 #include "gk_const.h"
 #include "h323util.h"
 #include "Toolkit.h"
-#include "stl_supp.h"
 #include "RasTbl.h"
 #include "RasSrv.h"
 #include "GkClient.h"
@@ -30,6 +27,10 @@
 #include "gksql.h"
 #include "config.h"
 
+#ifdef HAS_H46023
+  #include <h460/h4601.h>
+#endif
+
 using std::string;
 using std::vector;
 using std::list;
@@ -49,33 +50,40 @@ const char *SectionName[] = {
 const long DEFAULT_ROUTE_REQUEST_TIMEOUT = 10;
 const char* const CTIsection = "CTI::Agents";
 
-Route::Route() : m_proxyMode(CallRec::ProxyDetect), m_flags(0)
+bool DNSPolicy::m_resolveNonLocalLRQs = false;
+
+Route::Route() : m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(1)
+{
+	m_destAddr.SetTag(H225_TransportAddress::e_nonStandardAddress);	// set to an invalid address
+	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
+}
+
+Route::Route(
+	const PString & policyName,
+	const endptr & destEndpoint,
+	unsigned priority
+	) : m_destAddr(destEndpoint->GetCallSignalAddress()), m_destEndpoint(destEndpoint), m_policy(policyName),
+	m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(priority)
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
 
 Route::Route(
-	const endptr &destEndpoint
-	) : m_destAddr(destEndpoint->GetCallSignalAddress()), m_destEndpoint(destEndpoint),
-	m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber(""), m_destOutNumber("")
+	const PString & policyName,
+	const H225_TransportAddress & destAddr,
+	unsigned priority
+	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(priority)
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
 
 Route::Route(
-	const PString &policyName,
-	const H225_TransportAddress &destAddr
-	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber(""), m_destOutNumber("")
-{
-	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
-}
-
-Route::Route(
-	const PString &policyName,
-	const PIPSocket::Address &destIpAddr,
-	WORD destPort
+	const PString & policyName,
+	const PIPSocket::Address & destIpAddr,
+	WORD destPort,
+	unsigned priority
 	) : m_destAddr(SocketToH225TransportAddr(destIpAddr, destPort)),
-	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber(""), m_destOutNumber("")
+	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(priority)
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -84,7 +92,7 @@ PString Route::AsString() const
 {
 	return AsDotString(m_destAddr) + " (policy: " + m_policy + ", proxy: "
 		+ PString(m_proxyMode) + ", flags: " + PString(m_flags) + ", Called-Station-Id: " + m_destNumber
-		+ ", Called-Station-Id-Out: " + m_destOutNumber + ")";
+		+ ", Called-Station-Id-Out: " + m_destOutNumber + ", priority: " + PString(m_priority) + ")";
 }
 
 bool Route::IsFailoverActive(
@@ -112,9 +120,7 @@ RoutingRequest::~RoutingRequest()
 {
 }
 
-bool RoutingRequest::AddRoute(
-	const Route &route
-	)
+bool RoutingRequest::AddRoute(const Route & route)
 {
 	PIPSocket::Address addr;
 	WORD port;
@@ -133,12 +139,11 @@ bool RoutingRequest::AddRoute(
 		++i;
 	}
 	m_routes.push_back(route);
+	m_routes.sort();	// put routes in priority order
 	return true;
 }
 
-bool RoutingRequest::GetFirstRoute(
-	Route &route
-	)
+bool RoutingRequest::GetFirstRoute(Route & route)
 {
 	if (m_routes.empty())
 		return false;
@@ -363,34 +368,71 @@ bool AliasesPolicy::OnRequest(FacilityRequest & request)
 }
 
 
-// the simplest policy, the destination has been explicitly specified
-class ExplicitPolicy : public Policy {
-public:
-	ExplicitPolicy() { m_name = "Explicit"; }
-protected:
-	virtual bool OnRequest(AdmissionRequest &);
-	// the policy doesn't apply to LocationRequest
-	virtual bool OnRequest(SetupRequest &);
-	virtual bool OnRequest(FacilityRequest &);
-};
+std::map<PString, H225_TransportAddress> ExplicitPolicy::m_destMap;
+
+ExplicitPolicy::ExplicitPolicy()
+{
+	m_name = "Explicit";
+}
+
+void ExplicitPolicy::OnReload()
+{
+	m_destMap.clear();
+	PStringToString mappings(GkConfig()->GetAllKeyValues("Routing::Explicit"));
+	for (PINDEX i = 0; i < mappings.GetSize(); ++i) {
+		PString src = mappings.GetKeyAt(i);
+		PString dest = mappings.GetDataAt(i);
+		if (!dest.IsEmpty()) {
+			H225_TransportAddress addr;
+			if (GetTransportAddress(dest, 1720, addr)) {
+				m_destMap[src] = addr;
+			} else {
+				PTRACE(1, "Error parsing entry in [Routing::Explicit]: " << src << "=" << dest);
+			}
+		}
+	}
+}
+
+void ExplicitPolicy::MapDestination(H225_TransportAddress & addr)
+{
+	PString orig = AsDotString(addr, false);	// original IP without port
+	std::map<PString, H225_TransportAddress>::const_iterator i = m_destMap.find(orig);
+	if (i != m_destMap.end()) {
+		addr = i->second;
+		PTRACE(4, "[Routing::Explicit]: map destination " << orig << " to " << AsDotString(i->second));
+	}
+}
 
 bool ExplicitPolicy::OnRequest(AdmissionRequest & request)
 {
 	H225_AdmissionRequest & arq = request.GetRequest();
 	if (arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
+		MapDestination(arq.m_destCallSignalAddress);
 		Route route(m_name, arq.m_destCallSignalAddress);
 		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 			route.m_destAddr
 			);
+#ifdef HAS_H46023
+        if (!route.m_destEndpoint && 
+           arq.HasOptionalField(H225_AdmissionRequest::e_genericData) &&
+           H460_FeatureSet(arq.m_genericData).HasFeature(24)) {
+	         H225_RasMessage ras;
+             ras.SetTag(H225_RasMessage::e_admissionRequest);
+			 H225_AdmissionRequest & req = (H225_AdmissionRequest &)ras;
+			 req = arq;
+		   route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);	
+        }
+#endif
 		return request.AddRoute(route);
 	}
 	return false;
 }
 
-bool ExplicitPolicy::OnRequest(SetupRequest &request)
+bool ExplicitPolicy::OnRequest(SetupRequest & request)
 {
 	H225_Setup_UUIE &setup = request.GetRequest();
 	if (setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
+		// don't map IP here, for Setup already done in OnSetup()
 		Route route(m_name, setup.m_destCallSignalAddress);
 		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 			route.m_destAddr
@@ -404,6 +446,7 @@ bool ExplicitPolicy::OnRequest(FacilityRequest & request)
 {
 	H225_Facility_UUIE & facility = request.GetRequest();
 	if (facility.HasOptionalField(H225_Facility_UUIE::e_alternativeAddress)) {
+		MapDestination(facility.m_alternativeAddress);
 		Route route(m_name, facility.m_alternativeAddress);
 		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 			route.m_destAddr
@@ -414,24 +457,6 @@ bool ExplicitPolicy::OnRequest(FacilityRequest & request)
 }
 
 
-// the classical policy, find the dstionation from the RegistrationTable
-class InternalPolicy : public AliasesPolicy {
-public:
-	InternalPolicy();
-
-protected:
-	virtual bool OnRequest(AdmissionRequest &);
-	virtual bool OnRequest(SetupRequest &);
-
-	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
-	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
-	virtual bool FindByAliases(SetupRequest &, H225_ArrayOf_AliasAddress &);
-	virtual bool FindByAliases(AdmissionRequest &, H225_ArrayOf_AliasAddress &);
-	
-private:
-	bool roundRobin;
-};
-
 InternalPolicy::InternalPolicy()
 	: roundRobin(Toolkit::AsBool(GkConfig()->GetString("RasSrv::ARQFeatures", "RoundRobinGateways", "1")))
 {
@@ -524,25 +549,6 @@ bool InternalPolicy::FindByAliases(
 }
 
 
-// a policy to route call to parent
-// the policy was originally in GkClient.cxx,
-// but damn VC has problem to instantiate the creator
-class ParentPolicy : public Policy {
-public:
-	ParentPolicy();
-
-private:
-	// override from class Policy
-	virtual bool IsActive() const;
-
-	virtual bool OnRequest(AdmissionRequest &);
-	virtual bool OnRequest(LocationRequest &);
-	virtual bool OnRequest(SetupRequest &);
-	virtual bool OnRequest(FacilityRequest &);
-
-	GkClient *m_gkClient;
-};
-
 ParentPolicy::ParentPolicy()
 {
 	m_gkClient = RasServer::Instance()->GetGkClient();
@@ -574,15 +580,11 @@ bool ParentPolicy::OnRequest(FacilityRequest & facility_obj)
 	return !(facility_obj.GetFlags() & RoutingRequest::e_fromParent) && m_gkClient->SendARQ(facility_obj);
 }
 
-
-// a policy to look up the destination from DNS
-class DNSPolicy : public AliasesPolicy {
-public:
-	DNSPolicy() { m_name = "DNS"; }
-protected:
-	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
-	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
-};
+DNSPolicy::DNSPolicy()
+{
+	m_name = "DNS";
+	m_resolveNonLocalLRQs = Toolkit::AsBool(GkConfig()->GetString("Routing::DNS", "ResolveNonLocalLRQ", "1"));
+}
 
 bool DNSPolicy::FindByAliases(
 	RoutingRequest &request,
@@ -597,6 +599,8 @@ bool DNSPolicy::FindByAliases(
 		PINDEX at = alias.Find('@');
 
 		PString domain = (at != P_MAX_INDEX) ? alias.Mid(at + 1) : alias;
+		if (domain.Find("ip$") == 0)
+			domain.Replace("ip$", "", false);
 		H225_TransportAddress dest;
 		if (GetTransportAddress(domain, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
 			PIPSocket::Address addr;
@@ -613,7 +617,6 @@ bool DNSPolicy::FindByAliases(
 				} else {
 					continue;	// can't route this alias locally, try next alias
 				}
-			} else {
 			}
 			Route route(m_name, dest);
 			route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
@@ -638,6 +641,8 @@ bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 		PINDEX at = alias.Find('@');
 
 		PString domain = (at != P_MAX_INDEX) ? alias.Mid(at + 1) : alias;
+		if (domain.Find("ip$") == 0)
+			domain.Replace("ip$", "", false);
 		H225_TransportAddress dest;
 		if (GetTransportAddress(domain, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
 			PIPSocket::Address addr;
@@ -663,10 +668,17 @@ bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 					PTRACE(4, "ROUTING\tDNS policy resolves to " << alias.Left(at));
 					return true;
 				}
+			} else if (m_resolveNonLocalLRQs) {
+				Route route(m_name, dest);
+				request.AddRoute(route);
+				PTRACE(4, "ROUTING\tDNS policy resolves to " << domain);
+				return true;
 			}
 		}
 	}
-	PTRACE(4, "ROUTING\tPolicy DNS only supports LRQs that resolve locally");
+	if (!m_resolveNonLocalLRQs) {
+		PTRACE(4, "ROUTING\tPolicy DNS only routes LRQs that resolve locally");
+	}
 	return false;
 }
 
@@ -772,6 +784,8 @@ bool VirtualQueue::SendRouteRequest(
 	PString* callSigAdr,
 	/// bind IP for BindAndRouteToGateway
 	PString* bindIP,
+	/// caller ID
+	PString* callerID,
 	/// should the call be rejected modified by this function on return)
 	bool & reject,
 	/// actual virtual queue name (should be present in destinationInfo too)
@@ -782,22 +796,25 @@ bool VirtualQueue::SendRouteRequest(
 	/// the callID as string
 	const PString& callID,
 	/// the called IP for unregistered calls
-	const PString& calledip
+	const PString& calledip,
+	/// vendor string of caller
+	const PString& vendorString
 	)
 {
 	bool result = false;
 	bool duprequest = false;
-	if (RouteRequest *r = InsertRequest(epid, crv, callID, destinationInfo, callSigAdr, bindIP, duprequest)) {
+	if (RouteRequest *r = InsertRequest(epid, crv, callID, destinationInfo, callSigAdr, bindIP, callerID, duprequest)) {
+		PString cid = callID;
+		cid.Replace(" ", "-", true);
 		PString msg = "RouteRequest|" + source
 						+ "|" + epid
 						+ "|" + PString(crv)
 						+ "|" + vqueue
-						+ "|" + sourceInfo;
-		PString cid = callID;
-		cid.Replace(" ", "-", true);
-		msg += PString("|") + cid;
-		msg += PString("|") + calledip;
-		msg += PString(";");
+						+ "|" + sourceInfo
+						+ "|" + cid
+						+ "|" + calledip
+						+ "|" + vendorString
+						+ ";";
 		// signal RouteRequest to the status line only once
 		if( duprequest ) {
 			PTRACE(4, "VQueue\tDuplicate request: "<<msg);
@@ -842,7 +859,7 @@ bool VirtualQueue::RouteToAlias(
 	/// aliases for the routing target (an agent that the call will be routed to) 
 	/// that will replace the original destination info
 	const H225_ArrayOf_AliasAddress& agent,
-	/// ip that will replace the destionationCallSignalAddress (RouteToGateway)
+	/// ip that will replace the destinationCallSignalAddress (RouteToGateway)
 	/// used only if set (!= NULL)
 	const PString& destinationip,
 	/// identifier of the endpoint associated with the route request
@@ -853,6 +870,8 @@ bool VirtualQueue::RouteToAlias(
 	const PString& callID,
 	// outgoing IP or empty
 	const PString& bindIP,
+	// caller ID or empty
+	const PString& callerID,
 	/// should this call be rejected
 	bool reject
 	)
@@ -879,6 +898,7 @@ bool VirtualQueue::RouteToAlias(
 				if (!destinationip.IsEmpty())	// RouteToGateway
 					*(r->m_callsignaladdr) = destinationip;
 				*(r->m_sourceIP) = bindIP;	// BindAndRouteToGateway
+				*(r->m_callerID) = callerID;
 			}
 			r->m_reject = reject;
 			r->m_sync.Signal();
@@ -918,6 +938,8 @@ bool VirtualQueue::RouteToAlias(
 	const PString& callID,
 	// outgoing IP or empty
 	const PString& bindIP,
+	// callerID or empty
+	const PString& callerID,
 	/// should this call be rejected
 	bool reject
 	)
@@ -927,7 +949,7 @@ bool VirtualQueue::RouteToAlias(
 		alias.SetSize(1);
 		H323SetAliasAddress(targetAlias, alias[0]);
 	}
-	return RouteToAlias(alias, destinationIp, callingEpId, crv, callID, bindIP, reject);
+	return RouteToAlias(alias, destinationIp, callingEpId, crv, callID, bindIP, callerID, reject);
 }
 
 bool VirtualQueue::RouteReject(
@@ -940,7 +962,7 @@ bool VirtualQueue::RouteReject(
 	)
 {
 	H225_ArrayOf_AliasAddress nullAgent;
-	return RouteToAlias(nullAgent, "", callingEpId, crv, callID, "", true);
+	return RouteToAlias(nullAgent, "", callingEpId, crv, callID, "", "", true);
 }
 
 VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
@@ -958,6 +980,8 @@ VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
 	PString* callSigAdr,
 	/// bind IP for BindAndRouteToGateway
 	PString* bindIP,
+	/// caller ID
+	PString* callerID,
 	/// set by the function to true if another route request for the same
 	/// call is pending
 	bool& duplicate
@@ -985,29 +1009,12 @@ VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
 	}
 
 	// insert the new pending route request
-	RouteRequest* r = new RouteRequest(callingEpId, crv, callID, agent, callSigAdr, bindIP);
+	RouteRequest* r = new RouteRequest(callingEpId, crv, callID, agent, callSigAdr, bindIP, callerID);
 	m_pendingRequests.push_back(r);
 	return r;
 }
 
 
-// a policy to route call via external program
-class VirtualQueuePolicy : public Policy {
-public:
-	VirtualQueuePolicy();
-
-protected:
-	// override from class Policy
-	virtual bool IsActive() const;
-
-	virtual bool OnRequest(AdmissionRequest &);
-	virtual bool OnRequest(LocationRequest &);
-	virtual bool OnRequest(SetupRequest &);
-
-private:
-	VirtualQueue *m_vqueue;
-};
-
 VirtualQueuePolicy::VirtualQueuePolicy()
 {
 	m_vqueue = RasServer::Instance()->GetVirtualQueue();
@@ -1029,21 +1036,35 @@ bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 	if (m_vqueue->IsDestinationVirtualQueue(vq)) {
 		H225_AdmissionRequest & arq = request.GetRequest();
 		PTRACE(5,"Routing\tPolicy " << m_name << " destination matched "
-			"a virtual queue " << vq << " (ARQ "
-			<< arq.m_requestSeqNum.GetValue() << ')'
-			);
+			"a virtual queue " << vq << " (ARQ " << arq.m_requestSeqNum.GetValue() << ')');
 		endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq.m_endpointIdentifier); // should not be null
 		if (ep) {
 			PString source = AsDotString(ep->GetCallSignalAddress());
 			PString epid = ep->GetEndpointIdentifier().GetValue();
 			PString * callSigAdr = new PString();
 			PString * bindIP = new PString();
-			if (m_vqueue->SendRouteRequest(source, epid, unsigned(arq.m_callReferenceValue), aliases, callSigAdr, bindIP, reject, vq, AsString(arq.m_srcInfo), AsString(arq.m_callIdentifier.m_guid)))
+			PString * callerID = new PString();
+			PString calledIP = "unknown";
+			PString vendorInfo;
+			if (arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
+				calledIP = AsString(arq.m_destCallSignalAddress, false);
+			}
+			if (ep->GetEndpointType().HasOptionalField(H225_EndpointType::e_vendor)) {
+				if (ep->GetEndpointType().m_vendor.HasOptionalField(H225_VendorIdentifier::e_productId)) {
+					vendorInfo += ep->GetEndpointType().m_vendor.m_productId.AsString();
+				}
+				if (ep->GetEndpointType().m_vendor.HasOptionalField(H225_VendorIdentifier::e_versionId)) {
+					vendorInfo += ep->GetEndpointType().m_vendor.m_versionId.AsString();
+				}
+			}
+
+			if (m_vqueue->SendRouteRequest(source, epid, unsigned(arq.m_callReferenceValue), aliases, callSigAdr, bindIP, callerID, reject, vq, AsString(arq.m_srcInfo), AsString(arq.m_callIdentifier.m_guid), calledIP, vendorInfo))
 				request.SetFlag(RoutingRequest::e_aliasesChanged);
 			if (reject) {
 				request.SetFlag(RoutingRequest::e_Reject);
 			}
 			request.SetSourceIP(*bindIP);
+			request.SetCallerID(*callerID);
 			if (!callSigAdr->IsEmpty()) {
 				if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
 					arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
@@ -1057,6 +1078,7 @@ bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 			}
 			delete callSigAdr;
 			delete bindIP;
+			delete callerID;
 		}
 		// the trick: if empty, the request is rejected
 		// so we return true to terminate the routing
@@ -1099,16 +1121,18 @@ bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 			}
 			PString * callSigAdr = new PString(); /* unused for LRQs */
 			PString * bindIP = new PString();
+			PString * callerID = new PString();
 			PString sourceInfo = "";
 			if (lrq.HasOptionalField(H225_LocationRequest::e_sourceInfo) && (lrq.m_sourceInfo.GetSize() > 0))
 				sourceInfo = AsString(lrq.m_sourceInfo);
 			PString callID = "-";	/* not available for LRQs */
-			if (m_vqueue->SendRouteRequest(source, epid, unsigned(lrq.m_requestSeqNum), aliases, callSigAdr, bindIP, reject, vq, sourceInfo, callID))
+			if (m_vqueue->SendRouteRequest(source, epid, unsigned(lrq.m_requestSeqNum), aliases, callSigAdr, bindIP, callerID, reject, vq, sourceInfo, callID))
 				request.SetFlag(RoutingRequest::e_aliasesChanged);
 			if (reject) {
 				request.SetFlag(RoutingRequest::e_Reject);
 			}
 			request.SetSourceIP(*bindIP);
+			request.SetCallerID(*callerID);
 			if (!reject && !callSigAdr->IsEmpty()) {
 				// 'explicit' policy can't handle LRQs, so we do it directly
 				PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
@@ -1123,10 +1147,12 @@ bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 				request.AddRoute(route);					
 				delete callSigAdr;
 				delete bindIP;
+				delete callerID;
 				return true;	// stop processing
 			}
 			delete callSigAdr;
 			delete bindIP;
+			delete callerID;
 			// the trick: if empty, the request is rejected
 			// so we return true to terminate the routing
 			// decision process, otherwise the aliases is
@@ -1155,24 +1181,36 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 		const unsigned crv = request.GetWrapper()->GetCallReference();
 		PString * callSigAdr = new PString();
 		PString * bindIP = new PString();
+		PString * callerID = new PString();
 		PString callid = AsString(setup.m_callIdentifier.m_guid);
 		PString src = AsString(setup.m_sourceAddress);
 		PIPSocket::Address localAddr;
 		WORD localPort;
 		request.GetWrapper()->GetLocalAddr(localAddr, localPort);
 		PString calledIP = localAddr;
+		PString vendorInfo;
+		if (setup.m_sourceInfo.HasOptionalField(H225_EndpointType::e_vendor)) {
+			if (setup.m_sourceInfo.m_vendor.HasOptionalField(H225_VendorIdentifier::e_productId)) {
+				vendorInfo += setup.m_sourceInfo.m_vendor.m_productId.AsString();
+			}
+			if (setup.m_sourceInfo.m_vendor.HasOptionalField(H225_VendorIdentifier::e_versionId)) {
+				vendorInfo += setup.m_sourceInfo.m_vendor.m_versionId.AsString();
+			}
+			vendorInfo.Replace("|", "", true);
+		}
 		PTRACE(5,"Routing\tPolicy " << m_name << " destination matched "
 			"a virtual queue " << vq << " (Setup "
 			<< crv << ')'
 			);
 
-		if (m_vqueue->SendRouteRequest(callerip, epid, crv, aliases, callSigAdr, bindIP, reject, vq, src, callid, calledIP))
+		if (m_vqueue->SendRouteRequest(callerip, epid, crv, aliases, callSigAdr, bindIP, callerID, reject, vq, src, callid, calledIP, vendorInfo))
 			request.SetFlag(RoutingRequest::e_aliasesChanged);
 		
 		if (reject) {
 			request.SetFlag(RoutingRequest::e_Reject);
 		}
 		request.SetSourceIP(*bindIP);
+		request.SetCallerID(*callerID);
 		if (!reject && callSigAdr->IsEmpty()) {
 			if (!setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress)) {
 				setup.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
@@ -1196,6 +1234,7 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 		}
 		delete callSigAdr;
 		delete bindIP;
+		delete callerID;
 		// the trick: if empty, the request is rejected
 		// so we return true to terminate the routing
 		// decision process, otherwise the aliases is
@@ -1210,31 +1249,6 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 	return false;
 }
 
-class NumberAnalysisPolicy : public Policy {
-public:
-	struct PrefixEntry {
-		string m_prefix;
-		int m_minLength;
-		int m_maxLength;
-	};
-	
-	NumberAnalysisPolicy();
-
-protected:
-	virtual bool OnRequest(AdmissionRequest &);
-	virtual bool OnRequest(SetupRequest &);
-
-private:
-	NumberAnalysisPolicy(const NumberAnalysisPolicy &);
-	NumberAnalysisPolicy& operator=(const NumberAnalysisPolicy &);
-	
-private:
-	typedef vector<PrefixEntry> Prefixes;
-
-	/// list of number prefixes, with min/max number length as values
-	Prefixes m_prefixes;
-};
-
 struct PrefixGreater : public binary_function<NumberAnalysisPolicy::PrefixEntry, NumberAnalysisPolicy::PrefixEntry, bool> {
 
 	bool operator()(const NumberAnalysisPolicy::PrefixEntry &e1, const NumberAnalysisPolicy::PrefixEntry &e2) const 
@@ -1347,17 +1361,6 @@ bool NumberAnalysisPolicy::OnRequest(SetupRequest & request)
 	return false;
 }
 
-// a policy to look up the destination from ENUM Name Server
-class ENUMPolicy : public AliasesPolicy {
-public:
-	ENUMPolicy() { m_name = "ENUM"; }
-protected:
-    virtual bool OnRequest(SetupRequest &) { return false; }
-    virtual bool OnRequest(FacilityRequest &) { return false; }
-
-	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
-	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
-};
 
 bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
@@ -1403,69 +1406,6 @@ bool ENUMPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_Ali
 
 
 
-class DestinationRoutes {
-public:
-	DestinationRoutes() { m_endChain = false; m_reject = false; m_rejectReason = 0; m_aliasesChanged = false; }
-	~DestinationRoutes() { }
-	
-	bool EndPolicyChain() const { return m_endChain; }
-	bool RejectCall() const { return m_reject; }
-	void SetRejectCall(bool reject) { m_reject = reject; m_endChain = true; }
-	unsigned int GetRejectReason() const { return m_rejectReason; }
-	void SetRejectReason(unsigned int reason) { m_rejectReason = reason; }
-	bool ChangeAliases() const { return m_aliasesChanged; }
-	H225_ArrayOf_AliasAddress GetNewAliases() const { return m_newAliases; }
-	void SetNewAliases(const H225_ArrayOf_AliasAddress & aliases) { m_newAliases = aliases; m_aliasesChanged = true; }
-	
-	void AddRoute(const Route & route) { m_routes.push_back(route); m_endChain = true; }
-	
-	std::list<Route> m_routes;
-
-protected:
-	bool m_endChain;
-	bool m_reject;
-	unsigned int m_rejectReason;
-	bool m_aliasesChanged;
-	H225_ArrayOf_AliasAddress m_newAliases;
-};
-
-// a policy to route calls via an SQL database
-class SqlPolicy : public Policy {
-public:
-	SqlPolicy();
-	virtual ~SqlPolicy();
-
-protected:
-	virtual bool IsActive() const { return m_active; }
-
-	virtual bool OnRequest(AdmissionRequest &);
-	virtual bool OnRequest(LocationRequest &);
-	virtual bool OnRequest(SetupRequest &);
-
-	virtual void DatabaseLookup(
-		/*in */
-		const PString & source,
-		const PString & calledAlias,
-		const PString & calledIP,
-		const PString & caller,
-		const PString & callingStationId,
-		const PString & callid,
-		const PString & messageType,
-		const PString & clientauthid,
-		/* out: */
-		DestinationRoutes & destination);
-
-private:
-	// active ?
-	bool m_active;
-	// connection to the SQL database
-	GkSQLConnection* m_sqlConn;
-	// parametrized query string for the routing query
-	PString m_query;
-	// query timeout
-	long m_timeout;
-};
-
 SqlPolicy::SqlPolicy()
 {
 	m_active = false;
@@ -1486,7 +1426,7 @@ SqlPolicy::SqlPolicy()
 		m_active = false;
 		return;
 	}
-	
+
 	m_sqlConn = GkSQLConnection::Create(driverName, m_name);
 	if (m_sqlConn == NULL) {
 		PTRACE(2, m_name << "\tmodule creation failed: "
@@ -1634,7 +1574,7 @@ bool SqlPolicy::OnRequest(SetupRequest & request)
 	PIPSocket::Address localAddr;
 	WORD localPort;
 	request.GetWrapper()->GetLocalAddr(localAddr, localPort);
-	PString calledIP = localAddr;
+	PString calledIP = localAddr;	// TODO: only correct if a gatekeeper IP was called, should we use explicit IP if present ?
 	PString caller = AsString(setup.m_sourceAddress, FALSE);
 	PString callingStationId = request.GetCallingStationId();
 	PString callid = AsString(setup.m_callIdentifier.m_guid);
@@ -1677,7 +1617,7 @@ bool SqlPolicy::OnRequest(SetupRequest & request)
 }
 
 void SqlPolicy::DatabaseLookup(
-		/*in */
+		/* in */
 		const PString & source,
 		const PString & calledAlias,
 		const PString & calledIP,
@@ -1717,12 +1657,13 @@ void SqlPolicy::DatabaseLookup(
 	if (result->GetNumRows() < 1)
 		PTRACE(3, m_name << ": query returned no rows");
 	else if (result->GetNumFields() < 1)
-		PTRACE(2, m_name << ": bad-formed query - "
+		PTRACE(2, m_name << ": bad query - "
 			"no columns found in the result set"
 			);
 	else if (!result->FetchRow(resultRow) || resultRow.empty())
 		PTRACE(2, m_name << ": query failed - could not fetch the result row");
-	else if (result->GetNumFields() == 1) {
+	else if ((result->GetNumFields() == 1)
+			|| ((result->GetNumFields() == 2) && (resultRow[1].first.ToUpper() == "IGNORE")) ) {
 		PString newDestination = resultRow[0].first;
 		PTRACE(5, m_name << "\tQuery result : " << newDestination);
 		if (newDestination.ToUpper() == "REJECT") {
@@ -1794,23 +1735,6 @@ void SqlPolicy::DatabaseLookup(
 }
 
 
-// a policy to route all calls to one default endpoint
-class CatchAllPolicy : public Policy {
-public:
-	CatchAllPolicy();
-	virtual ~CatchAllPolicy() { }
-
-protected:
-	virtual bool OnRequest(AdmissionRequest & request) { return CatchAllRoute(request); }
-	virtual bool OnRequest(LocationRequest & request) { return CatchAllRoute(request); }
-	virtual bool OnRequest(SetupRequest & request) { return CatchAllRoute(request); }
-
-	bool CatchAllRoute(RoutingRequest & request) const;
-	
-	PString m_catchAllAlias;
-	PString m_catchAllIP;
-};
-
 CatchAllPolicy::CatchAllPolicy()
 {
 	m_name = "CatchAllPolicy";
@@ -1828,10 +1752,10 @@ bool CatchAllPolicy::CatchAllRoute(RoutingRequest & request) const
 	if (!m_catchAllIP.IsEmpty()) {
 		H225_TransportAddress destAddr;
 		if (GetTransportAddress(m_catchAllIP, GK_DEF_UNICAST_RAS_PORT, destAddr)) {
-			Route route("catchall", destAddr);
+			Route route("catchall", destAddr, 999);
 			route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
 			request.AddRoute(route);
-			return true;
+			return (m_next == NULL);
 		} else {
 			PTRACE(1, m_name << "\tInvalid catch-all IP " << m_catchAllIP);
 		}
@@ -1841,10 +1765,9 @@ bool CatchAllPolicy::CatchAllRoute(RoutingRequest & request) const
 	H323SetAliasAddress(m_catchAllAlias, find_aliases[0]);
 	endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
 	if (ep) {
-		Route route(ep);
-		route.m_policy = m_name;
+		Route route("catchall", ep, 999);
 		request.AddRoute(route);
-		return true;
+		return (m_next == NULL);
 	}
 	PTRACE(1, m_name << "\tCatch-all endpoint " << m_catchAllAlias << " not found!");
 	return false;	// configured default endpoint not found
