@@ -5,25 +5,25 @@
 // Routing Mechanism for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2003
+// Copyright (c) 2004-2010, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
-// initial version: 06/18/2003
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.39 2010/04/06 07:05:33 willamowius Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.54 2011/07/25 09:10:21 willamowius Exp $"
 
 #include <map>
 #include <list>
 #include "slist.h"
 #include "singleton.h"
 #include "RasTbl.h"
+#include "stl_supp.h"
+
 
 // forward references to avoid includes
 class H225_AdmissionRequest;
@@ -39,11 +39,16 @@ typedef H225SignalingMsg<H225_Setup_UUIE> SetupMsg;
 typedef H225SignalingMsg<H225_Facility_UUIE> FacilityMsg;
 
 class RasMsg;
+class GkClient;
+class GkSQLConnection;
 
 namespace Routing {
 
+class VirtualQueue;
+
 /// An entry for a single call destination route
-struct Route {
+class Route {
+public:
 	// a policy can set flags to indicate extra status of a processed request
 	enum Flags {
 		e_toParent = 1,
@@ -53,34 +58,41 @@ struct Route {
 	
 	Route();
 	Route(
-		const endptr &destEndpoint
+		const PString & policyName,
+		const endptr & destEndpoint,
+		unsigned priority = 1
 		);
 	Route(
-		const PString &policyName,
-		const H225_TransportAddress &destAddr
+		const PString & policyName,
+		const H225_TransportAddress & destAddr,
+		unsigned priority = 1
 		);
 	Route(
-		const PString &policyName,
-		const PIPSocket::Address &destIpAddr,
-		WORD destPort
+		const PString & policyName,
+		const PIPSocket::Address & destIpAddr,
+		WORD destPort,
+		unsigned priority = 1
 		);
-		
+
+	bool operator< (const Route & rhs) { return m_priority < rhs.m_priority; }
+
 	PString AsString() const;
 
-	bool IsFailoverActive(
-		unsigned cause
-		) const;
+	bool IsFailoverActive(unsigned cause) const;
+
 
 	H225_TransportAddress m_destAddr; /// destination address for signaling
 	endptr m_destEndpoint; /// destination endpoint record (if available)
-	H225_AliasAddress m_srcAddr; /// source alias that matched the route
 	PString m_policy; /// name of the policy that found the route
 	PString m_routeId; /// optional policy-specific route identifier
 	int m_proxyMode; /// per-route proxy mode flag
 	unsigned m_flags; /// additional route specific flags
-	unsigned char m_rerouteCauses[16]; /// bit flags to trigger rerouting on particular Q931 causes
 	PString m_destNumber; /// rewritten number (corresponds to Toolkit::RewriteE164)
 	PString m_destOutNumber; /// number actually sent to the called party  (corresponds to Toolkit::GWRewriteE164)
+
+protected:
+	unsigned char m_rerouteCauses[16]; /// bit flags to trigger rerouting on particular Q931 causes
+	unsigned m_priority;	/// priority of this route (less is more important)
 };
 
 class RoutingRequest {
@@ -93,21 +105,14 @@ public:
 		e_Reject = 16
 	};
 	
-	// note this is not a polymorphic class
 	RoutingRequest();
-	RoutingRequest(
-		const std::list<Route> &failedRoutes
-		);
-	~RoutingRequest();
+	RoutingRequest(const std::list<Route> &failedRoutes);
+	virtual ~RoutingRequest();
 
-	bool AddRoute(
-		const Route &route
-		);
-	bool GetFirstRoute(
-		Route &route
-		);
+	bool AddRoute(const Route &route);
+	bool GetFirstRoute(Route &route);
 	void RemoveAllRoutes();
-	std::list<Route> &GetRoutes() { return m_routes; }
+	std::list<Route> & GetRoutes() { return m_routes; }
 	
 	void SetRejectReason(unsigned reason) { m_reason = reason; }
 	void SetFlag(unsigned f) { m_flags |= f; }
@@ -115,6 +120,8 @@ public:
 	unsigned GetFlags() const { return m_flags; }
 	void SetSourceIP(const PString & ip) { m_sourceIP = ip; }
 	PString GetSourceIP() const { return m_sourceIP; }
+	void SetCallerID(const PString & id) { m_callerID = id; }
+	PString GetCallerID() const { return m_callerID; }
 
 private:
 	RoutingRequest(const RoutingRequest&);
@@ -126,6 +133,7 @@ private:
 	std::list<Route> m_routes;
 	std::list<Route> m_failedRoutes;
 	PString m_sourceIP;
+	PString m_callerID;
 };
 
 template<class R, class W>
@@ -217,6 +225,25 @@ protected:
 	const char* m_name;
 };
 
+
+// the simplest policy, the destination has been explicitly specified
+class ExplicitPolicy : public Policy {
+public:
+	ExplicitPolicy();
+
+	static void MapDestination(H225_TransportAddress & addr);
+	static void OnReload();
+
+protected:
+	virtual bool OnRequest(AdmissionRequest &);
+	// the policy doesn't apply to LocationRequest
+	virtual bool OnRequest(SetupRequest &);
+	virtual bool OnRequest(FacilityRequest &);
+
+	static std::map<PString, H225_TransportAddress> m_destMap;
+};
+
+
 class AliasesPolicy : public Policy {
 public:
 	AliasesPolicy() { m_name = "Aliases"; }
@@ -256,6 +283,189 @@ private:
 };
 
 
+// the classical policy, find the dstination from the RegistrationTable
+class InternalPolicy : public AliasesPolicy {
+public:
+	InternalPolicy();
+
+protected:
+	virtual bool OnRequest(AdmissionRequest &);
+	virtual bool OnRequest(SetupRequest &);
+
+	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
+	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
+	virtual bool FindByAliases(SetupRequest &, H225_ArrayOf_AliasAddress &);
+	virtual bool FindByAliases(AdmissionRequest &, H225_ArrayOf_AliasAddress &);
+	
+private:
+	bool roundRobin;
+};
+
+
+// a policy to route call to parent
+class ParentPolicy : public Policy {
+public:
+	ParentPolicy();
+
+private:
+	// override from class Policy
+	virtual bool IsActive() const;
+
+	virtual bool OnRequest(AdmissionRequest &);
+	virtual bool OnRequest(LocationRequest &);
+	virtual bool OnRequest(SetupRequest &);
+	virtual bool OnRequest(FacilityRequest &);
+
+	GkClient *m_gkClient;
+};
+
+
+// a policy to look up the destination from DNS
+class DNSPolicy : public AliasesPolicy {
+public:
+	DNSPolicy();
+
+protected:
+	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
+	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
+
+	static bool m_resolveNonLocalLRQs;
+};
+
+// a policy to route call via external program
+class VirtualQueuePolicy : public Policy {
+public:
+	VirtualQueuePolicy();
+
+protected:
+	// override from class Policy
+	virtual bool IsActive() const;
+
+	virtual bool OnRequest(AdmissionRequest &);
+	virtual bool OnRequest(LocationRequest &);
+	virtual bool OnRequest(SetupRequest &);
+
+private:
+	VirtualQueue *m_vqueue;
+};
+
+
+class NumberAnalysisPolicy : public Policy {
+public:
+	struct PrefixEntry {
+		string m_prefix;
+		int m_minLength;
+		int m_maxLength;
+	};
+	
+	NumberAnalysisPolicy();
+
+protected:
+	virtual bool OnRequest(AdmissionRequest &);
+	virtual bool OnRequest(SetupRequest &);
+
+private:
+	NumberAnalysisPolicy(const NumberAnalysisPolicy &);
+	NumberAnalysisPolicy& operator=(const NumberAnalysisPolicy &);
+	
+private:
+	typedef vector<PrefixEntry> Prefixes;
+
+	/// list of number prefixes, with min/max number length as values
+	Prefixes m_prefixes;
+};
+
+// a policy to look up the destination from ENUM Name Server
+class ENUMPolicy : public AliasesPolicy {
+public:
+	ENUMPolicy() { m_name = "ENUM"; }
+protected:
+	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
+	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
+};
+
+class DestinationRoutes {
+public:
+	DestinationRoutes() { m_endChain = false; m_reject = false; m_rejectReason = 0; m_aliasesChanged = false; }
+	~DestinationRoutes() { }
+	
+	bool EndPolicyChain() const { return m_endChain; }
+	bool RejectCall() const { return m_reject; }
+	void SetRejectCall(bool reject) { m_reject = reject; m_endChain = true; }
+	unsigned int GetRejectReason() const { return m_rejectReason; }
+	void SetRejectReason(unsigned int reason) { m_rejectReason = reason; }
+	bool ChangeAliases() const { return m_aliasesChanged; }
+	H225_ArrayOf_AliasAddress GetNewAliases() const { return m_newAliases; }
+	void SetNewAliases(const H225_ArrayOf_AliasAddress & aliases) { m_newAliases = aliases; m_aliasesChanged = true; }
+	
+	void AddRoute(const Route & route, bool endChain = true) { m_routes.push_back(route); m_endChain = endChain; }
+
+	std::list<Route> m_routes;
+
+protected:
+	bool m_endChain;
+	bool m_reject;
+	unsigned int m_rejectReason;
+	bool m_aliasesChanged;
+	H225_ArrayOf_AliasAddress m_newAliases;
+};
+
+// a policy to route calls via an SQL database
+class SqlPolicy : public Policy {
+public:
+	SqlPolicy();
+	virtual ~SqlPolicy();
+
+protected:
+	virtual bool IsActive() const { return m_active; }
+
+	virtual bool OnRequest(AdmissionRequest &);
+	virtual bool OnRequest(LocationRequest &);
+	virtual bool OnRequest(SetupRequest &);
+
+	virtual void DatabaseLookup(
+		/*in */
+		const PString & source,
+		const PString & calledAlias,
+		const PString & calledIP,
+		const PString & caller,
+		const PString & callingStationId,
+		const PString & callid,
+		const PString & messageType,
+		const PString & clientauthid,
+		/* out: */
+		DestinationRoutes & destination);
+
+protected:
+	// active ?
+	bool m_active;
+	// connection to the SQL database
+	GkSQLConnection* m_sqlConn;
+	// parametrized query string for the routing query
+	PString m_query;
+	// query timeout
+	long m_timeout;
+};
+
+// a policy to route all calls to one default endpoint
+class CatchAllPolicy : public Policy {
+public:
+	CatchAllPolicy();
+	virtual ~CatchAllPolicy() { }
+
+protected:
+	virtual bool OnRequest(AdmissionRequest & request) { return CatchAllRoute(request); }
+	virtual bool OnRequest(LocationRequest & request) { return CatchAllRoute(request); }
+	virtual bool OnRequest(SetupRequest & request) { return CatchAllRoute(request); }
+
+	bool CatchAllRoute(RoutingRequest & request) const;
+	
+	PString m_catchAllAlias;
+	PString m_catchAllIP;
+};
+
+
+
 template<class R, class W>
 inline bool Request<R, W>::Process()
 {
@@ -269,7 +479,7 @@ inline bool Request<R, W>::Process()
 	answe the call). Basically, it rewrites virtual queue alias
 	into the alias of the specified agent.
 
-	The route request is uniquelly identified by (EndpointIdentifier,CRV)
+	The route request is uniquely identified by (EndpointIdentifier,CRV)
 	values pair.
 */
 class VirtualQueue
@@ -311,6 +521,8 @@ public:
 		PString* callSigAdr,
 		/// bind IP for BindAndRouteToGateway
 		PString* bindIP,
+		/// caller ID
+		PString* callerID,
 		/// should the call be rejected modified by this function on return)
 		bool & reject,
 		/// an actual virtual queue name (should be present in destinationInfo too)
@@ -321,7 +533,9 @@ public:
 		/// the callID as string
 		const PString& callID,
 		/// the called IP for unregistered calls
-		const PString& calledip = "unknown"
+		const PString& calledip = "unknown",
+		/// vendor string of caller
+		const PString& vendorString = "unknown"
 		);
 
 	/** Make a routing decision for a pending route request (inserted
@@ -334,7 +548,7 @@ public:
 		/// aliases for the routing target (an agent that the call will be routed to)
 		/// that will replace the original destination info
 		const H225_ArrayOf_AliasAddress& agent,
-		/// ip that will replace the destionationCallSignalAddress (RouteToGateway)
+		/// ip that will replace the destinationCallSignalAddress (RouteToGateway)
 		/// used only if set (port != 0)
 		const PString& destinationip,
 		/// identifier of the endpoint associated with the route request
@@ -344,7 +558,9 @@ public:
 		/// callID of the call associated with the route request
 		const PString& callID,
 		// outgoing IP or empty
-		const PString& bindIP = PString::Empty(),
+		const PString& bindIP,
+		// callerID or empty
+		const PString& callerID,
 		/// should this call be rejected
 		bool reject = false
 		);
@@ -368,7 +584,9 @@ public:
 		/// callID of the call associated with the route request
 		const PString& callID,
 		// outgoing IP or empty
-		const PString& bindIP = PString::Empty(),
+		const PString& bindIP,
+		// callerID or empty
+		const PString& callerID,
 		/// should this call be rejected
 		bool reject = false
 		);
@@ -404,11 +622,13 @@ private:
 			const PString& callID,
 			H225_ArrayOf_AliasAddress* agent,
 			PString* callsignaladdr,
-			PString* bindIP
+			PString* bindIP,
+			PString* callerID
 			)
 			:
 			m_callingEpId((const char*)callingEpId), m_crv(crv), m_callID(callID),
-			m_agent(agent), m_callsignaladdr(callsignaladdr), m_sourceIP(bindIP) {}
+			m_agent(agent), m_callsignaladdr(callsignaladdr), m_sourceIP(bindIP),
+			m_callerID(callerID), m_reject(false) {}
 
 		/// identifier for the endpoint associated with this request
 		PString m_callingEpId;
@@ -423,6 +643,8 @@ private:
 		PString* m_callsignaladdr;
 		/// bindIP or empty
 		PString* m_sourceIP;
+		/// callerID or empty
+		PString* m_callerID;
 		/// should this call be rejected
 		bool m_reject;
 		/// a synchronization point for signaling that routing decision
@@ -447,6 +669,8 @@ private:
 		PString* callSigAdr,
 		/// bind IP for BindAndRouteToGateway
 		PString* bindIP,
+		/// caller ID
+		PString* callerID,
 		/// set by the function to true if another route request for the same
 		/// call is pending
 		bool& duplicate
