@@ -2,12 +2,12 @@
 //
 // SoftPBX.cxx
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2011, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Jan Willamowius
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -101,7 +101,7 @@ void SoftPBX::UnregisterAllEndpoints()
 	RegistrationTable::Instance()->ClearTable();
 }
 
-void SoftPBX::UnregisterAlias(PString Alias)
+void SoftPBX::UnregisterAlias(const PString & Alias)
 {
 	H225_ArrayOf_AliasAddress EpAlias;
 	EpAlias.SetSize(1);
@@ -126,7 +126,7 @@ void SoftPBX::UnregisterAlias(PString Alias)
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
-void SoftPBX::UnregisterIp(PString Ip)
+void SoftPBX::UnregisterIp(const PString & Ip)
 {
 	H225_TransportAddress callSignalAddress;
 	GetTransportAddress(Ip, (WORD)GkConfig()->GetInteger("EndpointSignalPort", GK_DEF_ENDPOINT_SIGNAL_PORT), callSignalAddress);
@@ -184,49 +184,71 @@ void SoftPBX::DisconnectCall(unsigned CallNumber)
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
-// send a DRQ to this endpoint
-void SoftPBX::DisconnectIp(PString Ip)
+void SoftPBX::DisconnectCallId(const PString & CallId)
+{
+	PTRACE(3, "GK\tSoftPBX: DisconnectCallId " << CallId);
+
+	callptr Call = CallTable::Instance()->FindCallRec(StringToCallId(CallId));
+	if (!Call) {
+		PString msg = "Can't find call id " + CallId;
+		PTRACE(2, "GK\tSoftPBX: " << msg);
+		GkStatus::Instance()->SignalStatus(msg + "\r\n");
+		return;
+	}
+
+	Call->Disconnect(true);
+	// remove the call directly so we don't have to handle DCF
+	CallTable::Instance()->RemoveCall(Call);
+
+	PString msg = "Call id " + CallId + " disconnected.";
+	PTRACE(2, "GK\tSoftPBX: " << msg);
+	GkStatus::Instance()->SignalStatus(msg + "\r\n");
+}
+
+// disconnect all calls on this IP[:port]
+void SoftPBX::DisconnectIp(const PString & Ip)
 {
 	H225_TransportAddress callSignalAddress;
 	GetTransportAddress(Ip, (WORD)GkConfig()->GetInteger("EndpointSignalPort", GK_DEF_ENDPOINT_SIGNAL_PORT), callSignalAddress);
 	PTRACE(3, "GK\tSoftPBX: DisconnectIp " << Ip);
 
 	callptr Call;
-	if (Ip.Find(':') == P_MAX_INDEX)
-		Call = CallTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress);	// no port specified
-	else
-		Call = CallTable::Instance()->FindBySignalAdr(callSignalAddress);	// port specify, search for it
-	if (!Call) {
-		PString msg = "Can't find call for IP " + Ip;
-		PTRACE(2, "GK\tSoftPBX: " << msg);
-		GkStatus::Instance()->SignalStatus(msg + "\r\n");
-		return;
+	if (Ip.Find(':') == P_MAX_INDEX) {
+		// no port specified
+		while (Call = CallTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress)) {
+			unsigned CallNumber = Call->GetCallNumber();
+			Call->Disconnect(true);
+			CallTable::Instance()->RemoveCall(Call);
+			PString msg(PString::Printf, "Call number %d disconnected.", CallNumber);
+			PTRACE(2, "GK\tSoftPBX: " << msg);
+			GkStatus::Instance()->SignalStatus(msg + "\r\n");
+		}
+	} else {
+		// port specify, search for it
+		while (Call = CallTable::Instance()->FindBySignalAdr(callSignalAddress)) {
+			unsigned CallNumber = Call->GetCallNumber();
+			Call->Disconnect(true);
+			CallTable::Instance()->RemoveCall(Call);
+			PString msg(PString::Printf, "Call number %d disconnected.", CallNumber);
+			PTRACE(2, "GK\tSoftPBX: " << msg);
+			GkStatus::Instance()->SignalStatus(msg + "\r\n");
+		}
 	}
-	unsigned CallNumber = Call->GetCallNumber();
-
-	Call->Disconnect(true);
-	// remove the call directly so we don't have to handle DCF
-	CallTable::Instance()->RemoveCall(Call);
-
-	PString msg(PString::Printf, "Call number %d disconnected.", CallNumber);
-	PTRACE(2, "GK\tSoftPBX: " << msg);
-	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
 // send a DRQ to this endpoint
-void SoftPBX::DisconnectAlias(PString Alias)
+void SoftPBX::DisconnectAlias(const PString & Alias)
 {
 	H225_ArrayOf_AliasAddress EpAlias;
 	EpAlias.SetSize(1);
 	H323SetAliasAddress(Alias, EpAlias[0]);
 	PTRACE(3, "GK\tSoftPBX: DisconnectAlias " << Alias);
 
-	// TODO: extend for unregistered endpoints
 	DisconnectEndpoint(RegistrationTable::Instance()->FindByAliases(EpAlias));
 }
 
 // send a DRQ to this endpoint
-void SoftPBX::DisconnectEndpoint(PString Id)
+void SoftPBX::DisconnectEndpoint(const PString & Id)
 {
 	H225_EndpointIdentifier EpId;	// id of endpoint to be disconnected
 	EpId = Id;
@@ -251,15 +273,11 @@ void SoftPBX::DisconnectEndpoint(const endptr &ep)
 	}
 }
 
-void SoftPBX::SendProceeding(PString CallId)
+void SoftPBX::SendProceeding(const PString & CallId)
 {
 	PTRACE(3, "GK\tSoftPBX: SendProceeding " << CallId);
 
-	H225_CallIdentifier cid;
-	CallId.Replace("-", "", true);
-	CallId.Replace(" ", "", true);
-	OpalGloballyUniqueID tmp_guid(CallId);
-	cid.m_guid = tmp_guid;
+	H225_CallIdentifier cid = StringToCallId(CallId);
  
 	// CallProceeding will be sent during the routing process
 	// at this time the call won't yet be in the CallTable,
@@ -286,67 +304,12 @@ void SoftPBX::SendProceeding(PString CallId)
 	PBYTEArray lBuffer;
 	lForwardedSocket->BuildProceedingPDU(q931, call->GetCallIdentifier(), call->GetCallRef() | 0x8000u);
 	q931.Encode(lBuffer);
-	lForwardedSocket->TransmitData(lBuffer); 
-}
-
-bool SoftPBX::TransferCall(endptr & lSrcForward, SmartPtr<CallRec> lCall, PString DestinationAlias)
-{
-	if (!lCall || !lSrcForward) {
-		PString msg("No call to transfer!");
-		PTRACE(1, "GK\tSoftPBX: " + msg);
-		return false;
-	}
-
-	endptr lCalling, lCalled;
-	lCalling = lCall->GetCallingParty();
-	lCalled = lCall->GetCalledParty();
-	H225_ArrayOf_AliasAddress aliases = lSrcForward->GetAliases();
-	CallSignalSocket *lForwardedSocket = 0;
-	if (lCalling && lCalling->CompareAlias(&aliases)) {
-		lForwardedSocket = lCall->GetCallSignalSocketCalling();
-	} else if (lCalled && lCalled->CompareAlias(&aliases)) {
-		lForwardedSocket = lCall->GetCallSignalSocketCalled();
-	}
-	if (!lForwardedSocket) {
-		PString msg("Can't transfer call in direct mode!");
-		PTRACE(1, "GK\tSoftPBX: " + msg);
-		GkStatus::Instance()->SignalStatus(msg + "\r\n");
-		return false;
-	}
-
-	//Search destination of call forwarding : lDestForward
-	PStringList lBufferAliasArrayString;
-	lBufferAliasArrayString.AppendString(DestinationAlias);
-	H225_ArrayOf_AliasAddress lBufferAliasArray;
-	H323SetAliasAddresses(lBufferAliasArrayString, lBufferAliasArray);
-
-	endptr lDestForward = RegistrationTable::Instance()->FindFirstEndpoint(lBufferAliasArray);
-	lBufferAliasArrayString.RemoveAll();
-	lBufferAliasArray.RemoveAll();
-
-	if (!lDestForward) {
-		PString msg("Transferred destination not found!");
-		PTRACE(1, "GK\tSoftPBX: " + msg);
-		return false;
-	}
-
-	Q931 q931;
-	PBYTEArray lBuffer;
-	lForwardedSocket->BuildFacilityPDU(q931, H225_FacilityReason::e_callForwarded, &DestinationAlias);
-	H225_H323_UserInformation uuie;
-	GetUUIE(q931, uuie);
-	PrintQ931(5, "Send to ", lForwardedSocket->GetName(), &q931, &uuie);
-	q931.Encode(lBuffer);
 	lForwardedSocket->TransmitData(lBuffer);
-
-	PString msg = PString("Call ") + PString(lCall->GetCallNumber()) + " transfer success.";
-	PTRACE(1, "GK\tSoftPBX: " + msg);
- 
-	return true;
-
+	call->SetProceedingSent(true);
 }
 
-void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
+// used by status port (old)
+void SoftPBX::TransferCall(const PString & SourceAlias, const PString & DestinationAlias)
 {
 	PTRACE(3, "GK\tSoftPBX: TransferCall " << SourceAlias << " -> " << DestinationAlias);
 
@@ -358,7 +321,7 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 	PStringList lBufferAliasArrayString;
 	H225_ArrayOf_AliasAddress lBufferAliasArray;
 
-	//Search for the call in CallTable
+	// search for the call in CallTable
 	lBufferAliasArrayString.AppendString(SourceAlias);
 	H323SetAliasAddresses(lBufferAliasArrayString, lBufferAliasArray);
 
@@ -374,7 +337,7 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 	}
 	lCall = CallTable::Instance()->FindCallRec(lSrcForward);
 
-	//Search for the Forwarded CallSignalSocket in lCall ( calling or caller socket ? )
+	// search for the Forwarded CallSignalSocket in lCall ( calling or caller socket ? )
 	if (lCall) {
 		endptr lCalling, lCalled;
 		lCalling = lCall->GetCallingParty();
@@ -398,7 +361,7 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 		return;
 	}
 
-	//Search destination of call forwarding : lDestForward
+	// search destination of call forwarding : lDestForward
 	lBufferAliasArrayString.AppendString(DestinationAlias);
 	H323SetAliasAddresses(lBufferAliasArrayString, lBufferAliasArray);
 
@@ -422,16 +385,97 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 	q931.Encode(lBuffer);
 	lForwardedSocket->TransmitData(lBuffer);
 
-	PString msg = PString("Call ") + PString(lCall->GetCallNumber()) + " transferred from" + SourceAlias + " to " + DestinationAlias;
+	PString msg = PString("Call ") + PString(lCall->GetCallNumber()) + " transferred from " + SourceAlias + " to " + DestinationAlias;
 	PTRACE(1, "GK\tSoftPBX: " + msg);
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
-void SoftPBX::MakeCall(PString SourceAlias, PString DestinationAlias)
+void SoftPBX::RerouteCall(const PString & CallId, const PCaselessString & whichLeg, const PString & destination)
+{
+	PTRACE(1, "GK\tSoftPBX: RerouteCall " << CallId << " (" << whichLeg << ") -> " << destination);
+
+	CallLeg which = (whichLeg == "called") ? Called : Caller;
+	callptr lCall = CallTable::Instance()->FindCallRec(StringToCallId(CallId));
+	CallSignalSocket * lForwardedSocket = NULL;
+
+	if (lCall) {
+		if (which == Called) {
+			lForwardedSocket = lCall->GetCallSignalSocketCalled();
+		} else {
+			lForwardedSocket = lCall->GetCallSignalSocketCalling();
+		}
+		if (lForwardedSocket) {
+			if (!lForwardedSocket->RerouteCall(which, destination, false)) {
+				PString msg("SoftPBX: Reroute failed");
+				PTRACE(1, "GK\t" + msg);
+				GkStatus::Instance()->SignalStatus(msg + "\r\n");
+				return;
+			}
+		} else {
+			PString msg("SoftPBX: can't find signalling socket (direct mode ?)");
+			PTRACE(1, "GK\t" + msg);
+			GkStatus::Instance()->SignalStatus(msg + "\r\n");
+			return;
+		}
+	} else {
+		PString msg("SoftPBX: no call to reroute!");
+		PTRACE(1, "GK\t" + msg);
+		GkStatus::Instance()->SignalStatus(msg + "\r\n");
+		return;
+	}
+
+	PString msg = PString("Call ") + PString(lCall->GetCallNumber()) + PString(" / ") + whichLeg + PString(" rerouted to ") + destination;
+	PTRACE(1, "GK\tSoftPBX: " + msg);
+	GkStatus::Instance()->SignalStatus(msg + "\r\n");
+}
+
+// used by status port (new)
+void SoftPBX::TransferCall(const PString & CallId, const PCaselessString & which, const PString & Destination, const PString & method)
+{
+	PTRACE(1, "GK\tSoftPBX: TransferCall " << CallId << " (" << which << ") -> " << Destination << " using " << method);
+ 
+	callptr lCall = CallTable::Instance()->FindCallRec(StringToCallId(CallId));
+	CallSignalSocket *lForwardedSocket = NULL;
+ 
+	if (lCall) {
+		if (which == "called") {
+			lForwardedSocket = lCall->GetCallSignalSocketCalled();
+		} else {
+			lForwardedSocket = lCall->GetCallSignalSocketCalling();
+		}
+		if (!lForwardedSocket) {
+			PString msg("SoftPBX: can't find signalling socket (direct mode ?)");
+			PTRACE(1, "GK\t" + msg);
+			GkStatus::Instance()->SignalStatus(msg + "\r\n");
+			return;
+		}
+	} else {
+		PString msg("SoftPBX: no call to transfer!");
+		PTRACE(1, "GK\t" + msg);
+		GkStatus::Instance()->SignalStatus(msg + "\r\n");
+		return;
+	}
+ 
+	Q931 q931;
+	PBYTEArray lBuffer;
+	if (method == "FacilityRouteCallToMC") {
+		lForwardedSocket->BuildFacilityPDU(q931, H225_FacilityReason::e_routeCallToMC, &Destination);
+	} else {
+		lForwardedSocket->BuildFacilityPDU(q931, H225_FacilityReason::e_callForwarded, &Destination);
+	}
+	q931.Encode(lBuffer);
+	lForwardedSocket->TransmitData(lBuffer);
+ 
+	PString msg = PString("SoftPBX: call ") + CallId + " transferred from " + which + " to " + Destination;
+	PTRACE(1, "GK\t" + msg);
+	GkStatus::Instance()->SignalStatus(msg + "\r\n");
+}
+
+void SoftPBX::MakeCall(const PString & SourceAlias, const PString & DestinationAlias)
 {
 	PTRACE(3, "GK\tSoftPBX: MakeCall " << SourceAlias << " -> " << DestinationAlias);
 	if (! MakeCallEndPoint::Instance()->IsRegisteredWithGk()) {
-		PProcess::Sleep(500);	// give pseudo-endpoint 0.5 sec to register
+		PThread::Sleep(500);	// give pseudo-endpoint 0.5 sec to register
 	}
 	if (MakeCallEndPoint::Instance()->IsRegisteredWithGk()) {
 		MakeCallEndPoint::Instance()->ThirdPartyMakeCall(SourceAlias, DestinationAlias);
@@ -441,7 +485,7 @@ void SoftPBX::MakeCall(PString SourceAlias, PString DestinationAlias)
 	}
 }
 
-void SoftPBX::PrintPrefixCapacities(USocket *client, PString alias)
+void SoftPBX::PrintPrefixCapacities(USocket *client, const PString & alias)
 {
 	PTRACE(3, "GK\tSoftPBX: PrintPrefixCapacities(" << alias << ")");
 	RegistrationTable::Instance()->PrintPrefixCapacities(client, alias);
@@ -455,6 +499,12 @@ void SoftPBX::PrintCapacityControlRules(USocket *client)
 	client->TransmitData(msg);
 }
 
+void SoftPBX::PrintEndpointQoS(USocket *client)
+{
+	PTRACE(3, "GK\tSoftPBX: PrintEndpointQoS");
+	RegistrationTable::Instance()->PrintEndpointQoS(client);
+}
+
 PString SoftPBX::Uptime()
 {
 	long total = (PTime() - SoftPBX::StartUp).GetSeconds();
