@@ -2,17 +2,17 @@
 //
 // SoftPBX.h
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2011, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Jan Willamowius
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 #ifndef SOFTPBX_H
-#define SOFTPBX_H "@(#) $Id: SoftPBX.h,v 1.22 2009/08/19 09:53:27 willamowius Exp $"
+#define SOFTPBX_H "@(#) $Id: SoftPBX.h,v 1.27 2011/02/11 09:18:09 willamowius Exp $"
 
 // nothing to include :)
 
@@ -34,20 +34,23 @@ namespace SoftPBX
 	void PrintStatistics(USocket *client, bool verbose=false);
 	void ResetCallCounters(USocket *client);
 	void UnregisterAllEndpoints();
-	void UnregisterAlias(PString Alias);
-	void UnregisterIp(PString Ip);
+	void UnregisterAlias(const PString & Alias);
+	void UnregisterIp(const PString & Ip);
 	void DisconnectAll();
 	void DisconnectCall(unsigned CallNumber);
-	void DisconnectIp(PString Ip);
-	void DisconnectAlias(PString Alias);
-	void DisconnectEndpoint(PString Id);
+	void DisconnectCallId(const PString & CallId);
+	void DisconnectIp(const PString & Ip);
+	void DisconnectAlias(const PString & Alias);
+	void DisconnectEndpoint(const PString & Id);
 	void DisconnectEndpoint(const endptr &);
-	void SendProceeding(PString CallId);
-	void TransferCall(PString SourceAlias, PString DestinationAlias);
-	bool TransferCall(endptr & lSrcForward, SmartPtr<CallRec> lCall, PString DestinationAlias);
-	void MakeCall(PString SourceAlias, PString DestinationAlias);
-	void PrintPrefixCapacities(USocket *client, PString alias);
+	void SendProceeding(const PString & CallId);
+	void TransferCall(const PString & SourceAlias, const PString & DestinationAlias);
+	void TransferCall(const PString & CallId, const PCaselessString & which, const PString & Destination, const PString & method);
+	void MakeCall(const PString & SourceAlias, const PString & DestinationAlias);
+	void RerouteCall(const PString & CallId, const PCaselessString & whichLeg, const PString & destination);
+	void PrintPrefixCapacities(USocket *client, const PString & alias);
 	void PrintCapacityControlRules(USocket *client);
+	void PrintEndpointQoS(USocket *client);
 	
 	PString Uptime();
 
