@@ -2,10 +2,12 @@
 //
 // Toolkit base class for the GnuGk
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -31,9 +33,9 @@
 #include "gk_const.h"
 
 #if H323_H350
-  const char *H350Section = "GkH350::Settings";
-  #include <ptclib/pldap.h>
-  #include "h350/h350.h"
+const char *H350Section = "GkH350::Settings";
+#include <ptclib/pldap.h>
+#include "h350/h350.h"
 #endif
 
 using namespace std;
@@ -181,7 +183,7 @@ PString NetworkAddress::AsString() const
 
 bool NetworkAddress::IsAny() const
 {
-	return const_cast<NetworkAddress*>(this)->m_address.IsAny();
+	return m_address.IsAny() && (GetNetmaskLen() == 0);
 }
 
 bool NetworkAddress::operator<(const NetworkAddress &addr) const
@@ -259,29 +261,38 @@ void Toolkit::RouteTable::InitTable()
 	if (!CreateTable())
 		return;
 
-	// Get all the interface addresses available
+	// get Home IPs (all detected IPs or set through config file)
 	std::vector<PIPSocket::Address> home;
-	PString foo = Toolkit::Instance()->GetGKHome(home);
-	if (!home.empty() && home.size() > 1) {
-		// Set default IP to Bind IP, if given.
-		PString bind = GkConfig()->GetString("Bind", "");
-		if (!bind) {
-			defAddr = bind;
-		} else {
-			defAddr = home[0];
-		}
+	Toolkit::Instance()->GetGKHome(home);
+	// if we only have 1 Home IP, then thats also the default IP
+	if (home.size() == 1) {
+		defAddr = home[0];
+	}
+	// Bind= always sets the default IP
+	PString bind = GkConfig()->GetString("Bind", "");
+	if (!bind.IsEmpty()) {
+		defAddr = bind;
 	}
 
-	// If we do not already have a valid entry, try and retrieve the default interface
+	// if we do not already have a valid entry, try and retrieve the default interface
 	if (defAddr.IsLoopback() || !defAddr.IsValid()) {
 		// Set default IP according to route table
 		PIPSocket::Address defGW;
 		PIPSocket::GetGatewayAddress(defGW);
-		if (defGW.AsString() == "0.0.0.0")
+		if (defGW.AsString() == "0.0.0.0") {
+			// no default gateway, use first interface as default
 			PIPSocket::GetNetworkInterface(defAddr);
-		else
+		} else {
+			// use the default gateway
 			defAddr = GetLocalAddress(defGW);
+		}
     }
+	// if we have a list of Home IPs and the default address is not in it, use the first Home IP,
+	// unless the default IP was explicitely specified in Bind=
+	if (bind.IsEmpty() &&
+		!home.empty() && (find(home.begin(), home.end(), defAddr) == home.end())) {
+		defAddr = home[0];
+	} 
 
 #if PTRACING
 	for (RouteEntry *entry = rtable_begin; entry != rtable_end; ++entry) {
@@ -381,13 +392,15 @@ bool Toolkit::RouteTable::CreateRouteTable(const PString & extroute)
 	int i = (!extroute) ? r_table.GetSize()+1 : r_table.GetSize();
 
 	rtable_end = rtable_begin = static_cast<RouteEntry *>(::malloc(i * sizeof(RouteEntry)));
-	for (PINDEX r = 0; r < i ; ++r) {
+	for (PINDEX r = 0; r < i; ++r) {
 		if (!extroute && (r==r_table.GetSize())) {
 			::new (rtable_end++) RouteEntry(extroute);
 		} else {
-		  PIPSocket::RouteEntry & r_entry = r_table[r];
-		  if (r_entry.GetNetMask() != INADDR_ANY) 
-			::new (rtable_end++) RouteEntry(r_entry, if_table);
+			PIPSocket::RouteEntry & r_entry = r_table[r];
+			//if (r_entry.GetNetMask() != INADDR_ANY && r_entry.GetNetwork().GetVersion() == 4 && r_entry.GetDestination().GetVersion() == 4) {
+			if (r_entry.GetNetMask() != INADDR_ANY && r_entry.GetNetwork().GetVersion() != 6) {
+				::new (rtable_end++) RouteEntry(r_entry, if_table);
+			}
 		}
 	}
 
@@ -397,12 +410,12 @@ bool Toolkit::RouteTable::CreateRouteTable(const PString & extroute)
 bool Toolkit::VirtualRouteTable::CreateTable()
 {
 	PString nets = GkConfig()->GetString("NetworkInterfaces", "");
-	if (!nets) {		
+	if (!nets) {
 		PStringArray networks(nets.Tokenise(" ,;\t", FALSE));
 		int i = networks.GetSize();
 		if (i > 0) {
 			rtable_end = rtable_begin = static_cast<RouteEntry *>(::malloc(i * sizeof(RouteEntry)));
-			for (PINDEX r = 0; r < i ; ++r) {
+			for (PINDEX r = 0; r < i; ++r) {
 				::new (rtable_end++) RouteEntry(networks[r]);
 			}
 		}
@@ -418,7 +431,7 @@ bool Toolkit::VirtualRouteTable::CreateTable()
 	ex.GetIpAddress(ext);
 	if (ext.IsValid()) {
 		ExtIP = extip;
-		PString extroute = PString();
+		PString extroute;
 		if (!DynExtIP) 
 		  extroute = ext.AsString() + "/0";
 
@@ -439,7 +452,7 @@ bool Toolkit::VirtualRouteTable::IsMasquerade(PIPSocket::Address & addr) const
 	  return true;
 	}
 
-	return false; 
+	return false;
 }
 
 // class Toolkit::ProxyCriterion
@@ -497,6 +510,10 @@ void Toolkit::ProxyCriterion::LoadConfig(PConfig *config)
 
 	// read [ModeSelection] section
 	PStringToString mode_rules(config->GetAllKeyValues("ModeSelection"));
+	if (mode_rules.GetSize() > 0) {
+		// if we have ModeSelection rules, only use those, don't try to merge them with detected
+		m_modeselection.clear();
+	}
 	for (PINDEX i = 0; i < mode_rules.GetSize(); ++i) {
 		PString network = mode_rules.GetKeyAt(i);
 		if (!network.IsEmpty()) {
@@ -508,16 +525,6 @@ void Toolkit::ProxyCriterion::LoadConfig(PConfig *config)
 				netmode.insideNetwork = netmode.fromExternal;
 				if (modes.GetSize() == 2)
 					netmode.insideNetwork = ToRoutingMode(modes[1].Trim());
-				// check if we have one or more (implied) rules for internal networks that we have to delete first
-				std::map<NetworkAddress, NetworkModes>::iterator iter = m_modeselection.begin();
-				while (iter != m_modeselection.end()) {
-					if (iter->first << addr) {
-						PTRACE(2, "Removing (implied) ModeSelection rule for " << iter->first.AsString());
-						m_modeselection.erase(iter++);
-					} else {
-						iter++;
-					}
-				}
 				m_modeselection[addr] = netmode;
 				PTRACE(2, "GK\tModeSelection rule: " << addr.AsString() << "=" << netmode.fromExternal << "," << netmode.insideNetwork);
 			} else {
@@ -561,21 +568,23 @@ int Toolkit::ProxyCriterion::SelectRoutingMode(const Address & ip1, const Addres
 	int mode = m_enable ? CallRec::Proxied : CallRec::SignalRouted;
 	if (mode == CallRec::SignalRouted && RasServer::Instance()->IsH245Routed())
 		mode = CallRec::H245Routed;
+	PTRACE(5, "ModeSelection for " << ip1.AsString() << " -> " << ip2.AsString() << " default=" << mode);
 
 	// check if we have a more specific setting
+	// TODO: add invalid state to NetworkAddress so we can have ModeSelection rules for 0.0.0.0/0 that set a global default
 	NetworkAddress bestMatchIP1 = FindModeRule(ip1);
 	NetworkAddress bestMatchIP2 = FindModeRule(ip2);
 
 	std::map<NetworkAddress, NetworkModes>::const_iterator iter;
 	// check for same network
-	// TODO: add invalid state to NetworkAddress so we can have ModeSlection rules for 0.0.0.0/0 that set a global default
 	if (!bestMatchIP1.IsAny() && !bestMatchIP2.IsAny()) {
 		// rules for both IPs
-		if (bestMatchIP1.Compare(bestMatchIP2) == 0) {	// operator== broken ?
+		if (bestMatchIP1.Compare(bestMatchIP2) == 0) {
 			// both on same network
 			iter = m_modeselection.find(bestMatchIP1);
 			if (iter != m_modeselection.end()) {
 				mode = iter->second.insideNetwork;
+				PTRACE(5, "ModeSelection: Both IPs on same network: mode=" << mode);
 			}
 		} else {
 			// on different networks, use maximum poxying
@@ -584,15 +593,24 @@ int Toolkit::ProxyCriterion::SelectRoutingMode(const Address & ip1, const Addres
 			iter = m_modeselection.find(bestMatchIP2);
 			int mode2 = iter->second.fromExternal;	// no check, must exist
 			mode = max(mode1, mode2);
+			PTRACE(5, "ModeSelection: Both IPs on different networks: mode1=" << mode1 << " mode2=" << mode2 << " => " << mode);
 		}
 	} else {
 		// only one rule, use that
-		iter = m_modeselection.find(bestMatchIP1);
-		if (iter != m_modeselection.end())
-			mode = iter->second.fromExternal;
-		iter = m_modeselection.find(bestMatchIP2);
-		if (iter != m_modeselection.end())
-			mode = iter->second.fromExternal;
+		if (!bestMatchIP1.IsAny()) {
+			iter = m_modeselection.find(bestMatchIP1);
+			if (iter != m_modeselection.end()) {
+				mode = iter->second.fromExternal;
+				PTRACE(5, "ModeSelection: Only rule for IP 1 = " << ip1.AsString() << " mode=" << mode);
+			}
+		}
+		if (!bestMatchIP2.IsAny()) {
+			iter = m_modeselection.find(bestMatchIP2);
+			if (iter != m_modeselection.end()) {
+				mode = iter->second.fromExternal;
+				PTRACE(5, "ModeSelection: Only rule for IP 2 = " << ip2.AsString() << " mode=" << mode);
+			}
+		}
 	}
 
 	return mode;
@@ -649,7 +667,7 @@ void Toolkit::RewriteData::AddSection(PConfig *config, const PString & section)
 			std::map<PString, PString, pstr_prefix_lesser>::iterator iter = rules.begin();
 			
 			// reverse the order
-			for (int i = m_size; i-- > 0 ; ++iter) {
+			for (int i = m_size; i-- > 0; ++iter) {
 //				m_RewriteKey[i] = iter->first;
 				::new(m_RewriteKey + i) PString(iter->first);
 //				m_RewriteValue[i] = iter->second;
@@ -667,16 +685,13 @@ Toolkit::RewriteData::RewriteData(PConfig *config, const PString & section)
 
 Toolkit::RewriteData::~RewriteData()
 {
-//	delete [] m_RewriteKey;
 	if (m_RewriteKey)
 		for (int i = 0; i < m_size * 2; i++)
 			(m_RewriteKey+i)->~PString();
 	delete[] ((BYTE*)m_RewriteKey);
 }
 
-void Toolkit::RewriteTool::LoadConfig(
-	PConfig *config
-	)
+void Toolkit::RewriteTool::LoadConfig(PConfig *config)
 {
 	m_RewriteFastmatch = config->GetString(RewriteSection, "Fastmatch", "");
 	m_TrailingChar = config->GetString("RasSrv::ARQFeatures", "RemoveTrailingChar", " ")[0];
@@ -775,20 +790,18 @@ bool Toolkit::RewriteTool::RewritePString(PString & s) const
 
 static const char *GWRewriteSection = "RasSrv::GWRewriteE164";
 
-Toolkit::GWRewriteTool::~GWRewriteTool() {
+Toolkit::GWRewriteTool::~GWRewriteTool()
+{
 	for (PINDEX i = 0; i < m_GWRewrite.GetSize(); ++i) {
 		delete &(m_GWRewrite.GetDataAt(i));
 	}
 	m_GWRewrite.RemoveAll();
 }
 
-bool Toolkit::GWRewriteTool::RewritePString(PString gw, bool direction, PString &data) {
-
-	GWRewriteEntry *gw_entry;
-	PString key, value;
-
+bool Toolkit::GWRewriteTool::RewritePString(const PString & gw, bool direction, PString & data)
+{
 	// First lookup the GW in the dictionary
-	gw_entry = m_GWRewrite.GetAt(gw);
+	GWRewriteEntry * gw_entry = m_GWRewrite.GetAt(gw);
 
 	if (gw_entry == NULL)
 		return false;
@@ -798,8 +811,8 @@ bool Toolkit::GWRewriteTool::RewritePString(PString gw, bool direction, PString
 	std::vector<pair<PString,PString> >::iterator end_iterator = direction
 		? gw_entry->m_entry_data.first.end() : gw_entry->m_entry_data.second.end();
 
+	PString key, value;
 	for (; rule_iterator != end_iterator; ++rule_iterator) {
-	
 		key = (*rule_iterator).first;
 			
 		const int len = MatchPrefix(data, key);
@@ -824,8 +837,8 @@ bool Toolkit::GWRewriteTool::RewritePString(PString gw, bool direction, PString
 	return false;
 }
 
-void Toolkit::GWRewriteTool::PrintData() {
-
+void Toolkit::GWRewriteTool::PrintData()
+{
 	std::vector<pair<PString,PString> >::iterator rule_iterator;
 
 	PTRACE(2, "GK\tLoaded per GW rewrite data:");
@@ -959,7 +972,10 @@ Toolkit::RouteTable *Toolkit::GetRouteTable(bool real)
 PConfig* Toolkit::Config()
 {
 	// Make sure the config would not be called before SetConfig
-	PAssert(!m_ConfigDefaultSection, "Error: Call Config() before SetConfig()!");
+	if (m_ConfigDefaultSection.IsEmpty()) {
+		PTRACE(0, "Error: Call Config() before SetConfig()!");
+		return NULL;
+	}
 	return (m_Config == NULL) ? ReloadConfig() : m_Config;
 }
 
@@ -1028,7 +1044,7 @@ PString Toolkit::GetTempDir() const
 			
 		// check if the directory exists and is accessible (access rights)
 		if (!(PFile::Exists(tmpdir) && PFile::Access(tmpdir, PFile::ReadWrite)))
-			tmpdir = PString();
+			tmpdir = PString::Empty();
 	}
 	
 	return tmpdir;
@@ -1061,13 +1077,12 @@ void Toolkit::CreateConfig()
 	} while (PFile::Exists(m_tmpconfig));
 
 #ifdef _WIN32
-	// Does WIN32 support symlink?
 	if (PFile::Copy(m_ConfigFilePath, m_tmpconfig)) {
 #else
 	if (symlink(m_ConfigFilePath, m_tmpconfig) == 0) {
 #endif
 		PConfig * testConfig = new PConfig(m_tmpconfig, m_ConfigDefaultSection);
-		if( (testConfig->GetInteger("Fortytwo") == 42) || (testConfig->GetInteger("Fourtytwo") == 42)) {
+		if (testConfig->GetSections().GetSize() > 0) {
 			delete m_Config;
 			m_Config = testConfig;
 		} else {
@@ -1208,7 +1223,7 @@ void Toolkit::ReloadSQLConfig()
 		queryResult = NULL;
 	}
 
-// Rewrite Alias Query
+	// Rewrite Alias Query
 	query = m_Config->GetString("SQLConfig", "RewriteAliasQuery", "");
 	if (!query.IsEmpty()) {
 		PTRACE(4, "SQLCONF\tLoading rewrite rules from SQL database");
@@ -1435,7 +1450,7 @@ void Toolkit::PrepareReloadConfig()
 {
 	if (!m_ConfigDirty)
 		CreateConfig();
-	else // the config have been changed via status port, use it directly
+	else // the config has been changed via status port, use it directly
 		m_ConfigDirty = false;
 }
 
@@ -1449,6 +1464,12 @@ PConfig* Toolkit::ReloadConfig()
 	// read the gatekeeper name from the config file, because it might be uased as a key into the SQL config
 	m_GKName = Config()->GetString("Name", "OpenH323GK");
 
+
+	// set the max size of an array in an ASN encoded message (eg. max length of alias list)
+	PINDEX maxArraySize = GkConfig()->GetInteger("MaxASNArraySize", 0);
+	if (maxArraySize > 0)
+		PASN_Object::SetMaximumArraySize(maxArraySize);
+
 	m_encryptAllPasswords = Toolkit::AsBool(
 		Config()->GetString("EncryptAllPasswords", "0")
 		);
@@ -1465,13 +1486,17 @@ PConfig* Toolkit::ReloadConfig()
 	m_H46018Enabled	= m_Config->GetBoolean(RoutedSec, "EnableH46018", 0);
 #endif
 #ifdef HAS_H46023
-	m_H46023Enabled	= (m_Config->GetBoolean(RoutedSec, "EnableH46023", 0) && 
+	m_H46023Enabled	= (m_Config->GetBoolean(RoutedSec, "EnableH46023", 0) &&
 						!m_Config->GetString(RoutedSec, "H46023STUN",""));
 #endif
 
 	// TODO/BUG: always call SetGKHome() on reload, even if we don't have a Home= setting
 	// otherwise we won't detect new IPs on the machine
 	PString GKHome(m_Config->GetString("Home", ""));
+	if (GKHome == "0.0.0.0") {
+		PTRACE(1, "Config error: Invalid Home setting (0.0.0.0), ignoring");
+		GKHome = "";
+	}
 	if (m_GKHome.empty() || !GKHome)
 		SetGKHome(GKHome.Tokenise(",:;", false));
 	
@@ -1479,7 +1504,8 @@ PConfig* Toolkit::ReloadConfig()
 	m_VirtualRouteTable.InitTable();
 	m_ProxyCriterion.LoadConfig(m_Config);
 #ifdef HAS_H46023
-	if (m_H46023Enabled)  LoadH46023STUN();
+	if (m_H46023Enabled)
+		LoadH46023STUN();
 #endif
 #ifdef HAS_H460P
 	m_presence.LoadConfig(m_Config);
@@ -1556,9 +1582,11 @@ void Toolkit::LoadReasonMap(PConfig *cfg)
 
 }
 
-bool Toolkit::MatchRegex(const PString &str, const PString &regexStr)
+bool Toolkit::MatchRegex(const PString & str, const PString & regexStr)
 {
-	PINDEX pos=0;
+	if (regexStr.IsEmpty())
+		return false;	// nothing matches an empty regex and it triggers a PTLib assertion
+	PINDEX pos = 0;
 	PRegularExpression regex(regexStr, PRegularExpression::Extended);
 	if(regex.GetErrorCode() != PRegularExpression::NoError) {
 		PTRACE(2, "Errornous '"<< regex.GetErrorText() <<"' compiling regex: " << regexStr);
@@ -1573,7 +1601,7 @@ bool Toolkit::MatchRegex(const PString &str, const PString &regexStr)
 
 #if HAS_DATABASE
 Toolkit::AssignedAliases::AssignedAliases()
-  : m_sqlactive(false), m_sqlConn(NULL)
+  : m_sqlactive(false), m_sqlConn(NULL), m_timeout(-1)
 {
 }
 
@@ -1630,7 +1658,7 @@ bool Toolkit::AssignedAliases::DatabaseLookup(
 		PStringArray & newAliases	
         )
 {
-	if (!m_sqlactive) 
+	if (!m_sqlactive)
 		return false;
 
 	GkSQLResult::ResultRow resultRow;
@@ -1664,7 +1692,7 @@ bool Toolkit::AssignedAliases::DatabaseLookup(
 			if (retval[0].IsEmpty()) {
 				PTRACE(1, "AliasSQL\tQuery Invalid value found.");
 				continue;
-			} 
+			}
 		    if (!success) success = true;
 		    PTRACE(5, "AliasSQL\tQuery result: " << retval[0]);
 		    newAliases.AppendString(retval[0]);
@@ -1683,14 +1711,14 @@ void Toolkit::AssignedAliases::LoadConfig(PConfig * m_config)
 #if HAS_DATABASE
   if (LoadSQL(m_config)) {
 	  m_sqlactive = true;
-  } else 
+  } else
 #endif
   {
    	const PStringToString kv = m_config->GetAllKeyValues(AssignedAliasSection);
 	for (PINDEX i=0; i < kv.GetSize(); i++) {
 		PString data = kv.GetDataAt(i);
 		PStringArray datalines = data.Tokenise(" ,;\t");
-		for (PINDEX j=0; j < datalines.GetSize(); j++) 
+		for (PINDEX j=0; j < datalines.GetSize(); j++)
 		   gkAssignedAliases.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),datalines[j]));
 	}
   }
@@ -1713,10 +1741,22 @@ bool Toolkit::CreateH350Session(H350_Session * session)
 		authMethod = PLDAPSession::AuthSASL;
 	else if (mode == "kerberos")
 		authMethod = PLDAPSession::AuthKerberos;
+	
+	bool startTLS = Toolkit::AsBool(GkConfig()->GetString(H350Section, "StartTLS", "0"));
 
 	if (!session->Open(server)) {
-	   PTRACE(4,"H350\tCannot locate H.350 Server");
-	   return false;
+		PTRACE(1,"H350\tCannot locate H.350 Server");
+		return false;
+	}
+	if (startTLS) {
+#ifdef hasLDAPStartTLS
+		if (!session->StartTLS()) {
+			PTRACE(1,"H350\tStartTLS failed");
+			return false;
+		}
+#else
+		PTRACE(1, "H350\tError: LDAP StartTLS not supported in this version");
+#endif
 	}
 	if (!user.IsEmpty())
 	    session->Bind(user,password,authMethod);
@@ -1726,11 +1766,11 @@ bool Toolkit::CreateH350Session(H350_Session * session)
 
 bool Toolkit::AssignedAliases::QueryH350Directory(const PString & alias, PStringArray & aliases)
 {
-// Support Assigned Aliases
-    if (!GkConfig()->GetString(H350Section, "AssignedAliases", "0"))
-		   return false;
+	// support Assigned Aliases
+    if (!Toolkit::AsBool(GkConfig()->GetString(H350Section, "AssignedAliases", "0")))
+		return false;
 
-// Search the Directory
+	// search the directory
 	PString search = GkConfig()->GetString(H350Section, "SearchBaseDN", "");
 
 	H225_AliasAddress aliasaddress;
@@ -1740,57 +1780,61 @@ bool Toolkit::AssignedAliases::QueryH350Directory(const PString & alias, PString
 	switch (aliasaddress.GetTag()) {
 	  case H225_AliasAddress::e_dialedDigits:
             filter = "h323IdentitydialedDigits=" + alias;
+            break;
 	  case H225_AliasAddress::e_h323_ID:
             filter = "h323Identityh323-ID=" + alias;
+            break;
 	  case H225_AliasAddress::e_url_ID:
 		    filter = "h323IdentityURL-ID=" + alias;
+		    break;
 	  default:
+		  PTRACE(4, "H350\tAssigned Alias: unhandled alias type " << aliasaddress.GetTagName());
 		  return false;
 	}
 
 	H350_Session session;
 	if (!Toolkit::Instance()->CreateH350Session(&session)) {
-	   PTRACE(4,"H350\tAssigned Alias: Could not connect to Server.");
+	   PTRACE(1, "H350\tAssigned Alias: Could not connect to directory server");
 	   return false;
 	}
 
 	H350_Session::LDAP_RecordList rec;
 	int count = session.Search(search,filter,rec);
 	if (count <= 0) {
-	   PTRACE(4,"H350\tAssigned Alias: No Record Found");
+	   PTRACE(4, "H350\tAssigned Alias: No Record Found");
 	   session.Close();
 	   return false;
 	}
 
-// Locate the record
+	// locate the record
 	for (H350_Session::LDAP_RecordList::const_iterator x = rec.begin(); x != rec.end(); ++x) {			
-       H350_Session::LDAP_Record entry = x->second;
-	   PString al;
-	   PINDEX i;
-       if (session.GetAttribute(entry,"h323Identityh323-ID",al)) {
-		   	PStringList als = al.Lines();
+		H350_Session::LDAP_Record entry = x->second;
+		PString al;
+		PINDEX i;
+		if (session.GetAttribute(entry,"h323Identityh323-ID", al)) {
+			PStringList als = al.Lines();
 			for (i=0; i< als.GetSize(); i++)
 				aliases.AppendString(als[i]);
-	   }
-       if (session.GetAttribute(entry,"h323IdentitydialedDigits",al)) {
-		   	PStringList als = al.Lines();
+		}
+		if (session.GetAttribute(entry,"h323IdentitydialedDigits", al)) {
+			PStringList als = al.Lines();
 			for (i=0; i< als.GetSize(); i++)
 				aliases.AppendString(als[i]);
-	   }
-       if (session.GetAttribute(entry,"h323IdentityURL-ID",al)) {
-		   	PStringList als = al.Lines();
+		}
+		if (session.GetAttribute(entry,"h323IdentityURL-ID", al)) {
+			PStringList als = al.Lines();
 			for (i=0; i< als.GetSize(); i++)
 				aliases.AppendString(als[i]);
-	   }
-	   session.Close();
-	   if (aliases.GetSize() > 0) {
-		   PTRACE(2,"H350\tAssigned Alias: Located " << aliases.GetSize() << " Aliases.");
-		   session.Close();
-		   return true;
-	   }
+		}
+		session.Close();
+		if (aliases.GetSize() > 0) {
+			PTRACE(2, "H350\tAssigned Alias: Located " << aliases.GetSize() << " aliases");
+			session.Close();
+			return true;
+		}
 	}
 
-	PTRACE(4,"H350\tAssigned Alias: No valid Assigned GK found.");
+	PTRACE(4, "H350\tAssigned Alias: No valid Assigned Alias found");
 	session.Close();
 	return false;
 }
@@ -1803,7 +1847,7 @@ bool Toolkit::AssignedAliases::QueryAssignedAliases(const PString & alias, PStri
 		return true;
 #endif
 #ifdef H323_H350
-	if (QueryH350Directory(alias,aliases))
+	if (QueryH350Directory(alias, aliases))
 		return true;
 #endif
 
@@ -1826,7 +1870,7 @@ bool Toolkit::AssignedAliases::GetAliases(const H225_ArrayOf_AliasAddress & alia
 	if (!found) {
 		for (PINDEX i=0; i < alias.GetSize(); i++) {
 		  PString search = H323GetAliasAddressString(alias[i]);
-		  
+
 		  for (unsigned j=0; j < gkAssignedAliases.size(); j++) {
              PTRACE(5,"Alias\tCompare " << gkAssignedAliases[j].first << " to " << search);
 			  if (gkAssignedAliases[j].first == search) {
@@ -1926,7 +1970,7 @@ bool Toolkit::AssignedGatekeepers::DatabaseLookup(
 		PStringArray & newGks	
         )
 {
-	if (!m_sqlactive) 
+	if (!m_sqlactive)
 		return false;
 
 	GkSQLResult::ResultRow resultRow;
@@ -1961,7 +2005,7 @@ bool Toolkit::AssignedGatekeepers::DatabaseLookup(
 			if (retval[0].IsEmpty()) {
 				PTRACE(1, "AssignSQL\tQuery Invalid value found.");
 				continue;
-			} 
+			}
 		    if (!success) success = true;
 		    PTRACE(5, "AssignSQL\tQuery result: " << retval[0]);
 		    newGks.AppendString(retval[0]);
@@ -1980,24 +2024,23 @@ void Toolkit::AssignedGatekeepers::LoadConfig(PConfig * m_config)
 #if HAS_DATABASE
   if (LoadSQL(m_config)) {
 	  m_sqlactive = true;
-  } else 
+  } else
 #endif
   {
 	const PStringToString kv = m_config->GetAllKeyValues(AssignedGatekeeperSection);
-	for (PINDEX i=0; i < kv.GetSize(); i++) 
+	for (PINDEX i=0; i < kv.GetSize(); i++)
 		   assignedGKList.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),kv.GetDataAt(i)));
   }
 }
 
 #ifdef H323_H350
-bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses)
+bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias, const PIPSocket::Address & ip, PStringArray & addresses)
 {
+	// support gatekeeper discovery
+    if (!Toolkit::AsBool(GkConfig()->GetString(H350Section, "GatekeeperDiscovery", "0")))
+	   return false;
 
-	// Support Gatekeeper discovery
-    if (!GkConfig()->GetString(H350Section, "GatekeeperDiscovery", "0"))
-		   return false;
-
-	// Search the Directory
+	// search the directory
 	PString search = GkConfig()->GetString(H350Section, "SearchBaseDN", "");
 
 	H225_AliasAddress aliasaddress;
@@ -2007,54 +2050,58 @@ bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias,cons
 	switch (aliasaddress.GetTag()) {
 	  case H225_AliasAddress::e_dialedDigits:
             filter = "h323IdentitydialedDigits=" + alias;
+            break;
 	  case H225_AliasAddress::e_h323_ID:
             filter = "h323Identityh323-ID=" + alias;
+            break;
 	  case H225_AliasAddress::e_url_ID:
 		    filter = "h323IdentityURL-ID=" + alias;
+		    break;
 	  default:
-		  return false;
+			PTRACE(4, "H350\tAssigned GK: unhandled alias type " << aliasaddress.GetTagName());
+			return false;
 	}
 
 	H350_Session session;
 	if (!Toolkit::Instance()->CreateH350Session(&session)) {
-	   PTRACE(4,"H350\tAssigned GK: Could not connect to Server.");
+	   PTRACE(1, "H350\tAssigned GK: Could not connect to Server.");
 	   return false;
 	}
 
 	H350_Session::LDAP_RecordList rec;
 	int count = session.Search(search,filter,rec);
 	if (count <= 0) {
-	   PTRACE(4,"H350\tAssigned GK: No Record Found");
-	   session.Close();
-	   return false;
+		PTRACE(4, "H350\tAssigned GK: No Record Found");
+		session.Close();
+		return false;
 	}
 
-	// Locate the record
+	// locate the record
 	for (H350_Session::LDAP_RecordList::const_iterator x = rec.begin(); x != rec.end(); ++x) {			
-       H350_Session::LDAP_Record entry = x->second;
-	   PString gk;
-	   if (session.GetAttribute(entry,"h323IdentityGKDomain",gk)) {
-           PTRACE(2,"H350\tAssigned GK: GK located " << gk);
-		   addresses = gk.Lines();
-		   session.Close();
-		   return true;
-	   }
+		H350_Session::LDAP_Record entry = x->second;
+		PString gk;
+		if (session.GetAttribute(entry, "h323IdentityGKDomain", gk)) {
+			PTRACE(2, "H350\tAssigned GK: GK located " << gk);
+			addresses = gk.Lines();
+			session.Close();
+			return true;
+		}
 	}
 
-	PTRACE(4,"H350\tAssigned GK: No valid Assigned GK found.");
+	PTRACE(4, "H350\tAssigned GK: No valid Assigned GK found");
 	session.Close();
 	return false;
 }
 #endif
 
-bool Toolkit::AssignedGatekeepers::QueryAssignedGK(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses)
+bool Toolkit::AssignedGatekeepers::QueryAssignedGK(const PString & alias, const PIPSocket::Address & ip, PStringArray & addresses)
 {
 #if HAS_DATABASE
-	if (DatabaseLookup(alias,ip,addresses))
+	if (DatabaseLookup(alias, ip, addresses))
 		return true;
 #endif
 #ifdef H323_H350
-	if (QueryH350Directory(alias,ip,addresses))
+	if (QueryH350Directory(alias, ip, addresses))
 		return true;
 #endif
 
@@ -2067,29 +2114,29 @@ static PString DNStoIP(const PString & dns)
    H323TransportAddress iface(dns);
    PIPSocket::Address ip;
    WORD port = GK_DEF_UNICAST_RAS_PORT;
-   iface.GetIpAndPort(ip, port); 
+   iface.GetIpAndPort(ip, port);
    return PString(ip + ":" + port);
 }
 #endif
 
-bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias,const PIPSocket::Address & ip, H225_ArrayOf_AlternateGK & gklist)
+bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias, const PIPSocket::Address & ip, H225_ArrayOf_AlternateGK & gklist)
 {
 	bool found = false;
 	PStringArray assignedGK;
-	found = QueryAssignedGK(alias,ip,assignedGK);
+	found = QueryAssignedGK(alias, ip, assignedGK);
 
 	if (!found) {
 		  for (unsigned j=0; j < assignedGKList.size(); j++) {
 		    PString match = assignedGKList[j].first.Trim();
 			if (match.Left(1) != "^") {
-			  // Do prefix match
+			  // prefix match
 			  if (MatchPrefix(alias,assignedGKList[j].first)) {
 				   assignedGK.AppendString(assignedGKList[j].second);
 				   if (!found) found = true;
 			  }
 			} else {
-	          // Do Regex match for IP address
-			  if (MatchRegex(ip.AsString(),match)) {
+	          // regex match for IP address
+			  if (MatchRegex(ip.AsString(), match)) {
 				   assignedGK.AppendString(assignedGKList[j].second);
 				   if (!found) found = true;
 		      }
@@ -2102,36 +2149,36 @@ bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias,const PIP
 		for (PINDEX k=0; k < assignedGK.GetSize(); k++) {
            PString number = assignedGK[k];
 
-			if (IsIPAddress(number)) 
+			if (IsIPAddress(number))
 				ipaddresses.AppendString(number);
 #ifdef hasSRV
 			else {
 				PString xnum = assignedGK[k];
-				if (xnum.Left(5) != "h323:") 
-						xnum = "h323:user@" + xnum;
+				if (xnum.Left(5) != "h323:")
+					xnum = "h323:user@" + xnum;
 
 				PStringList str;
 				if (PDNS::LookupSRV(xnum,"_h323rs._udp.",str)) {
 					PTRACE(4, "AssignGK\t" << str.GetSize() << " SRV Records found" );
 					for (PINDEX i = 0; i < str.GetSize(); i++) {
 						PString newhost = str[i].Right(str[i].GetLength()-5);
-						PTRACE(4, "AssignGK\tDNS SRV converted GK address " << number << " to " << newhost );
+						PTRACE(4, "AssignedGK\tDNS SRV converted GK address " << number << " to " << newhost );
 						ipaddresses.AppendString(newhost);
 					}
 				} else {
-						ipaddresses.AppendString(DNStoIP(number));
+					ipaddresses.AppendString(DNStoIP(number));
 				}
 			}
-#endif   
+#endif
 		}
 
-		for (PINDEX k=0; k < ipaddresses.GetSize(); k++) {
+		for (PINDEX k = 0; k < ipaddresses.GetSize(); k++) {
            PString num = ipaddresses[k];
            WORD port = GK_DEF_UNICAST_RAS_PORT;
 		   PStringArray tokens = num.Tokenise(":", FALSE);
-		   if (tokens.GetSize() == 2) 
+		   if (tokens.GetSize() == 2)
                port = (WORD)tokens[1].AsUnsigned();
-	        
+
 		   H225_AlternateGK * alt = new H225_AlternateGK;
 		     alt->m_rasAddress = SocketToH225TransportAddr(PIPSocket::Address(tokens[0]),port);
 		     alt->m_needToRegister = true;
@@ -2148,14 +2195,14 @@ bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias,const PIP
 
 #if HAS_DATABASE
 Toolkit::QoSMonitor::QoSMonitor()
-  : m_sqlactive(false), m_sqlConn(NULL)
+  : m_sqlactive(false), m_sqlConn(NULL), m_timeout(-1)
 {
 }
 
 Toolkit::QoSMonitor::~QoSMonitor()
 {
 }
-	    
+
 void Toolkit::QoSMonitor::LoadConfig(PConfig * cfg)
 {
 	delete m_sqlConn;
@@ -2203,7 +2250,7 @@ void Toolkit::QoSMonitor::LoadConfig(PConfig * cfg)
 
 bool Toolkit::QoSMonitor::PostRecord(const std::map<PString, PString>& params)
 {
-	if (!m_sqlactive) 
+	if (!m_sqlactive)
 		return false;
 
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
@@ -2228,7 +2275,7 @@ bool Toolkit::QoSMonitor::PostRecord(const std::map<PString, PString>& params)
 			return false;
 		}
 	}
-    
+
 	delete result;
 	return true;
 }
@@ -2256,7 +2303,7 @@ bool Toolkit::RewriteE164(H225_AliasAddress & alias)
 			H323SetAliasAddress(E164, alias,H225_AliasAddress::e_url_ID);
 		else if (IsNumeric(E164))
 			H323SetAliasAddress(E164, alias, H225_AliasAddress::e_dialedDigits);
-		else if (alias.GetTag() != H225_AliasAddress::e_partyNumber) 
+		else if (alias.GetTag() != H225_AliasAddress::e_partyNumber)
 			H323SetAliasAddress(E164, alias, H225_AliasAddress::e_h323_ID);
 		else {
 			H225_PartyNumber & partyNumber = alias;
@@ -2281,11 +2328,8 @@ bool Toolkit::RewriteE164(H225_ArrayOf_AliasAddress & aliases)
 	return changed;
 }
 
-bool Toolkit::GWRewriteE164(PString gw, bool direction, H225_AliasAddress &alias) {
-
-	PString E164;
-	bool changed;
-
+bool Toolkit::GWRewriteE164(const PString & gw, bool direction, H225_AliasAddress & alias)
+{
 	if (alias.GetTag() != H225_AliasAddress::e_dialedDigits) {
 		if (alias.GetTag() != H225_AliasAddress::e_partyNumber)
 			return false;
@@ -2294,8 +2338,8 @@ bool Toolkit::GWRewriteE164(PString gw, bool direction, H225_AliasAddress &alias
 			return false;
 	}
 
-	E164 = ::AsString(alias, FALSE);
-	changed = GWRewritePString(gw,direction,E164);
+	PString E164 = ::AsString(alias, FALSE);
+	bool changed = GWRewritePString(gw, direction, E164);
 
 	if (changed) {
 		if (alias.GetTag() == H225_AliasAddress::e_dialedDigits)
@@ -2315,27 +2359,25 @@ bool Toolkit::GWRewriteE164(PString gw, bool direction, H225_AliasAddress &alias
 	return changed;
 }
 
-bool Toolkit::GWRewriteE164(PString gw, bool direction, H225_ArrayOf_AliasAddress &aliases) {
+bool Toolkit::GWRewriteE164(const PString & gw, bool direction, H225_ArrayOf_AliasAddress & aliases)
+{
+	bool changed = false;
 
-	bool changed;
-	PINDEX n;
-
-	changed = false;
-	for (n = 0; n < aliases.GetSize(); ++n) {
-		changed |= GWRewriteE164(gw,direction,aliases[n]);
+	for (PINDEX n = 0; n < aliases.GetSize(); ++n) {
+		changed |= GWRewriteE164(gw, direction, aliases[n]);
 	}
 
 	return changed;
 }
 
-bool Toolkit::isBehindNAT(PIPSocket::Address & externalIP) const {
-
+bool Toolkit::isBehindNAT(PIPSocket::Address & externalIP) const
+{
    return (m_VirtualRouteTable.IsMasquerade(externalIP));
 }
 
 #ifdef HAS_H46023
-bool Toolkit::IsH46023Enabled() const 
-{ 
+bool Toolkit::IsH46023Enabled() const
+{
 	return (m_H46023Enabled && (             // is enabled and
 #ifdef HAS_H46018
 		   m_H46018Enabled ||              // used with H.460.18 or
@@ -2387,20 +2429,20 @@ bool Toolkit::H46023SameNetwork(const PIPSocket::Address & addr1, const PIPSocke
 #endif
 
 #ifdef HAS_H460P
-bool Toolkit::IsH460PEnabled() const 
+bool Toolkit::IsH460PEnabled() const
 {
 	return m_presence.IsEnabled();
 }
 
-GkPresence & Toolkit::GetPresenceHandler() 
-{ 
-	return m_presence; 
+GkPresence & Toolkit::GetPresenceHandler()
+{
+	return m_presence;
 }
 #endif
 
 std::vector<NetworkAddress> Toolkit::GetInternalNetworks() {
 
-    return GkConfig()->HasKey("ExternalIP") ? m_VirtualRouteTable.GetInternalNetworks() : m_RouteTable.GetInternalNetworks(); 
+    return !GkConfig()->GetString("ExternalIP", "").IsEmpty() ? m_VirtualRouteTable.GetInternalNetworks() : m_RouteTable.GetInternalNetworks();
 }
 
 PString Toolkit::GetGKHome(vector<PIPSocket::Address> & GKHome) const
@@ -2418,62 +2460,51 @@ PString Toolkit::GetGKHome(vector<PIPSocket::Address> & GKHome) const
 
 void Toolkit::SetGKHome(const PStringArray & home)
 {
-	std::vector<PIPSocket::Address>::iterator begin;
 	m_GKHome.clear();
-	int n, i, size = home.GetSize();
-	if (size > 0)
-		for (n = 0; n < size; ++n)
-			m_GKHome.push_back(home[n]);
+	for (PINDEX n = 0; n < home.GetSize(); ++n)
+		m_GKHome.push_back(home[n]);
 
 	PIPSocket::InterfaceTable it;
 	if (PIPSocket::GetInterfaceTable(it)) {
 		int is = it.GetSize();
-		if (size > 0) {
-			// check if the interface is valid
-			for (n = 0; n < size; ++n) {
-				for (i = 0; i < is; ++i)
-					if (m_GKHome[n] == it[i].GetAddress())
-						break;
-				if (i == is) {
-					PTRACE(1, "GK\tAddress " << m_GKHome[n] << " not found"
-						" in the PWLib interface table"
-						);
-					//begin = m_GKHome.begin();
-					//copy(begin + n + 1, begin + size, begin + n);
-					//--size, --n;
+		// check if the interface is valid
+		for (size_t n = 0; n < m_GKHome.size(); ++n) {
+			PINDEX i = 0;
+			for (i = 0; i < is; ++i)
+				if (m_GKHome[n] == it[i].GetAddress())
+					break;
+			if (i == is) {
+				PTRACE(1, "GK\tAddress " << m_GKHome[n] << " not found"
+					" in the PTLib interface table");
+			}
+		}
+
+		// if no home interfaces specified, set all IPs from interface table
+		// except INADDR_ANY and any non IPv4 adresses
+		if (m_GKHome.empty()) {
+			for (PINDEX n = 0; n < it.GetSize(); ++n) {
+				const PIPSocket::Address addr = it[n].GetAddress();
+				if (addr != INADDR_ANY && addr.GetVersion() == 4) {
+					m_GKHome.push_back(addr);
 				}
 			}
 		}
-		if (size == 0) {
-			m_GKHome.clear();
-			size = is;
-			for (n = 0; n < size; ++n)
-				m_GKHome.push_back(it[n].GetAddress());
-		}
-		// remove INADDR_ANY
-		for (n = 0; n < size; ++n)
-			if (m_GKHome[n] == INADDR_ANY) {
-				begin = m_GKHome.begin();
-				copy(begin + n + 1, begin + size, begin + n);
-				--size, --n;
-			}
 	}
 
 	// remove duplicate interfaces
-	for (n = 0; n < size; ++n)
-		for (i = 0; i < n; ++i)
-			if (m_GKHome[n] == m_GKHome[i]) {
-				begin = m_GKHome.begin();
-				copy(begin + n + 1, begin + size, begin + n);
-				--size, --n;
-				break;
-			}
+	sort(m_GKHome.begin(), m_GKHome.end());
+	unique(m_GKHome.begin(), m_GKHome.end());
 
-	m_GKHome.resize(size);
-	// put the default IP to the first
-	begin = find(m_GKHome.begin(), m_GKHome.end(), m_RouteTable.GetLocalAddress());
-	if (begin != m_GKHome.end())
-		swap(m_GKHome[0], *begin);
+	// move loopback interfaces to the end
+	std::list<PIPSocket::Address> sortedHomes;
+	for (unsigned j=0; j < m_GKHome.size(); j++) {
+		if (m_GKHome[j].IsLoopback()) {
+			sortedHomes.push_back(m_GKHome[j]);
+		} else {
+			sortedHomes.push_front(m_GKHome[j]);
+		}		
+	}
+	m_GKHome.assign(sortedHomes.begin(), sortedHomes.end());
 }
 
 bool Toolkit::IsGKHome(const PIPSocket::Address & addr) const
@@ -2548,7 +2579,7 @@ void Toolkit::GetNetworkFromString(
 	PINDEX slashPos = s.Find('/');
 	if (slashPos == P_MAX_INDEX) {
 		// a single IP
-		static BYTE fullNetMask[16] = { 
+		static BYTE fullNetMask[16] = {
 			255, 255, 255, 255, 255, 255, 255, 255,
 			255, 255, 255, 255, 255, 255, 255, 255
 			};
@@ -2595,7 +2626,7 @@ PString Toolkit::AsString(
 {
 	PString fmtStr = !formatStr ? formatStr : m_timestampFormatStr;
 	if (fmtStr.IsEmpty())
-		return PString();
+		return PString::Empty();
 
 	if (fmtStr *= "Cisco")
 		fmtStr = "%H:%M:%S.%u %Z %a %b %d %Y";
@@ -2638,15 +2669,12 @@ PString Toolkit::AsString(
 		}
 	} while (i != P_MAX_INDEX && i < length);
 	
-	PString buf;
-	
-	buf.SetSize(128);
-	if (strftime(buf.GetPointer(), 128, (const char*)fmtStr, tmptr) == 0) {
+	char buf[128];
+	if (strftime(buf, sizeof(buf), (const char*)fmtStr, tmptr) == 0) {
 		PTRACE(0, "TOOLKIT\tCould not apply timestamp formatting - using default");
 		return tm.AsString( "hh:mm:ss.uuu z www MMM d yyyy" );
 	}
-	
-	buf.MakeMinimumSize();
+
 	return buf;
 }
 
@@ -2657,11 +2685,11 @@ PString Toolkit::ReadPassword(
 	)
 {
 	if (cfgSection.IsEmpty() || cfgKey.IsEmpty())
-		return PString();
+		return PString::Empty();
 		
 	PConfig* const cfg = Config();
 	if (!cfg->HasKey(cfgSection, cfgKey))
-		return PString();
+		return PString::Empty();
 
 	int paddingByte = m_encKeyPaddingByte;
 	if (cfg->HasKey(cfgSection, paddingByteConfigKey)) {
