@@ -2,15 +2,17 @@
 //
 // Toolkit class for the GnuGk
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.74 2010/04/29 22:22:12 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.78 2011/06/15 20:24:50 willamowius Exp $"
 
 #include <vector>
 #include <map>
@@ -38,7 +40,7 @@ struct SetupAuthData;
 /// Hold an address of a single host or a whole network
 class NetworkAddress {
 public:
-	/// Build an "any" address 
+	/// Build an "any" address
 	NetworkAddress();
 	/// Build a single host address
 	NetworkAddress(
@@ -107,6 +109,9 @@ class GkSQLConnection;
 class GkTimerManager;
 class CLIRewrite;
 
+#define GW_REWRITE_IN		true
+#define GW_REWRITE_OUT	false
+
 class Toolkit : public Singleton<Toolkit>
 {
  public:
@@ -140,7 +145,7 @@ class Toolkit : public Singleton<Toolkit>
 			bool Compare(const Address *) const;
 		};
 		virtual bool CreateTable() { return CreateRouteTable(); }
-		bool CreateRouteTable(const PString & extroute=PString());
+		bool CreateRouteTable(const PString & extroute = PString::Empty());
 
 		RouteEntry *rtable_begin, *rtable_end;
 		Address defAddr;
@@ -189,7 +194,7 @@ class Toolkit : public Singleton<Toolkit>
 	{ return m_ProxyCriterion.SelectRoutingMode(ip1, ip2); }
 
 	bool IsInternal(const PIPSocket::Address & ip1)
-	{ return (m_ProxyCriterion.IsInternal(ip1) > 0);  }
+	{ return (m_ProxyCriterion.IsInternal(ip1) > 0); }
 
 	// Since PStringToString is not thread-safe,
 	// I write this small class to replace that
@@ -343,7 +348,7 @@ class Toolkit : public Singleton<Toolkit>
 			~GWRewriteTool();
 			void LoadConfig(PConfig *);
 			void PrintData();
-			bool RewritePString(PString gw, bool direction, PString &data);
+			bool RewritePString(const PString & gw, bool direction, PString & data);
 
 		private:
 			PDictionary<PString, GWRewriteEntry> m_GWRewrite;
@@ -351,9 +356,9 @@ class Toolkit : public Singleton<Toolkit>
 	};
 
 	// Equivalent functions to RewriteE164 group
-	bool GWRewriteE164(PString gw, bool direction, H225_AliasAddress &alias);
-	bool GWRewriteE164(PString gw, bool direction, H225_ArrayOf_AliasAddress &aliases);
-	bool GWRewritePString(PString gw, bool direction, PString &data) { return m_GWRewrite.RewritePString(gw,direction,data); }
+	bool GWRewriteE164(const PString & gw, bool direction, H225_AliasAddress &alias);
+	bool GWRewriteE164(const PString & gw, bool direction, H225_ArrayOf_AliasAddress &aliases);
+	bool GWRewritePString(const PString & gw, bool direction, PString & data) { return m_GWRewrite.RewritePString(gw, direction, data); }
 
 	PString GetGKHome(std::vector<PIPSocket::Address> &) const;
 	void SetGKHome(const PStringArray &);
@@ -445,7 +450,7 @@ class Toolkit : public Singleton<Toolkit>
 		iecUnknown     = -1,   /// internal extension code for an unknown triple(cntry,ext,manuf)
 		iecFailoverRAS  = 1,   /// i.e.c. for "This RQ is a failover RQ" and must not be answerd.
 		iecNeighborId = 2,     /// i.e.c. for neighbor id sent inside LRQ/LCF nonStandardData parameter
-		iecNATTraversal = 3,   /// i.e.c. for IP=/NAT= parameters sent inside RRQ/RCF nonStandardData 
+		iecNATTraversal = 3,   /// i.e.c. for IP=/NAT= parameters sent inside RRQ/RCF nonStandardData
 		iecUserbase    = 1000 /// first guaranteed unused 'iec' by GnuGk Toolkit.
 	};
 	/** t35 extension or definitions as field for H225_NonStandardIdentifier */
@@ -515,7 +520,7 @@ class Toolkit : public Singleton<Toolkit>
 	*/
 	PString AsString(
 		const PTime& tm, /// timestamp to convert into a string
-		const PString& formatStr = PString() /// format string to use
+		const PString& formatStr = PString::Empty() /// format string to use
 		);
 
 	/** Read and decrypt a password from the config. As a decryption key
