@@ -3,14 +3,16 @@
  *
  * Module for accoutning per IP/H.323 ID/CLI/prefix inbound call volume
  *
- * $Id: capctrl.cxx,v 1.18 2009/05/24 20:48:26 willamowius Exp $
+ * $Id: capctrl.cxx,v 1.22 2011/05/09 21:38:53 willamowius Exp $
  *
  * Copyright (c) 2006, Michal Zygmuntowicz
+ * Copyright (c) 2008-2010, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
+ * with the OpenH323/H323Plus and OpenSSL library.
+ *
  */
 
 #include <ptlib.h>
@@ -30,7 +32,7 @@ namespace {
 
 struct IpRule_greater : public std::binary_function<CapacityControl::IpCallVolume, CapacityControl::IpCallVolume, bool> {
 
-	bool operator()(const CapacityControl::IpCallVolume &e1, const CapacityControl::IpCallVolume &e2) const 
+	bool operator()(const CapacityControl::IpCallVolume &e1, const CapacityControl::IpCallVolume &e2) const
 	{
 		if (e1.first.IsAny()) {
 			if (!e2.first.IsAny())
@@ -48,7 +50,7 @@ struct IpRule_greater : public std::binary_function<CapacityControl::IpCallVolum
 
 struct H323IdRule_greater : public std::binary_function<CapacityControl::H323IdCallVolume, CapacityControl::H323IdCallVolume, bool> {
 
-	bool operator()(const CapacityControl::H323IdCallVolume &e1, const CapacityControl::H323IdCallVolume &e2) const 
+	bool operator()(const CapacityControl::H323IdCallVolume &e1, const CapacityControl::H323IdCallVolume &e2) const
 	{
 		return H323GetAliasAddressString(e1.first) > H323GetAliasAddressString(e2.first);
 	}
@@ -56,7 +58,7 @@ struct H323IdRule_greater : public std::binary_function<CapacityControl::H323IdC
 
 struct CLIRule_greater : public std::binary_function<CapacityControl::CLICallVolume, CapacityControl::CLICallVolume, bool> {
 
-	bool operator()(const CapacityControl::CLICallVolume &e1, const CapacityControl::CLICallVolume &e2) const 
+	bool operator()(const CapacityControl::CLICallVolume &e1, const CapacityControl::CLICallVolume &e2) const
 	{
 		return e1.first.compare(e2.first) > 0;
 	}
@@ -115,9 +117,6 @@ void CapacityControl::LoadConfig()
 	const PString cfgSec("CapacityControl");
 	
 	unsigned ipRules = 0, h323IdRules = 0, cliRules = 0;
-	IpCallVolumes::iterator ipRule = ipCallVolumes.end();
-	H323IdCallVolumes::iterator h323IdRule  = h323IdCallVolumes.end();
-	CLICallVolumes::iterator cliRule  = cliCallVolumes.end();
 	
 	const PStringToString kv = cfg->GetAllKeyValues(cfgSec);
 	for (PINDEX i = 0; i < kv.GetSize(); ++i) {
@@ -166,7 +165,7 @@ void CapacityControl::LoadConfig()
 				
 				rule = &(cliCallVolumes.back().second);
 			} else {
-				PTRACE(1, "CAPCTRL\tUknown CapacityControl rule: " << key << '=' 
+				PTRACE(1, "CAPCTRL\tUknown CapacityControl rule: " << key << '='
 					<< kv.GetDataAt(i)
 					);
 				continue;
@@ -174,7 +173,7 @@ void CapacityControl::LoadConfig()
 
 			PStringArray tokens(data.Tokenise(" \t", FALSE));
 			if (tokens.GetSize() < 1) {
-				PTRACE(1, "CAPCTRL\tInvalid CapacityControl rule syntax: " << key << '=' 
+				PTRACE(1, "CAPCTRL\tInvalid CapacityControl rule syntax: " << key << '='
 					<< kv.GetDataAt(i)
 					);
 				if (newIpRule)
@@ -425,7 +424,7 @@ void CapacityControl::LogCall(
 {
 	if (callStart) {
 		if (callNumber < 1) {
-			PTRACE(0, "CAPCTRL\tInvalid call number used (" << callNumber << ")");
+			PTRACE(1, "CAPCTRL\tInvalid call number used (" << callNumber << ")");
 		}
 	
 		// find longest matching rule by ip/h323id/cli
@@ -553,7 +552,7 @@ public:
 	};
 	
 	/// Create a logger that updates information about inbound traffic
-	CapCtrlAcct( 
+	CapCtrlAcct(
 		/// name from Gatekeeper::Acct section
 		const char* moduleName
 		);
@@ -563,7 +562,7 @@ public:
 		@return
 		Status of this logging operation (see #Status enum#)
 	*/
-	virtual Status Log( 
+	virtual Status Log(
 		AcctEvent evt, /// accounting event to log
 		const callptr& call /// additional data for the event
 		);
@@ -588,7 +587,7 @@ CapCtrlAcct::CapCtrlAcct(
 }
 
 GkAcctLogger::Status CapCtrlAcct::Log(
-	GkAcctLogger::AcctEvent evt, 
+	GkAcctLogger::AcctEvent evt,
 	const callptr &call
 	)
 {
@@ -668,9 +667,9 @@ CapCtrlAuth::CapCtrlAuth(
 	const char* authName,
 	unsigned supportedRasChecks,
 	unsigned supportedMiscChecks
-	) 
-	: 
-	GkAuthenticator(authName, supportedRasChecks, supportedMiscChecks), 
+	)
+	:
+	GkAuthenticator(authName, supportedRasChecks, supportedMiscChecks),
 	m_capacityControl(CapacityControl::Instance())
 {
 }
