@@ -2,23 +2,28 @@
 //
 // config.h configuration header
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2008-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 
 #ifndef CONFIG_H
-#define CONFIG_H "@(#) $Id: config.h,v 1.5 2009/09/19 16:36:19 willamowius Exp $"
+#define CONFIG_H "@(#) $Id: config.h,v 1.7 2011/03/06 14:46:01 willamowius Exp $"
 
 #include "pwlib_compat.h"
 
 #ifdef _WIN32
 #include "gnugkbuildopts.h"
+#else
+#define GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
 #endif
 
+
 #if (defined P_LINUX) || (defined P_FREEBSD) || (defined P_HPUX9) || (defined P_SOLARIS) || (defined P_OPENBSD)
 // On some OS we don't get broadcasts on a socket that is
 // bound to a specific interface. For those we have to start
