@@ -2,10 +2,12 @@
 //
 // gk.cxx for GNU Gatekeeper
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2011, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -34,8 +36,10 @@
 
 using std::vector;
 
+PCREATE_PROCESS(Gatekeeper)
+
 /*
- * many things here should be members of Gatekeeper. 
+ * many things here should be members of Gatekeeper
  */
 
 PReadWriteMutex ConfigReloadMutex;
@@ -51,10 +55,10 @@ int Gatekeeper::m_rotateDay = 0;
 GkTimer* Gatekeeper::m_rotateTimer = GkTimerManager::INVALID_HANDLE;
 #endif // PTRACING
 
+PSemaphore ShutdownMutex(1,1);
+
 namespace { // keep the global objects private
 
-
-PMutex ShutdownMutex;
 PTimedMutex ReloadMutex;
 
 #ifndef _WIN32
@@ -84,22 +88,490 @@ void ShutdownHandler()
 #endif
 }
 
-bool CheckSectionName(PConfig *cfg)
+static const char * KnowConfigEntries[][2] = {
+	// valid config entries
+	{ "AssignedAliases::SQL", "CacheTimeout" },
+	{ "AssignedAliases::SQL", "Database" },
+	{ "AssignedAliases::SQL", "Driver" },
+	{ "AssignedAliases::SQL", "Host" },
+	{ "AssignedAliases::SQL", "Library" },
+	{ "AssignedAliases::SQL", "MinPoolSize" },
+	{ "AssignedAliases::SQL", "Password" },
+	{ "AssignedAliases::SQL", "Query" },
+	{ "AssignedAliases::SQL", "Username" },
+	{ "AssignedGatekeepers::SQL", "CacheTimeout" },
+	{ "AssignedGatekeepers::SQL", "Database" },
+	{ "AssignedGatekeepers::SQL", "Driver" },
+	{ "AssignedGatekeepers::SQL", "Host" },
+	{ "AssignedGatekeepers::SQL", "Library" },
+	{ "AssignedGatekeepers::SQL", "MinPoolSize" },
+	{ "AssignedGatekeepers::SQL", "Query" },
+	{ "CTI::Agents", "CTI_Timeout" },	// obsolete, but still accepted
+	{ "CTI::Agents", "RequestTimeout" },
+	{ "CTI::Agents", "VirtualQueue" },	// obsolete, but still accepted
+	{ "CTI::Agents", "VirtualQueueAliases" },
+	{ "CTI::Agents", "VirtualQueuePrefixes" },
+	{ "CTI::Agents", "VirtualQueueRegex" },
+	{ "CTI::MakeCall", "DisableFastStart" },
+	{ "CTI::MakeCall", "DisableH245Tunneling" },
+	{ "CTI::MakeCall", "EndpointAlias" },
+	{ "CTI::MakeCall", "Gatekeeper" },
+	{ "CTI::MakeCall", "Interface" },
+	{ "CTI::MakeCall", "UseH450" },
+	{ "CallTable", "AcctUpdateInterval" },
+	{ "CallTable", "DefaultCallDurationLimit" },
+	{ "CallTable", "DefaultCallTimeout" },
+	{ "CallTable", "DisabledCodecs" },
+	{ "CallTable", "GenerateNBCDR" },
+	{ "CallTable", "GenerateUCCDR" },
+	{ "CallTable", "IRRCheck" },
+	{ "CallTable", "IRRFrequency" },
+	{ "CallTable", "SingleFailoverCDR" },
+	{ "CallTable", "TimestampFormat" },
+	{ "Endpoint", "DisableH46018" },
+	{ "Endpoint", "Discovery" },
+	{ "Endpoint", "E164" },
+	{ "Endpoint", "EndpointIdentifier" },
+	{ "Endpoint", "ForwardDestIp" },
+	{ "Endpoint", "Gatekeeper" },
+	{ "Endpoint", "GatekeeperIdentifier" },
+	{ "Endpoint", "H323ID" },
+	{ "Endpoint", "NATKeepaliveInterval" },
+	{ "Endpoint", "NATRetryInterval" },
+	{ "Endpoint", "Password" },
+	{ "Endpoint", "Prefix" },
+	{ "Endpoint", "RRQRetryInterval" },
+	{ "Endpoint", "TimeToLive" },
+	{ "Endpoint", "Type" },
+	{ "Endpoint", "UnregisterOnReload" },
+	{ "Endpoint", "UseAlternateGK" },
+	{ "Endpoint", "Vendor" },
+	{ "FileAcct", "CDRString" },
+	{ "FileAcct", "DetailFile" },
+	{ "FileAcct", "Rotate" },
+	{ "FileAcct", "RotateDay" },
+	{ "FileAcct", "RotateTime" },
+	{ "FileAcct", "StandardCDRFormat" },
+	{ "FileAcct", "TimestampFormat" },
+	{ "Gatekeeper::Acct", "CapacityControl" },
+	{ "Gatekeeper::Acct", "FileAcct" },
+	{ "Gatekeeper::Acct", "RadAcct" },
+	{ "Gatekeeper::Acct", "SQLAcct" },
+	{ "Gatekeeper::Acct", "StatusAcct" },
+	{ "Gatekeeper::Acct", "SyslogAcct" },
+	{ "Gatekeeper::Acct", "default" },
+	{ "Gatekeeper::Auth", "AliasAuth" },
+	{ "Gatekeeper::Auth", "CapacityControl" },
+	{ "Gatekeeper::Auth", "FileIPAuth" },
+	{ "Gatekeeper::Auth", "H350PasswordAuth" },
+	{ "Gatekeeper::Auth", "PrefixAuth" },
+	{ "Gatekeeper::Auth", "RadAliasAuth" },
+	{ "Gatekeeper::Auth", "RadAuth" },
+	{ "Gatekeeper::Auth", "SQLAliasAuth" },
+	{ "Gatekeeper::Auth", "SQLAuth" },
+	{ "Gatekeeper::Auth", "SQLPasswordAuth" },
+	{ "Gatekeeper::Auth", "SimplePasswordAuth" },
+	{ "Gatekeeper::Auth", "default" },
+	{ "Gatekeeper::Main", "AlternateGKs" },
+	{ "Gatekeeper::Main", "Bind" },
+	{ "Gatekeeper::Main", "CompareAliasCase" },
+	{ "Gatekeeper::Main", "CompareAliasType" },
+	{ "Gatekeeper::Main", "DefaultDomain" },
+	{ "Gatekeeper::Main", "DisconnectCallsOnShutdown" },
+	{ "Gatekeeper::Main", "EncryptAllPasswords" },
+	{ "Gatekeeper::Main", "EndpointIDSuffix" },
+	{ "Gatekeeper::Main", "EndpointSignalPort" },
+	{ "Gatekeeper::Main", "EndpointSuffix" },
+	{ "Gatekeeper::Main", "ExternalIP" },
+	{ "Gatekeeper::Main", "ExternalIsDynamic" },
+	{ "Gatekeeper::Main", "FortyTwo" },	// obsolete
+	{ "Gatekeeper::Main", "FourtyTwo" },	// obsolete
+	{ "Gatekeeper::Main", "Home" },
+	{ "Gatekeeper::Main", "ListenQueueLength" },
+	{ "Gatekeeper::Main", "MaxASNArraySize" },
+	{ "Gatekeeper::Main", "MinimumBandwidthPerCall" },
+	{ "Gatekeeper::Main", "MulticastGroup" },
+	{ "Gatekeeper::Main", "MulticastPort" },
+	{ "Gatekeeper::Main", "Name" },
+	{ "Gatekeeper::Main", "NetworkInterfaces" },
+	{ "Gatekeeper::Main", "RedirectGK" },
+	{ "Gatekeeper::Main", "SendTo" },
+	{ "Gatekeeper::Main", "SkipForwards" },
+	{ "Gatekeeper::Main", "SshStatusPort" },
+	{ "Gatekeeper::Main", "StatusPort" },
+	{ "Gatekeeper::Main", "StatusTraceLevel" },
+	{ "Gatekeeper::Main", "StatusWriteTimeout" },
+	{ "Gatekeeper::Main", "TimeToLive" },
+	{ "Gatekeeper::Main", "TimestampFormat" },
+	{ "Gatekeeper::Main", "TotalBandwidth" },
+	{ "Gatekeeper::Main", "TraceLevel" },
+	{ "Gatekeeper::Main", "UnicastRasPort" },
+	{ "Gatekeeper::Main", "UseBroadcastListener" },
+	{ "Gatekeeper::Main", "UseMulticastListener" },
+	{ "GkH350::Settings", "AssignedAliases" },
+	{ "GkH350::Settings", "BindAuthMode" },
+	{ "GkH350::Settings", "BindUserDN" },
+	{ "GkH350::Settings", "BindUserPW" },
+	{ "GkH350::Settings", "GatekeeperDiscovery" },
+	{ "GkH350::Settings", "SearchBaseDN" },
+	{ "GkH350::Settings", "ServerName" },
+	{ "GkH350::Settings", "ServerPort" },
+	{ "GkH350::Settings", "ServiceControl" },
+	{ "GkH350::Settings", "StartTLS" },
+	{ "GkPresence::SQL", "CacheTimeout" },
+	{ "GkPresence::SQL", "Database" },
+	{ "GkPresence::SQL", "Driver" },
+	{ "GkPresence::SQL", "Host" },
+	{ "GkPresence::SQL", "Library" },
+	{ "GkPresence::SQL", "MinPoolSize" },
+	{ "GkPresence::SQL", "QueryAdd" },
+	{ "GkPresence::SQL", "QueryDelete" },
+	{ "GkPresence::SQL", "QueryList" },
+	{ "GkPresence::SQL", "QueryUpdate" },
+	{ "GkQoSMonitor", "CallEndOnly" },
+	{ "GkQoSMonitor", "DetailFile" },
+	{ "GkQoSMonitor", "Enable" },
+	{ "GkQoSMonitor::SQL", "CacheTimeout" },
+	{ "GkQoSMonitor::SQL", "Database" },
+	{ "GkQoSMonitor::SQL", "Driver" },
+	{ "GkQoSMonitor::SQL", "Host" },
+	{ "GkQoSMonitor::SQL", "Library" },
+	{ "GkQoSMonitor::SQL", "Password" },
+	{ "GkQoSMonitor::SQL", "Query" },
+	{ "GkQoSMonitor::SQL", "Username" },
+	{ "LogFile", "Filename" },
+	{ "LogFile", "Rotate" },
+	{ "LogFile", "RotateDay" },
+	{ "LogFile", "RotateTime" },
+	{ "Proxy", "DisableRTPQueueing" },
+	{ "Proxy", "Enable" },
+	{ "Proxy", "EnableRTCPStats" },
+	{ "Proxy", "EnableRTPMute" },
+	{ "Proxy", "InternalNetwork" },
+	{ "Proxy", "ProxyAlways" },
+	{ "Proxy", "ProxyForNAT" },
+	{ "Proxy", "ProxyForSameNAT" },
+	{ "Proxy", "RTPPortRange" },
+	{ "Proxy", "T120PortRange" },
+	{ "RadAcct", "AppendCiscoAttributes" },
+	{ "RadAcct", "DefaultAcctPort" },
+	{ "RadAcct", "FixedUsername" },
+	{ "RadAcct", "IdCacheTimeout" },
+	{ "RadAcct", "LocalInterface" },
+	{ "RadAcct", "RadiusPortRange" },
+	{ "RadAcct", "RequestRetransmissions" },
+	{ "RadAcct", "RequestTimeout" },
+	{ "RadAcct", "RoundRobinServers" },
+	{ "RadAcct", "Servers" },
+	{ "RadAcct", "SharedSecret" },
+	{ "RadAcct", "SocketDeleteTimeout" },
+	{ "RadAcct", "TimestampFormat" },
+	{ "RadAcct", "UseDialedNumber" },
+	{ "RadAliasAuth", "AppendCiscoAttributes" },
+	{ "RadAliasAuth", "DefaultAuthPort" },
+	{ "RadAliasAuth", "FixedPassword" },
+	{ "RadAliasAuth", "FixedUsername" },
+	{ "RadAliasAuth", "IdCacheTimeout" },
+	{ "RadAliasAuth", "IncludeTerminalAliases" },
+	{ "RadAliasAuth", "LocalInterface" },
+	{ "RadAliasAuth", "RadiusPortRange" },
+	{ "RadAliasAuth", "RequestRetransmissions" },
+	{ "RadAliasAuth", "RequestTimeout" },
+	{ "RadAliasAuth", "RoundRobinServers" },
+	{ "RadAliasAuth", "Servers" },
+	{ "RadAliasAuth", "SharedSecret" },
+	{ "RadAliasAuth", "SocketDeleteTimeout" },
+	{ "RadAliasAuth", "UseDialedNumber" },
+	{ "RadAuth", "AppendCiscoAttributes" },
+	{ "RadAuth", "DefaultAuthPort" },
+	{ "RadAuth", "IdCacheTimeout" },
+	{ "RadAuth", "IncludeTerminalAliases" },
+	{ "RadAuth", "LocalInterface" },
+	{ "RadAuth", "RadiusPortRange" },
+	{ "RadAuth", "RequestRetransmissions" },
+	{ "RadAuth", "RequestTimeout" },
+	{ "RadAuth", "RoundRobinServers" },
+	{ "RadAuth", "Servers" },
+	{ "RadAuth", "SharedSecret" },
+	{ "RadAuth", "SocketDeleteTimeout" },
+	{ "RadAuth", "UseDialedNumber" },
+	{ "RasSrv::ARQFeatures", "ArjReasonRouteCallToGatekeeper" },
+	{ "RasSrv::ARQFeatures", "CallUnregisteredEndpoints" },
+	{ "RasSrv::ARQFeatures", "RemoveTrailingChar" },
+	{ "RasSrv::ARQFeatures", "RoundRobinGateways" },
+	{ "RasSrv::ARQFeatures", "SendRIP" },
+	{ "RasSrv::LRQFeatures", "AcceptForwardedLRQ" },
+	{ "RasSrv::LRQFeatures", "AcceptNonNeighborLCF" },
+	{ "RasSrv::LRQFeatures", "AcceptNonNeighborLRQ" },
+	{ "RasSrv::LRQFeatures", "AlwaysForwardLRQ" },
+	{ "RasSrv::LRQFeatures", "ForwardHopCount" },
+	{ "RasSrv::LRQFeatures", "ForwardLRQ" },
+	{ "RasSrv::LRQFeatures", "ForwardResponse" },
+	{ "RasSrv::LRQFeatures", "NeighborTimeout" },
+	{ "RasSrv::LRQFeatures", "SendRetries" },
+	{ "RasSrv::LRQFeatures", "SendRIP" },
+	{ "RasSrv::RRQFeatures", "AcceptEndpointIdentifier" },
+	{ "RasSrv::RRQFeatures", "AcceptGatewayPrefixes" },
+	{ "RasSrv::RRQFeatures", "AcceptMCUPrefixes" },
+	{ "RasSrv::RRQFeatures", "AliasTypeFilter" },
+	{ "RasSrv::RRQFeatures", "IRQPollCount" },
+	{ "RasSrv::RRQFeatures", "OverwriteEPOnSameAddress" },
+	{ "RasSrv::RRQFeatures", "SupportDynamicIP" },
+	{ "RoutedMode", "AcceptNeighborCalls" },
+	{ "RoutedMode", "AcceptNeighborsCalls" },
+	{ "RoutedMode", "AcceptUnregisteredCalls" },
+	{ "RoutedMode", "ActivateFailover" },
+	{ "RoutedMode", "AlertingTimeout" },
+	{ "RoutedMode", "CallSignalHandlerNumber" },
+	{ "RoutedMode", "CallSignalPort" },
+	{ "RoutedMode", "CpsCheckInterval" },
+	{ "RoutedMode", "CpsLimit" },
+	{ "RoutedMode", "DisableRetryChecks" },
+	{ "RoutedMode", "DropCallsByReleaseComplete" },
+	{ "RoutedMode", "ENUMservers" },
+	{ "RoutedMode", "EnableH450.2" },
+	{ "RoutedMode", "EnableH46018" },
+	{ "RoutedMode", "EnableH46023" },
+	{ "RoutedMode", "FailoverCauses" },
+	{ "RoutedMode", "ForceNATKeepAlive" },
+	{ "RoutedMode", "ForwardOnFacility" },
+	{ "RoutedMode", "GKRouted" },
+	{ "RoutedMode", "GenerateCallProceeding" },
+	{ "RoutedMode", "GkRouted" },
+	{ "RoutedMode", "H245PortRange" },
+	{ "RoutedMode", "H245Routed" },
+	{ "RoutedMode", "H4502EmulatorTransferMethod" },
+	{ "RoutedMode", "H46018NoNat" },
+	{ "RoutedMode", "H46023PublicIP" },
+	{ "RoutedMode", "H46023STUN" },
+	{ "RoutedMode", "NATStdMin" },
+	{ "RoutedMode", "Q931PortRange" },
+	{ "RoutedMode", "RemoveCallOnDRQ" },
+	{ "RoutedMode", "RemoveH235Call" },
+	{ "RoutedMode", "RemoveH245AddressOnTunneling" },
+	{ "RoutedMode", "RemoveH460Call" },
+	{ "RoutedMode", "RtpHandlerNumber" },
+	{ "RoutedMode", "ScreenCallingPartyNumberIE" },
+	{ "RoutedMode", "ScreenDisplayIE" },
+	{ "RoutedMode", "ScreenSourceAddress" },
+	{ "RoutedMode", "SendReleaseCompleteOnDRQ" },
+	{ "RoutedMode", "SetupTimeout" },
+	{ "RoutedMode", "ShowForwarderNumber" },
+	{ "RoutedMode", "SignalTimeout" },
+	{ "RoutedMode", "SocketCleanupTimeout" },
+	{ "RoutedMode", "SupportCallingNATedEndpoints" },
+	{ "RoutedMode", "SupportNATedEndpoints" },
+	{ "RoutedMode", "TcpKeepAlive" },
+	{ "RoutedMode", "TranslateFacility" },
+	{ "RoutedMode", "TranslateReceivedQ931Cause" },
+	{ "RoutedMode", "TreatUnregisteredNAT" },
+	{ "RoutedMode", "TranslateSentQ931Cause" },
+	{ "RoutedMode", "UseProvisionalRespToH245Tunneling" },
+	{ "Routing::CatchAll", "CatchAllAlias" },
+	{ "Routing::CatchAll", "CatchAllIP" },
+	{ "Routing::DNS", "ResolveNonLocalLRQ" },
+	{ "Routing::Sql", "Database" },
+	{ "Routing::Sql", "Driver" },
+	{ "Routing::Sql", "Host" },
+	{ "Routing::Sql", "Library" },
+	{ "Routing::Sql", "MinPoolSize" },
+	{ "Routing::Sql", "Password" },
+	{ "Routing::Sql", "Query" },
+	{ "Routing::Sql", "Username" },
+	{ "SQLAcct", "AlertQuery" },
+	{ "SQLAcct", "Database" },
+	{ "SQLAcct", "Driver" },
+	{ "SQLAcct", "Host" },
+	{ "SQLAcct", "Library" },
+	{ "SQLAcct", "MinPoolSize" },
+	{ "SQLAcct", "Password" },
+	{ "SQLAcct", "RegisterQuery" },
+	{ "SQLAcct", "StartQuery" },
+	{ "SQLAcct", "StartQueryAlt" },
+	{ "SQLAcct", "StopQuery" },
+	{ "SQLAcct", "StopQueryAlt" },
+	{ "SQLAcct", "TimestampFormat" },
+	{ "SQLAcct", "UnregisterQuery" },
+	{ "SQLAcct", "UpdateQuery" },
+	{ "SQLAcct", "Username" },
+	{ "SQLAliasAuth", "CacheTimeout" },
+	{ "SQLAliasAuth", "Database" },
+	{ "SQLAliasAuth", "Driver" },
+	{ "SQLAliasAuth", "Host" },
+	{ "SQLAliasAuth", "Library" },
+	{ "SQLAliasAuth", "MinPoolSize" },
+	{ "SQLAliasAuth", "Password" },
+	{ "SQLAliasAuth", "Query" },
+	{ "SQLAliasAuth", "Table" },
+	{ "SQLAliasAuth", "Username" },
+	{ "SQLAuth", "CacheTimeout" },
+	{ "SQLAuth", "CallQuery" },
+	{ "SQLAuth", "Database" },
+	{ "SQLAuth", "Driver" },
+	{ "SQLAuth", "Host" },
+	{ "SQLAuth", "Library" },
+	{ "SQLAuth", "MinPoolSize" },
+	{ "SQLAuth", "NbQuery" },
+	{ "SQLAuth", "Password" },
+	{ "SQLAuth", "RegQuery" },
+	{ "SQLAuth", "Username" },
+	{ "SQLConfig", "ConfigQuery" },
+	{ "SQLConfig", "Database" },
+	{ "SQLConfig", "Driver" },
+	{ "SQLConfig", "GWPrefixesQuery" },
+	{ "SQLConfig", "Host" },
+	{ "SQLConfig", "Library" },
+	{ "SQLConfig", "NeighborsQuery" },
+	{ "SQLConfig", "Password" },
+	{ "SQLConfig", "PermanentEndpointsQuery" },
+	{ "SQLConfig", "RewriteAliasQuery" },
+	{ "SQLConfig", "RewriteE164Query" },
+	{ "SQLConfig", "Username" },
+	{ "SQLPasswordAuth", "CacheTimeout" },
+	{ "SQLPasswordAuth", "Database" },
+	{ "SQLPasswordAuth", "Driver" },
+	{ "SQLPasswordAuth", "Host" },
+	{ "SQLPasswordAuth", "Library" },
+	{ "SQLPasswordAuth", "MinPoolSize" },
+	{ "SQLPasswordAuth", "Password" },
+	{ "SQLPasswordAuth", "Query" },
+	{ "SQLPasswordAuth", "Username" },
+	{ "StatusAcct", "ConnectEvent" },
+	{ "StatusAcct", "StartEvent" },
+	{ "StatusAcct", "StopEvent" },
+	{ "StatusAcct", "TimestampFormat" },
+	{ "StatusAcct", "UpdateEvent" },
+	{ "SyslogAcct", "ConnectEvent" },
+	{ "SyslogAcct", "StartEvent" },
+	{ "SyslogAcct", "StopEvent" },
+	{ "SyslogAcct", "TimestampFormat" },
+	{ "SyslogAcct", "UpdateEvent" },
+
+	// ignore name partially to check
+	{ "EP::", "CalledTypeOfNumber" },
+	{ "EP::", "CallingTypeOfNumber" },
+	{ "EP::", "Capacity" },
+	{ "EP::", "DisableH46018" },
+	{ "EP::", "GatewayPrefixes" },
+	{ "EP::", "GatewayPriority" },
+	{ "EP::", "MaxBandwidth" },
+	{ "EP::", "PrefixCapacities" },
+	{ "EP::", "Proxy" },
+	{ "EP::", "TranslateReceivedQ931Cause" },
+	{ "EP::", "TranslateSentQ931Cause" },
+	{ "Neighbor::", "AcceptForwardedLRQ" },
+	{ "Neighbor::", "AcceptPrefixes" },
+	{ "Neighbor::", "Dynamic" },
+	{ "Neighbor::", "ForwardHopCount" },
+	{ "Neighbor::", "ForwardLRQ" },
+	{ "Neighbor::", "ForwardResponse" },
+	{ "Neighbor::", "GatekeeperIdentifier" },
+	{ "Neighbor::", "Host" },
+	{ "Neighbor::", "Password" },
+	{ "Neighbor::", "SendPrefixes" },
+	{ "Neighbor::", "UseH46018" },
+
+	// uncheckable sections
+	{ "CapacityControl", "*" },
+	{ "Endpoint::RewriteE164", "*" },
+	{ "FileIPAuth", "*" },
+	{ "GkStatus::Auth", "*" },
+	{ "H225toQ931", "*" },
+	{ "ModeSelection", "*" },
+	{ "NATedEndpoints", "*" },
+	{ "PrefixAuth", "*" },
+	{ "RasSrv::AlternateGatekeeper", "*" },
+	{ "RasSrv::AssignedAlias", "*" },
+	{ "RasSrv::AssignedGatekeeper", "*" },
+	{ "RasSrv::GWPrefixes", "*" },
+	{ "RasSrv::GWRewriteE164", "*" },
+	{ "RasSrv::Neighbors", "*" },
+	{ "RasSrv::PermanentEndpoints", "*" },
+	{ "RasSrv::RRQAuth", "*" },
+	{ "RasSrv::RewriteAlias", "*" },
+	{ "RasSrv::RewriteE164", "*" },
+	{ "ReplyToRasAddress", "*" },
+	{ "RewriteCLI", "*" },
+	{ "Routing::Explicit", "*" },
+	{ "Routing::NumberAnalysis", "*" },
+	{ "RoutingPolicy", "*" },
+	{ "RoutingPolicy::OnARQ", "*" },
+	{ "RoutingPolicy::OnFacility", "*" },
+	{ "RoutingPolicy::OnLRQ", "*" },
+	{ "RoutingPolicy::OnSetup", "*" },
+	{ "SimplePasswordAuth", "*" },
+
+	{ NULL }	// the end
+};
+
+bool CheckConfig(PConfig * cfg, const PString & mainsection)
 {
-	bool result = true;
+	unsigned warnings = 0;
+	bool mainsectionfound = false;
 	PStringList sections = cfg->GetSections();
 	for (PINDEX i = 0; i < sections.GetSize(); ++i) {
-		PString sec = sections[i];
-		if (sec.Find("RasSvr") == 0) {
-			cerr << "The section " << sec << " should be ";
-			sec.Replace("RasSvr", "RasSrv");
-		       	cerr << sec << '\n';
-			result = false;
+		// check section names
+		PCaselessString sect = sections[i];
+		if ((sect.Left(1) == ";") || (sect.Left(1) == "#")) {
+			continue;
+		}
+		if (sect.Left(4) == "EP::") {
+			sect = "EP::";
+		}
+		if (sect.Left(10) == "Neighbor::") {
+			sect = "Neighbor::";
+		}
+		if (sect == mainsection) {
+			mainsectionfound = true;
+		}
+		const char * ks = NULL;
+		unsigned j = 0;
+		bool found = false;
+		bool section_checkable = true;
+		while ((ks = KnowConfigEntries[j][0])) {
+			if (sect == ks) {
+				found = true;
+				section_checkable = (PString(KnowConfigEntries[j][1]) != "*");
+				break;
+			}
+			j++;
+		}
+		if (!found) {
+			PTRACE(0, "WARNING: Config section [" << sect << "] unknown");
+			warnings++;
+		} else if (!section_checkable) {
+			//PTRACE(0, "Section " << sect << " can't be checked in detail");
+		} else {
+			// check all entries in this section
+			PStringToString entries = cfg->GetAllKeyValues(sect);
+			for (PINDEX j = 0; j < entries.GetSize(); j++) {
+				PCaselessString key = entries.GetKeyAt(j);
+				PString value = entries.GetDataAt(j);
+				if (value.IsEmpty()) {
+					PTRACE(0, "WARNING: Empty switch: [" << sect << "] " << key << "=");
+				}
+				unsigned k = 0;
+				bool entry_found = false;
+				while ((ks = KnowConfigEntries[k][0])) {
+					const char * ke = KnowConfigEntries[k][1];
+					k++;
+					if ((sect == ks) && (key == ke)) {
+						entry_found = true;
+						break;
+					}
+				}
+				if (!entry_found) {
+					PTRACE(0, "WARNING: Config entry [" << sect << "] " << key << "=" << value << " unknown");
+					warnings++;
+				}
+			}
 		}
 	}
-	if (!result)
-		cerr << endl;
-	return result;
+	if (!mainsectionfound) {
+		PTRACE(0, "WARNING: This doesn't look like a GNU Gatekeeper configuration file!");
+	}
+
+	return (warnings == 0);
 }
 
 // due to some unknown reason (PWLib bug?),
@@ -128,7 +600,7 @@ void ReloadHandler()
 	// only one thread must do this
 	if (ReloadMutex.WillBlock())
 		return;
-	
+
 	/*
 	** Enter critical Section
 	*/
@@ -163,7 +635,7 @@ void ReloadHandler()
 	RasServer::Instance()->LoadConfig();
 
 	Gatekeeper::EnableLogFileRotation();
-	
+
 	ConfigReloadMutex.EndWrite();
 
 	/*
@@ -175,7 +647,7 @@ void ReloadHandler()
 	** Leave critical Section
 	*/
 	// give other threads the chance to pass by this handler
-	PProcess::Sleep(500);
+	PThread::Sleep(500);
 }
 
 #ifdef _WIN32
@@ -183,7 +655,7 @@ void ReloadHandler()
 BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 {
 	PString eventName = "CTRL_UNKNOWN_EVENT";
-	
+
 	if( dwCtrlType == CTRL_LOGOFF_EVENT ) {
 		eventName = "CTRL_LOGOFF_EVENT";
 #if PTRACING
@@ -192,7 +664,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 		// prevent shut down
 		return FALSE;
 	}
-	
+
 	if( dwCtrlType == CTRL_C_EVENT )
 		eventName = "CTRL_C_EVENT";
 	else if( dwCtrlType == CTRL_BREAK_EVENT )
@@ -217,7 +689,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 	// clean-up time - This may at times not be sufficient
 	// for the GK to shut down in an organized fashion. The
 	// only safe way to handle this, is to remove the
-	// 'Close' menu item from the System menu and we will 
+	// 'Close' menu item from the System menu and we will
 	// never have to deal with this event again.
 	if( dwCtrlType == CTRL_CLOSE_EVENT )
 		WaitForSingleObject(GetCurrentProcess(), 15000);
@@ -269,12 +741,16 @@ bool Gatekeeper::SetUserAndGroup(const PString &username)
 	struct passwd *userptr = ::getpwnam(username);
 #endif
 
-	return userptr && userptr->pw_name 
+	return userptr && userptr->pw_name
 		&& (::setgid(userptr->pw_gid) == 0) && (::setuid(userptr->pw_uid) == 0);
 }
 
 void UnixShutdownHandler(int sig)
 {
+//#ifdef P_MACOSX
+//	RasServer::Instance()->Stop();
+//	_exit(0);
+//#endif
 	if (ShutdownMutex.WillBlock() || !RasServer::Instance()->IsRunning())
 		return;
 	PWaitAndSignal shutdown(ShutdownMutex);
@@ -313,7 +789,7 @@ Gatekeeper::Gatekeeper(const char * _manuf,
 	// save original arguments
 	for (int i = 0; i < GetArguments().GetCount(); i++) {
 		savedArguments += GetArguments().GetParameter(i);
-		savedArguments += " " ;
+		savedArguments += " ";
 	}
 #ifndef _WIN32
 	// set startup arguments for service process
@@ -350,8 +826,8 @@ void Gatekeeper::Terminate()
 		return;
 	PWaitAndSignal shutdown(ShutdownMutex);
 	RasServer::Instance()->Stop();
-	// TODO: wait for termination here
-	PProcess::Sleep(10 * 1000);	// sleep 10 sec
+	// wait for termination
+	PThread::Sleep(10 * 1000);	// sleep 10 sec
 }
 
 PBoolean Gatekeeper::OnPause()
@@ -405,7 +881,7 @@ bool Gatekeeper::InitHandlers(const PArgList& args)
 	SetConsoleCtrlHandler(WinCtrlHandlerProc, TRUE);
 #else
 	struct sigaction sigact;
-	
+
 	memset(&sigact, 0, sizeof(sigact));
 	sigact.sa_handler = UnixShutdownHandler;
 	sigemptyset(&sigact.sa_mask);
@@ -414,7 +890,7 @@ bool Gatekeeper::InitHandlers(const PArgList& args)
 	sigaddset(&sigact.sa_mask, SIGQUIT);
 	sigaddset(&sigact.sa_mask, SIGHUP);
 	sigaddset(&sigact.sa_mask, SIGUSR1);
-	
+
 	sigaction(SIGTERM, &sigact, NULL);
 	sigaction(SIGINT, &sigact, NULL);
 	sigaction(SIGQUIT, &sigact, NULL);
@@ -432,7 +908,7 @@ bool Gatekeeper::InitHandlers(const PArgList& args)
 	sigemptyset(&sigact.sa_mask);
 	sigaddset(&sigact.sa_mask, SIGHUP);
 	sigaddset(&sigact.sa_mask, SIGUSR1);
-	
+
 	sigaction(SIGHUP, &sigact, NULL);
 	sigaction(SIGUSR1, &sigact, NULL);
 
@@ -460,7 +936,7 @@ bool Gatekeeper::InitLogging(const PArgList &args)
 		}
 	}
 #endif
-	
+
 	return TRUE;
 }
 
@@ -471,24 +947,24 @@ bool Gatekeeper::InitConfig(const PArgList &args)
 	PFilePath fp;
 	PString section("Gatekeeper::Main");
 
-	if (args.HasOption('c')) 
+	if (args.HasOption('c'))
 		fp = PFilePath(args.GetOptionString('c'));
 	else
 		fp = "gatekeeper.ini";
+	if (!PFile::Exists(fp)) {
+		cerr << "WARNING: Config file " << fp << " doesn't exist!"
+			 << " Use the -c switch to specify the config file.\n" << endl;
+	}
 
-	if (args.HasOption('s')) 
+	if (args.HasOption('s'))
 		section = args.GetOptionString('s');
 
 	Toolkit::Instance()->SetConfig(fp, section);
 
-	if( (GkConfig()->GetInteger("Fortytwo")  != 42) &&
-		(GkConfig()->GetInteger("Fourtytwo") != 42)) { 
-		cerr << "WARNING: No config file found!\n"
-			 << "- Use the -c switch to specify the config file.\n"
-			 << "- Make sure the line 'Fortytwo=42' is present in the '[Gatekeeper::Main]' section.\n" << endl;
-	}
-	
-	return CheckSectionName(GkConfig());
+	// check config for unknown options (only warn about them)
+	CheckConfig(GkConfig(), section);
+
+	return true;
 }
 
 
@@ -541,7 +1017,7 @@ void Gatekeeper::Main()
 			const PCaselessString s = args.GetOptionString("core");
 			rlim_t v = (s == "unlimited" ? RLIM_INFINITY : (rlim_t)s.AsInteger());
 			rlim.rlim_cur = v;
-			if (setrlimit(RLIMIT_CORE, &rlim) != 0) 
+			if (setrlimit(RLIMIT_CORE, &rlim) != 0)
 				cout << "Could not set current core file size to " << v << " : error = " << errno << endl;
 			else {
 				getrlimit(RLIMIT_CORE, &rlim);
@@ -596,12 +1072,11 @@ void Gatekeeper::Main()
 	if (!fake_cmdline.IsEmpty()) {
 		PArgList fake_args(fake_cmdline);
 		fake_args.Parse(GetArgumentsParseString());
-		if (!InitLogging(fake_args))
-			return;
+		InitLogging(fake_args);
 	}
 
 	EnableLogFileRotation();
-	
+
 	PString welcome("GNU Gatekeeper with ID '" + Toolkit::GKName() + "' started\n" + Toolkit::GKVersion());
 	cout << welcome << '\n';
 	PTRACE(1, welcome);
@@ -646,11 +1121,13 @@ void Gatekeeper::Main()
 		GKcapacity = args.GetOptionString('b').AsInteger();
 	else
 		GKcapacity = GkConfig()->GetInteger("TotalBandwidth", -1);
+	if (GKcapacity == 0)
+		GKcapacity = -1;	// turn bw management off for 0, too
 	CallTable::Instance()->SetTotalBandwidth(GKcapacity);
 	if (GKcapacity < 0)
-		PTRACE(2, "GK\tDisable Bandwidth Management");
+		PTRACE(2, "GK\tTotal bandwidth not limited");
 	else
-		PTRACE(2, "GK\tAvailable Bandwidth " << GKcapacity);
+		PTRACE(2, "GK\tAvailable bandwidth: " << GKcapacity);
 
 	// read timeToLive from command line
 	if (args.HasOption('l'))
@@ -658,7 +1135,7 @@ void Gatekeeper::Main()
 	else
 		SoftPBX::TimeToLive = GkConfig()->GetInteger("TimeToLive", -1);
 	PTRACE(2, "GK\tTimeToLive for Registrations: " << SoftPBX::TimeToLive);
-  
+
 	RasServer *RasSrv = RasServer::Instance();
 
 	// read signaling method from commandline
@@ -668,16 +1145,16 @@ void Gatekeeper::Main()
 		RasSrv->SetRoutedMode(false, false);
 	else
 		RasSrv->SetRoutedMode();
-   
+
 	// Load ENUM servers
 	RasSrv->SetENUMServers();
 
 	// Load RDS servers
-	RasSrv->SetRDSServers();	
+	RasSrv->SetRDSServers();
 
 #if defined(_WIN32)
-	// 1) prevent CTRL_CLOSE_EVENT, CTRL_LOGOFF_EVENT and CTRL_SHUTDOWN_EVENT 
-	//    dialog box from being displayed. 
+	// 1) prevent CTRL_CLOSE_EVENT, CTRL_LOGOFF_EVENT and CTRL_SHUTDOWN_EVENT
+	//    dialog box from being displayed.
 	// 2) set process shutdown priority - we want as much time as possible
 	//    for tasks, such as unregistering endpoints during the shut down process.
 	//    0x3ff is a maximimum permitted for windows app
@@ -717,7 +1194,7 @@ void Gatekeeper::GetRotateInterval(
 	)
 {
 	PString s;
-	
+
 	if (m_rotateInterval == Hourly)
 		m_rotateMinute = cfg.GetInteger(section, "RotateTime", 59);
 	else {
@@ -726,15 +1203,15 @@ void Gatekeeper::GetRotateInterval(
 		m_rotateMinute = 0;
 		if (s.Find(':') != P_MAX_INDEX)
 			m_rotateMinute = s.Mid(s.Find(':') + 1).AsInteger();
-			
+
 		if (m_rotateHour < 0 || m_rotateHour > 23 || m_rotateMinute < 0
 			|| m_rotateMinute > 59) {
-			PTRACE(1, "GK\tInvalid log file RotateTime specified: " << s);
+			PTRACEX(1, "GK\tInvalid log file RotateTime specified: " << s);
 			m_rotateMinute = 59;
 			m_rotateHour = 0;
 		}
 	}
-			
+
 	if (m_rotateInterval == Weekly)	{
 		s = cfg.GetString(section, "RotateDay", "Sun");
 		if (strspn(s, "0123456") == (size_t)s.GetLength()) {
@@ -752,13 +1229,13 @@ void Gatekeeper::GetRotateInterval(
 			m_rotateDay = (i != dayNames.end()) ? i->second : -1;
 		}
 		if (m_rotateDay < 0 || m_rotateDay > 6) {
-			PTRACE(1, "GK\tInvalid log file RotateDay specified: " << s);
+			PTRACEX(1, "GK\tInvalid log file RotateDay specified: " << s);
 			m_rotateDay = 0;
 		}
 	} else if (m_rotateInterval == Monthly) {
 		m_rotateDay = cfg.GetInteger(section, "RotateDay", 1);
 		if (m_rotateDay < 1 || m_rotateDay > 31) {
-			PTRACE(1, "GK\tInvalid RotateDay specified: " 
+			PTRACEX(1, "GK\tInvalid RotateDay specified: "
 				<< cfg.GetString(section, "RotateDay", "")
 				);
 			m_rotateDay = 1;
@@ -771,29 +1248,29 @@ void Gatekeeper::EnableLogFileRotation(
 	)
 {
 	PWaitAndSignal lock(m_logFileMutex);
-	
+
 	if (m_rotateTimer != GkTimerManager::INVALID_HANDLE) {
 		Toolkit::Instance()->GetTimerManager()->UnregisterTimer(m_rotateTimer);
 		m_rotateTimer = GkTimerManager::INVALID_HANDLE;
 	}
-	
+
 	if (!enable)
 		return;
-		
+
 	PConfig* const config = GkConfig();
-	// determine rotation type (by lines, by size, by time)	
+	// determine rotation type (by lines, by size, by time)
 	const PString rotateCondition = config->GetString(
 		logConfigSectionName, "Rotate", ""
 		).Trim();
 	if (rotateCondition.IsEmpty())
 		return;
-		
+
 	for (int i = 0; i < RotationIntervalMax; i++)
 		if (strcasecmp(rotateCondition, m_intervalNames[i]) == 0)
 			m_rotateInterval = i;
 
 	if (m_rotateInterval < 0 || m_rotateInterval >= RotationIntervalMax) {
-		PTRACE(1, "GK\tUnsupported log file rotation method: " 
+		PTRACEX(1, "GK\tUnsupported log file rotation method: "
 			<< rotateCondition << " - rotation disabled"
 			);
 		return;
@@ -804,11 +1281,11 @@ void Gatekeeper::EnableLogFileRotation(
 
 	// setup rotation timer in case of time based rotation
 	PTime now, rotateTime;
-			
-	switch (m_rotateInterval) 
+
+	switch (m_rotateInterval)
 	{
 	case Hourly:
-		rotateTime = PTime(0, m_rotateMinute, now.GetHour(), now.GetDay(), 
+		rotateTime = PTime(0, m_rotateMinute, now.GetHour(), now.GetDay(),
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		if (rotateTime <= now)
@@ -816,13 +1293,13 @@ void Gatekeeper::EnableLogFileRotation(
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 			&Gatekeeper::RotateOnTimer, rotateTime, 60*60
 			);
-		PTRACE(5, "GK\tHourly log file rotation enabled (first "
+		PTRACEX(5, "GK\tHourly log file rotation enabled (first "
 			"rotation scheduled at " << rotateTime
 			);
 		break;
-		
+
 	case Daily:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(), 
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(),
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		if (rotateTime <= now)
@@ -830,13 +1307,13 @@ void Gatekeeper::EnableLogFileRotation(
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 			&Gatekeeper::RotateOnTimer, rotateTime, 60*60*24
 			);
-		PTRACE(5, "GK\tDaily rotation enabled (first rotation scheduled at " 
+		PTRACEX(5, "GK\tDaily rotation enabled (first rotation scheduled at "
 			<< rotateTime
 			);
 		break;
-		
+
 	case Weekly:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(), 
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(),
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		if (rotateTime.GetDayOfWeek() < m_rotateDay)
@@ -852,35 +1329,35 @@ void Gatekeeper::EnableLogFileRotation(
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 			&Gatekeeper::RotateOnTimer, rotateTime, 60*60*24*7
 			);
-		PTRACE(5, "GK\tWeekly rotation enabled (first rotation scheduled at " 
+		PTRACEX(5, "GK\tWeekly rotation enabled (first rotation scheduled at "
 			<< rotateTime
-			);
+		      );
 		break;
-		
+
 	case Monthly:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1, 
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1,
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		rotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - 1);
-		while (rotateTime.GetMonth() != now.GetMonth())				
+		while (rotateTime.GetMonth() != now.GetMonth())
 			rotateTime -= PTimeInterval(0, 0, 0, 0, 1); // 1 day
 
 		if (rotateTime <= now) {
-			rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1, 
-				now.GetMonth() + (now.GetMonth() == 12 ? -11 : 1), 
-				now.GetYear() + (now.GetMonth() == 12 ? 1 : 0), 
+			rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1,
+				now.GetMonth() + (now.GetMonth() == 12 ? -11 : 1),
+				now.GetYear() + (now.GetMonth() == 12 ? 1 : 0),
 				now.GetTimeZone()
 				);
 			const int month = rotateTime.GetMonth();
 			rotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - 1);
-			while (rotateTime.GetMonth() != month)				
+			while (rotateTime.GetMonth() != month)
 				rotateTime -= PTimeInterval(0, 0, 0, 0, 1); // 1 day
 		}
-					
+
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 			&Gatekeeper::RotateOnTimer, rotateTime
 			);
-		PTRACE(5, "GK\tMonthly rotation enabled (first rotation scheduled at " 
+		PTRACEX(5, "GK\tMonthly rotation enabled (first rotation scheduled at "
 			<< rotateTime
 			);
 		break;
@@ -896,21 +1373,21 @@ void Gatekeeper::RotateOnTimer(
 		// setup next time for one-shot timer
 		const PTime& rotateTime = timer->GetExpirationTime();
 		PTime newRotateTime(rotateTime.GetSecond(), rotateTime.GetMinute(),
-			rotateTime.GetHour(), 1, 
-			rotateTime.GetMonth() < 12 ? rotateTime.GetMonth() + 1 : 1, 
+			rotateTime.GetHour(), 1,
+			rotateTime.GetMonth() < 12 ? rotateTime.GetMonth() + 1 : 1,
 			rotateTime.GetMonth() < 12 ? rotateTime.GetYear() : rotateTime.GetYear() + 1,
 			rotateTime.GetTimeZone()
 			);
-	
+
 		newRotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - 1);
-		
+
 		const int month = newRotateTime.GetMonth();
-		while (newRotateTime.GetMonth() != month)				
+		while (newRotateTime.GetMonth() != month)
 			newRotateTime -= PTimeInterval(0, 0, 0, 0, 1); // 1 day
-		
+
 		timer->SetExpirationTime(newRotateTime);
 		timer->SetFired(false);
-	}	
+	}
 	m_logFileMutex.Signal();
 	RotateLogFile();
 }
@@ -921,24 +1398,24 @@ bool Gatekeeper::SetLogFilename(
 {
 	if (filename.IsEmpty())
 		return false;
-		
+
 	PWaitAndSignal lock(m_logFileMutex);
 	if (!m_logFilename && m_logFile != NULL && m_logFile->IsOpen()
 		&& m_logFilename == filename)
 		return true;
 
 	if (m_logFile) {
-		PTRACE(1, "GK\tLogging redirected to the file '" << filename << '\'');
+		PTRACEX(1, "GK\tLogging redirected to the file '" << filename << '\'');
 		EnableLogFileRotation(false);
 	}
-	
+
 	PTrace::SetStream(&cerr);
-	
+
 #ifndef hasDeletingSetStream
 	delete m_logFile;
 #endif
 	m_logFile = NULL;
-	
+
 	m_logFilename = filename;
 	m_logFile = new PTextFile(m_logFilename, PFile::WriteOnly, PFile::Create);
 	if (!m_logFile->IsOpen()) {
@@ -948,15 +1425,15 @@ bool Gatekeeper::SetLogFilename(
 	}
 	m_logFile->SetPosition(0, PFile::End);
 	PTrace::SetStream(m_logFile);
-	return true;	
+	return true;
 }
-		
+
 bool Gatekeeper::RotateLogFile()
 {
 	PWaitAndSignal lock(m_logFileMutex);
 
 	if (m_logFile) {
-		PTRACE(1, "GK\tLogging closed (log file rotation)");
+		PTRACEX(1, "GK\tLogging closed (log file rotation)");
 		PTrace::SetStream(&cerr); // redirect to cerr
 #ifndef hasDeletingSetStream
 		delete m_logFile;
@@ -966,8 +1443,8 @@ bool Gatekeeper::RotateLogFile()
 
 	if (m_logFilename.IsEmpty())
 		return false;
-	
-	PFile* const oldLogFile = new PTextFile(m_logFilename, PFile::WriteOnly, 
+
+	PFile* const oldLogFile = new PTextFile(m_logFilename, PFile::WriteOnly,
 		PFile::MustExist
 		);
 	if (oldLogFile->IsOpen()) {
@@ -983,7 +1460,7 @@ bool Gatekeeper::RotateLogFile()
 		oldLogFile->Move(oldLogFile->GetFilePath(), filename);
 	}
 	delete oldLogFile;
-		
+
 	m_logFile = new PTextFile(m_logFilename, PFile::WriteOnly, PFile::Create);
 	if (!m_logFile->IsOpen()) {
 		cerr << "Warning: could not open the log file \""
@@ -995,16 +1472,16 @@ bool Gatekeeper::RotateLogFile()
 
 	m_logFile->SetPosition(0, PFile::End);
 	PTrace::SetStream(m_logFile);
-	PTRACE(1, "GK\tLogging restarted.");
+	PTRACEX(1, "GK\tLogging restarted.");
 	return true;
 }
-	
+
 bool Gatekeeper::ReopenLogFile()
 {
 	PWaitAndSignal lock(m_logFileMutex);
 
 	if (m_logFile) {
-		PTRACE(1, "GK\tLogging closed (reopen log file)");
+		PTRACEX(1, "GK\tLogging closed (reopen log file)");
 		PTrace::SetStream(&cerr); // redirect to cerr
 #ifndef hasDeletingSetStream
 		delete m_logFile;
@@ -1014,16 +1491,16 @@ bool Gatekeeper::ReopenLogFile()
 
 	if (m_logFilename.IsEmpty())
 		return false;
-	
-	m_logFile = new PTextFile(m_logFilename, PFile::WriteOnly, 
+
+	m_logFile = new PTextFile(m_logFilename, PFile::WriteOnly,
 		PFile::MustExist
 		);
 	if (!m_logFile->IsOpen()) {
 		delete m_logFile;
 		m_logFile = NULL;
 	}
-	
-	if (m_logFile == NULL) {	
+
+	if (m_logFile == NULL) {
 		m_logFile = new PTextFile(m_logFilename, PFile::WriteOnly, PFile::Create);
 		if (!m_logFile->IsOpen()) {
 			cerr << "Warning: could not open the log file \""
@@ -1035,7 +1512,7 @@ bool Gatekeeper::ReopenLogFile()
 	}
 	m_logFile->SetPosition(0, PFile::End);
 	PTrace::SetStream(m_logFile);
-	PTRACE(1, "GK\tLogging restarted");
+	PTRACEX(1, "GK\tLogging restarted");
 	return true;
 }
 
@@ -1044,7 +1521,7 @@ void Gatekeeper::CloseLogFile()
 	PWaitAndSignal lock(m_logFileMutex);
 
 	if (m_logFile) {
-		PTRACE(1, "GK\tLogging closed");
+		PTRACEX(1, "GK\tLogging closed");
 	}
 	PTrace::SetStream(&cerr);
 #ifndef hasDeletingSetStream
