@@ -2,16 +2,18 @@
 //
 // gk.h gatekeeper process
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 
 #ifndef GK_H
-#define GK_H "@(#) $Id: gk.h,v 1.32 2009/02/09 13:25:59 willamowius Exp $"
+#define GK_H "@(#) $Id: gk.h,v 1.36 2010/10/25 15:01:51 willamowius Exp $"
 
 #include "config.h"
 #include "version.h"
@@ -27,21 +29,23 @@
 #define GNUGK_NAME	"Gatekeeper"
 #endif
 
+extern PSemaphore ShutdownMutex;
+
 class GkTimer;
 
 #ifdef COMPILE_AS_SERVICE
-class Gatekeeper : public PServiceProcess 
+class Gatekeeper : public PServiceProcess
 {
 	PCLASSINFO(Gatekeeper, PServiceProcess)
 #else
-class Gatekeeper : public PProcess 
+class Gatekeeper : public PProcess
 {
 	PCLASSINFO(Gatekeeper, PProcess)
 #endif
  public:
 	Gatekeeper
-		(const char * _manuf = "GNU", 
-		 const char * _name = GNUGK_NAME, 
+		(const char * _manuf = "GNU",
+		 const char * _name = GNUGK_NAME,
 		 WORD _majorVersion = GNUGK_MAJOR_VERSION,
 		 WORD _minorVersion = GNUGK_MINOR_VERSION,
 		 CodeStatus _status = GNUGK_BUILD_TYPE,
@@ -57,7 +61,7 @@ class Gatekeeper : public PProcess
 	virtual void OnContinue();
     virtual void OnControl();
 #endif
-    
+
 #if PTRACING
 	enum RotationIntervals {
 		Hourly,
@@ -92,9 +96,9 @@ class Gatekeeper : public PProcess
 	/** returns the template string for which the cmommand line is parsed */
 	virtual const PString GetArgumentsParseString() const;
 
-	/**@name Initialization 
+	/**@name Initialization
 	 * A sequence of virtual initialization methods is called from #Main#
-	 * before the fun starts. 
+	 * before the fun starts.
 	 * Every method may return #FALSE# to abort #Main# and end the program.
 	 */
 	//@{
