@@ -5,153 +5,12 @@
  * support for accounting to the gatekeeper.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
+ * Copyright (c) 2005-2010, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
- *
- * $Log: gkacct.cxx,v $
- * Revision 1.42  2010/02/28 22:35:04  willamowius
- * fix spelling
- *
- * Revision 1.41  2009/12/02 15:02:46  willamowius
- * select the new accounting events separately
- *
- * Revision 1.40  2009/12/02 14:32:13  willamowius
- * add new accounting events: AcctAlert, AcctRegister, AcctUnregister
- *
- * Revision 1.39  2009/08/19 09:53:28  willamowius
- * SqlAuth can set a ClientAuthId, new status port command ResetCallCounters
- *
- * Revision 1.38  2009/05/24 20:48:26  willamowius
- * remove hacks for VC6 which isn't supported any more since quite a while
- *
- * Revision 1.37  2009/02/09 13:25:59  willamowius
- * typo in comment
- *
- * Revision 1.36  2008/07/10 08:21:20  willamowius
- * avoid gcc 4.3.x warnings
- *
- * Revision 1.35  2008/05/08 08:22:40  zvision
- * New GetAuthInfo,GetAcctInfo status port commands
- *
- * Revision 1.34  2008/01/18 12:17:54  willamowius
- * add accounting variable %{cause-translated}
- *
- * Revision 1.33  2007/09/26 14:57:54  shorne
- * Added CallLinkage SQL variable
- *
- * Revision 1.32  2007/09/11 01:02:29  willamowius
- * clean up includes
- *
- * Revision 1.31  2007/07/14 09:47:42  willamowius
- * new accounting variable %{bandwidth}
- *
- * Revision 1.30  2006/04/14 13:56:19  willamowius
- * call failover code merged
- *
- * Revision 1.3  2006/01/19 11:48:45  zvision
- * New media-ip and codec accounting variables for fast start calls
- *
- * Revision 1.2  2005/12/15 12:40:39  willamowius
- * new accounting variables
- *
- * Revision 1.1.1.1  2005/11/21 20:20:00  willamowius
- *
- *
- * Revision 1.4  2005/11/15 19:52:56  jan
- * Michal v1 (works, but on in routed, not proxy mode)
- *
- * Revision 1.29  2005/05/19 16:41:16  zvision
- * Solaris need explicit std::map
- *
- * Revision 1.28  2005/04/24 16:39:44  zvision
- * MSVC6.0 compatibility fixed
- *
- * Revision 1.27  2005/03/18 14:54:47  zvision
- * Various accounting variables ported from 2.0 branch
- *
- * Revision 1.26  2005/02/02 22:16:46  zvision
- * Different Username was presented during auth and acct steps in some case
- *
- * Revision 1.25  2005/01/17 08:42:24  zvision
- * Compilation error fixed (missing std:: prefix before map)
- *
- * Revision 1.24  2005/01/16 22:37:15  zvision
- * Redundant config reload mutex removed
- *
- * Revision 1.23  2005/01/12 18:01:34  willamowius
- * small cleanup
- *
- * Revision 1.22  2005/01/12 17:55:00  willamowius
- * fix gkip accounting parameter
- *
- * Revision 1.21  2005/01/10 23:49:06  willamowius
- * provide mechanism for accounting modules to escape the parameters
- *
- * Revision 1.20  2005/01/05 15:42:31  willamowius
- * new accounting event 'connect', parameter substitution unified in parent class
- *
- * Revision 1.19  2005/01/04 17:50:51  willamowius
- * space in trace msg
- *
- * Revision 1.18  2004/11/15 23:57:41  zvision
- * Ability to choose between the original and the rewritten dialed number
- *
- * Revision 1.17  2004/11/10 18:30:41  zvision
- * Ability to customize timestamp strings
- *
- * Revision 1.16  2004/06/28 19:09:33  zvision
- * Monthly CDR files rotation fixes.
- *
- * Revision 1.15  2004/06/28 18:47:58  zvision
- * Fixed a bug in Username determination for unregistered endpoints
- *
- * Revision 1.14  2004/06/25 13:33:18  zvision
- * Better Username, Calling-Station-Id and Called-Station-Id handling.
- * New SetupUnreg option in Gatekeeper::Auth section.
- *
- * Revision 1.13  2004/05/31 22:45:52  zvision
- * Fixed CDR rotation per number of lines
- *
- * Revision 1.12  2004/05/22 12:17:12  zvision
- * Parametrized FileAcct CDR format
- *
- * Revision 1.11  2004/05/12 11:49:08  zvision
- * New flexible CDR file rotation
- *
- * Revision 1.10  2004/04/17 11:43:42  zvision
- * Auth/acct API changes.
- * Header file usage more consistent.
- *
- * Revision 1.9  2003/12/21 00:58:17  zvision
- * FileAcct logger should work fine with the reload command now
- *
- * Revision 1.8  2003/11/01 10:36:34  zvision
- * Fixed missing semicolon. Thanks to Hu Yuxin
- *
- * Revision 1.7  2003/10/31 00:01:23  zvision
- * Improved accounting modules stacking control, optimized radacct/radauth a bit
- *
- * Revision 1.6  2003/10/15 10:16:57  zvision
- * Fixed VC6 compiler warnings. Thanks to Hu Yuxin.
- *
- * Revision 1.5  2003/10/08 12:40:48  zvision
- * Realtime accounting updates added
- *
- * Revision 1.4  2003/09/28 15:45:49  zvision
- * Microsecond field added back to h323-xxx-time attributes
- *
- * Revision 1.3  2003/09/14 21:09:29  zvision
- * Added new FileAcct logger from Tamas Jalsovszky. Thanks!
- * Fixed module stacking. Redesigned API.
- *
- * Revision 1.2  2003/09/12 16:31:16  zvision
- * Accounting initially added to the 2.2 branch
- *
- * Revision 1.1.2.1  2003/06/19 15:36:04  zvision
- * Initial generic accounting support for GNU GK.
+ * with the OpenH323/H323Plus and OpenSSL library.
  *
  */
 
@@ -178,7 +37,7 @@ extern const char* CallTableSection;
 GkAcctLogger::GkAcctLogger(
 	const char* moduleName,
 	const char* cfgSecName
-	) 
+	)
 	: NamedObject(moduleName), m_controlFlag(Required), m_defaultStatus(Fail),
 	m_enabledEvents(AcctAll), m_supportedEvents(AcctNone), m_config(GkConfig()),
 	m_configSectionName(cfgSecName)
@@ -186,7 +45,7 @@ GkAcctLogger::GkAcctLogger(
 	if (m_configSectionName.IsEmpty())
 		m_configSectionName = moduleName;
 		
-	const PStringArray control( 
+	const PStringArray control(
 		m_config->GetString(GkAcctSectionName, moduleName, "").Tokenise(";,")
 		);
 
@@ -295,7 +154,7 @@ void GkAcctLogger::SetupAcctParams(
 		params["gkip"] = interfaces.front().AsString();
 	params["CallId"] = ::AsString(call->GetCallIdentifier().m_guid);
 	params["ConfId"] = ::AsString(call->GetConferenceIdentifier());
-	params["CallLink"] = call->GetCallLinkage();  
+	params["CallLink"] = call->GetCallLinkage();
 	
 	t = call->GetSetupTime();
 	if (t)
@@ -370,6 +229,11 @@ void GkAcctLogger::SetupAcctEndpointParams(
 	params["epid"] = ep->GetEndpointIdentifier().GetValue();
 }
 
+// avoid warning in PTLib object.h
+#if (!_WIN32) && (GCC_VERSION >= 40400)
+#pragma GCC diagnostic ignored "-Wstrict-overflow"
+#endif
+
 PString GkAcctLogger::ReplaceAcctParams(
 	/// parametrized CDR string
 	const PString& cdrStr,
@@ -440,7 +304,7 @@ PString GkAcctLogger::GetUsername(
 	) const
 {
 	if (!call)
-		return PString();
+		return PString::Empty();
 		
 	const endptr callingEP = call->GetCallingParty();
 	PString username;
@@ -451,7 +315,7 @@ PString GkAcctLogger::GetUsername(
 			| AliasAddressTagMask(H225_AliasAddress::e_url_ID)
 		);
 			
-	if (callingEP && (username.IsEmpty() 
+	if (callingEP && (username.IsEmpty()
 			|| FindAlias(callingEP->GetAliases(), username) == P_MAX_INDEX))
 		username = GetBestAliasAddressString(callingEP->GetAliases(), false,
 			AliasAddressTagMask(H225_AliasAddress::e_h323_ID),
@@ -472,7 +336,7 @@ PString GkAcctLogger::GetUsername(
 	if (username.IsEmpty()) {
 		PIPSocket::Address callingSigAddr;
 		WORD callingSigPort;
-		if (call->GetSrcSignalAddr(callingSigAddr, callingSigPort) 
+		if (call->GetSrcSignalAddr(callingSigAddr, callingSigPort)
 			&& callingSigAddr.IsValid())
 			username = callingSigAddr.AsString();
 	}
@@ -486,7 +350,7 @@ PString GkAcctLogger::GetCallingStationId(
 	) const
 {
 	if (!call)
-		return PString();
+		return PString::Empty();
 
 	PString id = call->GetCallingStationId();
 	if (!id)
@@ -510,7 +374,7 @@ PString GkAcctLogger::GetCallingStationId(
 	if (id.IsEmpty()) {
 		PIPSocket::Address callingSigAddr;
 		WORD callingSigPort = 0;
-		if (call->GetSrcSignalAddr(callingSigAddr, callingSigPort) 
+		if (call->GetSrcSignalAddr(callingSigAddr, callingSigPort)
 			&& callingSigAddr.IsValid())
 			id = ::AsString(callingSigAddr, callingSigPort);
 	}
@@ -524,7 +388,7 @@ PString GkAcctLogger::GetCalledStationId(
 	) const
 {
 	if (!call)
-		return PString();
+		return PString::Empty();
 
 	PString id = call->GetCalledStationId();
 	if (!id)
@@ -548,7 +412,7 @@ PString GkAcctLogger::GetCalledStationId(
 	if (id.IsEmpty()) {
 		PIPSocket::Address calledSigAddr;
 		WORD calledSigPort = 0;
-		if (call->GetDestSignalAddr(calledSigAddr, calledSigPort) 
+		if (call->GetDestSignalAddr(calledSigAddr, calledSigPort)
 		 	&& calledSigAddr.IsValid())
 			id = ::AsString(calledSigAddr, calledSigPort);
 	}
@@ -562,7 +426,7 @@ PString GkAcctLogger::GetDialedNumber(
 	) const
 {
 	if (!call)
-		return PString();
+		return PString::Empty();
 
 	PString id = call->GetDialedNumber();
 	if (!id)
@@ -586,7 +450,7 @@ PString GkAcctLogger::GetDialedNumber(
 	if (id.IsEmpty()) {
 		PIPSocket::Address calledSigAddr;
 		WORD calledSigPort = 0;
-		if (call->GetDestSignalAddr(calledSigAddr, calledSigPort) 
+		if (call->GetDestSignalAddr(calledSigAddr, calledSigPort)
 		 	&& calledSigAddr.IsValid())
 			id = ::AsString(calledSigAddr, calledSigPort);
 	}
@@ -604,14 +468,14 @@ const char* const FileAcct::m_intervalNames[] =
 	"Hourly", "Daily", "Weekly", "Monthly"
 };
 
-FileAcct::FileAcct( 
+FileAcct::FileAcct(
 	const char* moduleName,
 	const char* cfgSecName
 	)
 	:
 	GkAcctLogger(moduleName, cfgSecName),
 	m_cdrFile(NULL), m_rotateLines(-1), m_rotateSize(-1), m_rotateInterval(-1),
-	m_rotateMinute(-1), m_rotateHour(-1), m_rotateDay(-1), 
+	m_rotateMinute(-1), m_rotateHour(-1), m_rotateDay(-1),
 	m_rotateTimer(GkTimerManager::INVALID_HANDLE), m_cdrLines(0),
 	m_standardCDRFormat(true)
 {
@@ -619,10 +483,10 @@ FileAcct::FileAcct(
 	
 	m_cdrString = GetConfig()->GetString(GetConfigSectionName(), "CDRString", "");
 	m_standardCDRFormat = Toolkit::AsBool(
-		GetConfig()->GetString(GetConfigSectionName(), "StandardCDRFormat", 
+		GetConfig()->GetString(GetConfigSectionName(), "StandardCDRFormat",
 			m_cdrString.IsEmpty() ? "1" : "0"
 			));
-	m_timestampFormat = GetConfig()->GetString(GetConfigSectionName(), 
+	m_timestampFormat = GetConfig()->GetString(GetConfigSectionName(),
 		"TimestampFormat", ""
 		);
 		
@@ -641,7 +505,7 @@ FileAcct::FileAcct(
 				m_rotateLines *= 1000*1000;
 		} else if (rotateCondition[0] == 'S' || rotateCondition[0] == 's') {
 			// rotate per CDR file size
-			m_rotateSize = rotateCondition.Mid(1).AsInteger(); 
+			m_rotateSize = rotateCondition.Mid(1).AsInteger();
 			if (suffix == 'k' || suffix == 'K')
 				m_rotateSize *= 1024;
 			else if (suffix == 'm' || suffix == 'M')
@@ -680,11 +544,11 @@ FileAcct::FileAcct(
 
 	// setup rotation timer in case of time based rotation
 	PTime now, rotateTime;
-		
-	switch (m_rotateInterval) 
+
+	switch (m_rotateInterval)
 	{
 	case Hourly:
-		rotateTime = PTime(0, m_rotateMinute, now.GetHour(), now.GetDay(), 
+		rotateTime = PTime(0, m_rotateMinute, now.GetHour(), now.GetDay(),
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		if (rotateTime <= now)
@@ -698,7 +562,7 @@ FileAcct::FileAcct(
 		break;
 		
 	case Daily:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(), 
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(),
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		if (rotateTime <= now)
@@ -712,11 +576,11 @@ FileAcct::FileAcct(
 		break;
 		
 	case Weekly:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(), 
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, now.GetDay(),
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		if (rotateTime.GetDayOfWeek() < m_rotateDay)
-			rotateTime += PTimeInterval(0, 0, 0, 0, 
+			rotateTime += PTimeInterval(0, 0, 0, 0,
 				m_rotateDay - rotateTime.GetDayOfWeek()
 				);
 		else if (rotateTime.GetDayOfWeek() > m_rotateDay)
@@ -734,7 +598,7 @@ FileAcct::FileAcct(
 		break;
 		
 	case Monthly:
-		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1, 
+		rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1,
 			now.GetMonth(), now.GetYear(), now.GetTimeZone()
 			);
 		rotateTime += PTimeInterval(0, 0, 0, 0, m_rotateDay - 1);
@@ -742,9 +606,9 @@ FileAcct::FileAcct(
 			rotateTime -= PTimeInterval(0, 0, 0, 0, 1); // 1 day
 
 		if (rotateTime <= now) {
-			rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1, 
-				now.GetMonth() + (now.GetMonth() == 12 ? -11 : 1), 
-				now.GetYear() + (now.GetMonth() == 12 ? 1 : 0), 
+			rotateTime = PTime(0, m_rotateMinute, m_rotateHour, 1,
+				now.GetMonth() + (now.GetMonth() == 12 ? -11 : 1),
+				now.GetYear() + (now.GetMonth() == 12 ? 1 : 0),
 				now.GetTimeZone()
 				);
 			const int month = rotateTime.GetMonth();
@@ -835,7 +699,7 @@ void FileAcct::GetRotateInterval(
 }
 
 GkAcctLogger::Status FileAcct::Log(
-	GkAcctLogger::AcctEvent evt, 
+	GkAcctLogger::AcctEvent evt,
 	const callptr& call
 	)
 {
@@ -861,7 +725,7 @@ GkAcctLogger::Status FileAcct::Log(
 	if (m_cdrFile && m_cdrFile->IsOpen()) {
 		if (m_cdrFile->WriteLine(PString(cdrString))) {
 			PTRACE(5, "GKACCT\t" << GetName() << " - CDR string for event "
-				<< evt << ", call no. " << call->GetCallNumber() 
+				<< evt << ", call no. " << call->GetCallNumber()
 				<< ": " << cdrString
 				);
 			m_cdrLines++;
@@ -920,8 +784,8 @@ void FileAcct::RotateOnTimer(
 		// setup next time for one-shot timer
 		const PTime& rotateTime = timer->GetExpirationTime();
 		PTime newRotateTime(rotateTime.GetSecond(), rotateTime.GetMinute(),
-			rotateTime.GetHour(), 1, 
-			rotateTime.GetMonth() < 12 ? rotateTime.GetMonth() + 1 : 1, 
+			rotateTime.GetHour(), 1,
+			rotateTime.GetMonth() < 12 ? rotateTime.GetMonth() + 1 : 1,
 			rotateTime.GetMonth() < 12 ? rotateTime.GetYear() : rotateTime.GetYear() + 1,
 			rotateTime.GetTimeZone()
 			);
