@@ -2,10 +2,12 @@
 //
 // gkauth.cxx
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2001-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -26,9 +28,9 @@
 #include "config.h"
 
 #if H323_H350
-  extern const char *H350Section;
-  #include <ptclib/pldap.h>
-  #include "h350/h350.h"
+extern const char *H350Section;
+#include <ptclib/pldap.h>
+#include "h350/h350.h"
 #endif
 
 namespace {
@@ -48,7 +50,7 @@ ARQAuthData::ARQAuthData(
 	const callptr& call
 	) : m_rejectReason(-1), m_callDurationLimit(-1), 
 	m_requestingEP(ep),	m_call(call), m_billingMode(-1),
-	m_routeToAlias(NULL), m_proxyMode(CallRec::ProxyDetect),
+	m_proxyMode(CallRec::ProxyDetect),
 	m_clientAuthId(0)
 {
 }
@@ -58,12 +60,10 @@ ARQAuthData::ARQAuthData(
 	) : m_rejectReason(obj.m_rejectReason),
 	m_callDurationLimit(obj.m_callDurationLimit), 
 	m_requestingEP(obj.m_requestingEP), m_call(obj.m_call), 
-	m_billingMode(obj.m_billingMode), m_routeToAlias(NULL),
+	m_billingMode(obj.m_billingMode), m_routeToAlias(obj.m_routeToAlias),
 	m_destinationRoutes(obj.m_destinationRoutes), m_proxyMode(obj.m_proxyMode),
 	m_clientAuthId(0)
 {
-	if (obj.m_routeToAlias)
-		m_routeToAlias = new H225_AliasAddress(*obj.m_routeToAlias);
 }
 
 ARQAuthData& ARQAuthData::operator=(const ARQAuthData& obj)
@@ -76,38 +76,21 @@ ARQAuthData& ARQAuthData::operator=(const ARQAuthData& obj)
 		m_billingMode = obj.m_billingMode;
 		m_proxyMode = obj.m_proxyMode;
 		m_clientAuthId = obj.m_clientAuthId;
-
-		delete m_routeToAlias;
-		m_routeToAlias = NULL;
-		if (obj.m_routeToAlias)
-			m_routeToAlias = new H225_AliasAddress(*obj.m_routeToAlias);
-
+		m_routeToAlias = obj.m_routeToAlias;
 		m_destinationRoutes = obj.m_destinationRoutes;
 	}
 	return *this;
 }
 
-ARQAuthData::~ARQAuthData()
+void ARQAuthData::SetRouteToAlias(const H225_ArrayOf_AliasAddress & alias)
 {
-	delete m_routeToAlias;
-	m_routeToAlias = NULL;
-}
-
-void ARQAuthData::SetRouteToAlias(H225_AliasAddress* alias)
-{
-	delete m_routeToAlias;
 	m_routeToAlias = alias;
 }
 
-void ARQAuthData::SetRouteToAlias(const H225_AliasAddress& alias)
-{
-	SetRouteToAlias(new H225_AliasAddress(alias));
-}
-
 void ARQAuthData::SetRouteToAlias(const PString& alias, int tag)
 {
-	SetRouteToAlias(new H225_AliasAddress);
-	H323SetAliasAddress(alias, *m_routeToAlias, tag);
+	m_routeToAlias.SetSize(1);
+	H323SetAliasAddress(alias, m_routeToAlias[0], tag);
 }
 
 SetupAuthData::SetupAuthData(
@@ -117,7 +100,7 @@ SetupAuthData::SetupAuthData(
 	bool fromRegistered
 	) : m_rejectReason(-1), m_rejectCause(-1), m_callDurationLimit(-1),
 	m_call(call), m_fromRegistered(fromRegistered), 
-	m_routeToAlias(NULL), m_proxyMode(CallRec::ProxyDetect),
+	m_proxyMode(CallRec::ProxyDetect),
 	m_clientAuthId(0)
 {
 }
@@ -127,11 +110,9 @@ SetupAuthData::SetupAuthData(
 	) : m_rejectReason(obj.m_rejectReason), m_rejectCause(obj.m_rejectCause), 
 	m_callDurationLimit(obj.m_callDurationLimit), m_call(obj.m_call), 
 	m_fromRegistered(obj.m_fromRegistered),
-	m_routeToAlias(NULL), m_destinationRoutes(obj.m_destinationRoutes),
+	m_routeToAlias(obj.m_routeToAlias), m_destinationRoutes(obj.m_destinationRoutes),
 	m_proxyMode(obj.m_proxyMode), m_clientAuthId(0)
 {
-	if (obj.m_routeToAlias)
-		m_routeToAlias = new H225_AliasAddress(*obj.m_routeToAlias);
 }
 
 SetupAuthData& SetupAuthData::operator=(const SetupAuthData& obj)
@@ -144,12 +125,7 @@ SetupAuthData& SetupAuthData::operator=(const SetupAuthData& obj)
 		m_fromRegistered = obj.m_fromRegistered;
 		m_proxyMode = obj.m_proxyMode;
 		m_clientAuthId = obj.m_clientAuthId;
-
-		delete m_routeToAlias;
-		m_routeToAlias = NULL;
-		if (obj.m_routeToAlias)
-			m_routeToAlias = new H225_AliasAddress(*obj.m_routeToAlias);
-
+		m_routeToAlias = obj.m_routeToAlias;
 		m_destinationRoutes = obj.m_destinationRoutes;
 	}
 
@@ -158,25 +134,17 @@ SetupAuthData& SetupAuthData::operator=(const SetupAuthData& obj)
 
 SetupAuthData::~SetupAuthData()
 {
-	delete m_routeToAlias;
-	m_routeToAlias = NULL;
 }
 
-void SetupAuthData::SetRouteToAlias(H225_AliasAddress* alias)
+void SetupAuthData::SetRouteToAlias(const H225_ArrayOf_AliasAddress & alias)
 {
-	delete m_routeToAlias;
 	m_routeToAlias = alias;
 }
 
-void SetupAuthData::SetRouteToAlias(const H225_AliasAddress& alias)
-{
-	SetRouteToAlias(new H225_AliasAddress(alias));
-}
-
 void SetupAuthData::SetRouteToAlias(const PString& alias, int tag)
 {
-	SetRouteToAlias(new H225_AliasAddress);
-	H323SetAliasAddress(alias, *m_routeToAlias, tag);
+	m_routeToAlias.SetSize(1);
+	H323SetAliasAddress(alias, m_routeToAlias[0], tag);
 }
 
 
@@ -185,7 +153,7 @@ GkAuthenticator::GkAuthenticator(
 	const char* name, /// a name for the module (to be used in the config file)
 	unsigned supportedRasChecks, /// RAS checks supported by this module
 	unsigned supportedMiscChecks /// non-RAS checks supported by this module
-	) 
+	)
 	: NamedObject(name), m_defaultStatus(e_fail), m_controlFlag(e_Required),
 	m_enabledRasChecks(~0U), m_supportedRasChecks(supportedRasChecks), 
 	m_enabledMiscChecks(~0U), m_supportedMiscChecks(supportedMiscChecks),
@@ -541,7 +509,7 @@ PString GkAuthenticator::GetUsername(
 			);
 		if (!username && callingEP 
 				&& FindAlias(callingEP->GetAliases(), username) == P_MAX_INDEX)
-			username = PString();
+			username = PString::Empty();
 	}
 
 	if (username.IsEmpty() && hasCall) {
@@ -553,7 +521,7 @@ PString GkAuthenticator::GetUsername(
 			);
 		if (!username && callingEP 
 				&& FindAlias(callingEP->GetAliases(), username) == P_MAX_INDEX)
-			username = PString();
+			username = PString::Empty();
 	}
 	
 	if (username.IsEmpty() && callingEP)
@@ -1494,7 +1462,7 @@ bool SimplePasswordAuth::ResolveUserName(
 		)
 {
 	for (PINDEX i = 0; i < cryptotokens.GetSize(); i++) {
-	  // MD5
+		// MD5
 		if (cryptotokens[i].GetTag() == H225_CryptoH323Token::e_cryptoEPPwdHash) {
 			H225_CryptoH323Token_cryptoEPPwdHash& pwdhash = cryptotokens[i];
 			username = AsString(pwdhash.m_alias, false);
@@ -1528,10 +1496,10 @@ bool SimplePasswordAuth::ResolveUserName(
 	for (PINDEX j = 0; j < tokens.GetSize(); j++) {
 		H235_ClearToken& token = tokens[j];
 		// CAT
-			if (token.HasOptionalField(H235_ClearToken::e_generalID)) {
-                username = token.m_generalID;
-				return true;
-			}
+		if (token.HasOptionalField(H235_ClearToken::e_generalID)) {
+			username = token.m_generalID;
+			return true;
+		}
 	}
 
 	return false;
@@ -1540,20 +1508,17 @@ bool SimplePasswordAuth::ResolveUserName(
 #ifdef H323_H350
 
 H350PasswordAuth::H350PasswordAuth(const char* authName)
-: SimplePasswordAuth(authName)
+	: SimplePasswordAuth(authName)
 {
-
 }
 	
 H350PasswordAuth::~H350PasswordAuth()
 {
-
 }
 
-bool H350PasswordAuth::GetPassword(const PString& alias,PString& password)
+bool H350PasswordAuth::GetPassword(const PString & alias, PString & password)
 {
-
-// Search the Directory
+	// search the directory
 	PString search = GkConfig()->GetString(H350Section, "SearchBaseDN", "");
 
 	H225_AliasAddress aliasaddress;
@@ -1563,32 +1528,32 @@ bool H350PasswordAuth::GetPassword(const PString& alias,PString& password)
 
 	H350_Session session;
 	if (!Toolkit::Instance()->CreateH350Session(&session)) {
-	   PTRACE(4,"H350\tH235Auth: Could not connect to Server.");
-	   return false;
+		PTRACE(1, "H350\tH235Auth: Could not connect to server");
+		return false;
 	}
 
 	H350_Session::LDAP_RecordList rec;
-	int count = session.Search(search,filter,rec);
+	int count = session.Search(search, filter, rec);
 	if (count <= 0) {
-	   PTRACE(4,"H350\tH235Auth:No Record Found");
-	   session.Close();
-	   return false;
+		PTRACE(4, "H350\tH235Auth: No record found");
+		session.Close();
+		return false;
 	}
 
-// Locate the record
+	// locate the record
 	for (H350_Session::LDAP_RecordList::const_iterator x = rec.begin(); x != rec.end(); ++x) {			
-       H350_Session::LDAP_Record entry = x->second;
-	   if (session.GetAttribute(entry,"h235IdentityPassword",password)) {
-           PTRACE(2,"H350\tH235Auth:Password Located.");
-	       session.Close();
-		   return true;
-	   }
+		H350_Session::LDAP_Record entry = x->second;
+		if (session.GetAttribute(entry, "h235IdentityPassword", password)) {
+			password = password.Trim();	// server may send newline at end etc.
+			PTRACE(4, "H350\tH235Auth: Password located");
+			session.Close();
+			return true;
+		}
 	}
 
-	PTRACE(4,"H350\tH235Auth: No Password found.");
+	PTRACE(4, "H350\tH235Auth: No password found");
 	session.Close();
 	return false;
-
 }
 
 #endif
@@ -2021,7 +1986,7 @@ PStringArray ARQAuthObj::GetPrefixes() const
 		}
 	}
 	if (array.GetSize() == 0)
-		array.AppendString(PString());
+		array.AppendString(PString::Empty());
 	return array;
 }
 
@@ -2068,7 +2033,7 @@ PIPSocket::Address LRQAuthObj::GetIP() const
 PString LRQAuthObj::GetAliases() const
 {
 	return m_lrq.HasOptionalField(H225_LocationRequest::e_sourceInfo) 
-		? AsString(m_lrq.m_sourceInfo) : PString();
+		? AsString(m_lrq.m_sourceInfo) : PString::Empty();
 }
 
 int AuthRule::Check(
@@ -2234,7 +2199,7 @@ int PrefixAuth::Check(
 		&& arq.HasOptionalField(H225_AdmissionRequest::e_callIdentifier)
 		&& CallTable::Instance()->FindCallRec(arq.m_callIdentifier)) {
 		PTRACE(5, "GKAUTH\t" << GetName() << " ARQ check skipped - call "
-			" already admitted and present in the call table"
+			"already admitted and present in the call table"
 			);
 		return e_ok;
 	}
