@@ -4,70 +4,17 @@
  * Generic interface to access SQL databases
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
+ * Copyright (c) 2006-2010, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
- *
- * $Log: gksql.h,v $
- * Revision 1.18  2009/07/16 12:48:40  willamowius
- * remove unused code
- *
- * Revision 1.17  2009/05/24 20:48:26  willamowius
- * remove hacks for VC6 which isn't supported any more since quite a while
- *
- * Revision 1.16  2009/02/09 13:25:59  willamowius
- * typo in comment
- *
- * Revision 1.15  2008/09/05 13:44:14  zvision
- * GetInfo implemented for SQL acct/auth modules
- *
- * Revision 1.14  2008/09/04 08:19:04  zvision
- * SQL reconnect thread safety fixes
- *
- * Revision 1.13  2008/04/03 10:24:20  willamowius
- * new header config.h for configuration details
- *
- * Revision 1.12  2008/04/02 22:32:22  willamowius
- * auto-reconnect on database errors
- *
- * Revision 1.11  2007/09/10 18:13:48  willamowius
- * clean up sql driver interface and remove unused methods from all drivers
- *
- * Revision 1.10  2006/04/30 09:22:56  willamowius
- * PTimedMutex patch for PWLib >= 1.9.2
- *
- * Revision 1.9  2006/04/14 13:56:19  willamowius
- * call failover code merged
- *
- * Revision 1.1.1.1  2005/11/21 20:19:59  willamowius
- *
- *
- * Revision 1.4  2005/11/15 19:52:56  jan
- * Michal v1 (works, but on in routed, not proxy mode)
- *
- * Revision 1.8  2005/04/24 16:39:44  zvision
- * MSVC6.0 compatibility fixed
- *
- * Revision 1.7  2005/02/11 17:23:05  zvision
- * Write SCCS keyword correctly
- *
- * Revision 1.6  2005/01/16 15:22:35  zvision
- * Database Host parameter accepts only one host now
- *
- * Revision 1.5  2004/12/15 13:41:33  zvision
- * Reconnect to the database, if the initial attempt failed. Thanks to Mikko Oilinki
- *
- * Revision 1.4  2004/08/02 10:52:07  zvision
- * Ability to extract column names from a result set
- *
- * Revision 1.3  2004/07/09 22:11:36  zvision
- * SQLAcct module ported from 2.0 branch
+ * with the OpenH323/H323Plus and OpenSSL library.
  *
  */
+
 #ifndef GKSQL_H
-#define GKSQL_H "@(#) $Id: gksql.h,v 1.18 2009/07/16 12:48:40 willamowius Exp $"
+#define GKSQL_H "@(#) $Id: gksql.h,v 1.20 2010/10/25 15:01:51 willamowius Exp $"
 
 #include <list>
 #include <map>
@@ -415,6 +362,8 @@ protected:
 	PString m_username;
 	/// password associated with the username (if any)
 	PString m_password;
+	/// name of shared library
+	PString m_library;
 
 private:
 	/// iterator typedefs for convenience
