@@ -4,69 +4,12 @@
  * PostgreSQL driver module for GnuGk
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
+ * Copyright (c) 2006-2010, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
- *
- * $Log: gksql_pgsql.cxx,v $
- * Revision 1.19  2009/05/24 20:48:26  willamowius
- * remove hacks for VC6 which isn't supported any more since quite a while
- *
- * Revision 1.18  2009/02/09 13:25:59  willamowius
- * typo in comment
- *
- * Revision 1.17  2008/04/19 07:10:21  shorne
- * added secur32.lib for windows
- *
- * Revision 1.16  2008/04/18 14:37:28  willamowius
- * never include gnugkbuildopts.h directly, always include config.h
- *
- * Revision 1.15  2008/04/18 13:14:11  shorne
- * Fixes for auto-configure on windows
- *
- * Revision 1.14  2008/04/02 22:32:22  willamowius
- * auto-reconnect on database errors
- *
- * Revision 1.13  2007/11/25 14:56:46  willamowius
- * cleanup: comment out unused arguments
- *
- * Revision 1.12  2007/09/10 18:13:48  willamowius
- * clean up sql driver interface and remove unused methods from all drivers
- *
- * Revision 1.11  2006/05/07 12:22:59  willamowius
- * fix comments
- *
- * Revision 1.10  2006/04/14 13:56:19  willamowius
- * call failover code merged
- *
- * Revision 1.1.1.1  2005/11/21 20:19:59  willamowius
- *
- *
- * Revision 1.4  2005/11/15 19:52:56  jan
- * Michal v1 (works, but on in routed, not proxy mode)
- *
- * Revision 1.9  2005/04/24 16:39:44  zvision
- * MSVC6.0 compatibility fixed
- *
- * Revision 1.8  2005/03/15 15:34:03  zvision
- * Compiler warning removed
- *
- * Revision 1.7  2005/01/16 15:22:35  zvision
- * Database Host parameter accepts only one host now
- *
- * Revision 1.6  2004/10/20 09:16:23  zvision
- * VC6 compilation errors fixed
- *
- * Revision 1.5.4.1  2004/10/20 09:10:12  zvision
- * VC6 compilation errors fixed
- *
- * Revision 1.5  2004/08/02 10:52:07  zvision
- * Ability to extract column names from a result set
- *
- * Revision 1.4  2004/07/09 22:11:36  zvision
- * SQLAcct module ported from 2.0 branch
+ * with the OpenH323/H323Plus and OpenSSL library.
  *
  */
 
@@ -78,10 +21,26 @@
 #include <libpq-fe.h>
 #include "gksql.h"
 
-#ifdef _WIN32
-#pragma comment( lib, PGSQL_LIBRARY )
-#pragma comment( lib, "secur32.lib")
-#endif
+static PDynaLink g_sharedLibrary;
+static void (*g_PQclear)(PGresult *res) = NULL;
+static char * (*g_PQcmdTuples)(PGresult *res) = NULL;
+static char * (*g_PQerrorMessage)(const PGconn *conn) = NULL;
+static size_t (*g_PQescapeStringConn)(PGconn *conn, char *to, const char *from, size_t length, int *error) = NULL;
+static PGresult * (*g_PQexec)(PGconn *conn, const char *query) = NULL;
+static void (*g_PQfinish)(PGconn *conn) = NULL;
+static char * (*g_PQfname)(const PGresult *res, int field_num) = NULL;
+static int (*g_PQgetlength)(const PGresult *res, int tup_num, int field_num) = NULL;
+static char * (*g_PQgetvalue)(const PGresult *res, int tup_num, int field_num) = NULL;
+static int (*g_PQnfields)(const PGresult *res) = NULL;
+static int (*g_PQntuples)(const PGresult *res) = NULL;
+static char * (*g_PQresultErrorMessage)(const PGresult *res) = NULL;
+static ExecStatusType (*g_PQresultStatus)(const PGresult *res) = NULL;
+static PGconn * (*g_PQsetdbLogin)(const char *pghost, const char *pgport,
+             const char *pgoptions, const char *pgtty,
+             const char *dbName,
+             const char *login, const char *pwd) = NULL;
+static ConnStatusType (*g_PQstatus)(const PGconn *conn) = NULL;
+
 
 /** Class that encapsulates SQL query result for PostgreSQL backend.
 	It does not provide any multithread safety, so should be accessed
@@ -242,8 +201,8 @@ GkPgSQLResult::GkPgSQLResult(
 	m_errorCode(0)
 {
 	if (m_sqlResult) {
-		m_numRows = PQntuples(m_sqlResult);
-		m_numFields = PQnfields(m_sqlResult);
+		m_numRows = (*g_PQntuples)(m_sqlResult);
+		m_numFields = (*g_PQnfields)(m_sqlResult);
 	} else
 		m_queryError = true;
 }
@@ -272,7 +231,7 @@ GkPgSQLResult::GkPgSQLResult(
 GkPgSQLResult::~GkPgSQLResult()
 {
 	if (m_sqlResult)
-		PQclear(m_sqlResult);
+		(*g_PQclear)(m_sqlResult);
 }
 
 PString GkPgSQLResult::GetErrorMessage()
@@ -303,8 +262,8 @@ bool GkPgSQLResult::FetchRow(
 	
 	for (PINDEX i = 0; i < m_numFields; i++)
 		result[i] = PString(
-			PQgetvalue(m_sqlResult, m_sqlRow, i), 
-			PQgetlength(m_sqlResult, m_sqlRow, i)
+			(*g_PQgetvalue)(m_sqlResult, m_sqlRow, i), 
+			(*g_PQgetlength)(m_sqlResult, m_sqlRow, i)
 			);
 	
 	m_sqlRow++;
@@ -330,10 +289,10 @@ bool GkPgSQLResult::FetchRow(
 	
 	for (PINDEX i = 0; i < m_numFields; i++) {
 		result[i].first = PString(
-			PQgetvalue(m_sqlResult, m_sqlRow, i), 
-			PQgetlength(m_sqlResult, m_sqlRow, i)
+			(*g_PQgetvalue)(m_sqlResult, m_sqlRow, i), 
+			(*g_PQgetlength)(m_sqlResult, m_sqlRow, i)
 			);
-		result[i].second = PQfname(m_sqlResult, i);
+		result[i].second = (*g_PQfname)(m_sqlResult, i);
 	}
 	
 	m_sqlRow++;
@@ -355,7 +314,7 @@ GkPgSQLConnection::~GkPgSQLConnection()
 
 GkPgSQLConnection::PgSQLConnWrapper::~PgSQLConnWrapper()
 {
-	PQfinish(m_conn);
+	(*g_PQfinish)(m_conn);
 }
 
 GkSQLConnection::SQLConnPtr GkPgSQLConnection::CreateNewConnection(
@@ -363,15 +322,51 @@ GkSQLConnection::SQLConnPtr GkPgSQLConnection::CreateNewConnection(
 	int id
 	)
 {
+	if (!g_sharedLibrary.IsLoaded()) {
+		if (m_library.IsEmpty()) {
+			m_library = "libpq" + g_sharedLibrary.GetExtension();
+		}
+
+		if (!g_sharedLibrary.Open(m_library)) {
+			PTRACE (1, GetName() << "\tCan't load library " << m_library);
+			return NULL;
+		}
+
+		if (!g_sharedLibrary.GetFunction("PQclear", (PDynaLink::Function &)g_PQclear)
+			|| !g_sharedLibrary.GetFunction("PQcmdTuples", (PDynaLink::Function &)g_PQcmdTuples)
+			|| !g_sharedLibrary.GetFunction("PQerrorMessage", (PDynaLink::Function &)g_PQerrorMessage)
+			|| !g_sharedLibrary.GetFunction("PQescapeStringConn", (PDynaLink::Function &)g_PQescapeStringConn)
+			|| !g_sharedLibrary.GetFunction("PQexec", (PDynaLink::Function &)g_PQexec)
+			|| !g_sharedLibrary.GetFunction("PQfinish", (PDynaLink::Function &)g_PQfinish)
+			|| !g_sharedLibrary.GetFunction("PQfname", (PDynaLink::Function &)g_PQfname)
+			|| !g_sharedLibrary.GetFunction("PQgetlength", (PDynaLink::Function &)g_PQgetlength)
+			|| !g_sharedLibrary.GetFunction("PQgetvalue", (PDynaLink::Function &)g_PQgetvalue)
+			|| !g_sharedLibrary.GetFunction("PQnfields", (PDynaLink::Function &)g_PQnfields)
+			|| !g_sharedLibrary.GetFunction("PQntuples", (PDynaLink::Function &)g_PQntuples)
+			|| !g_sharedLibrary.GetFunction("PQresultErrorMessage", (PDynaLink::Function &)g_PQresultErrorMessage)
+			|| !g_sharedLibrary.GetFunction("PQresultStatus", (PDynaLink::Function &)g_PQresultStatus)
+			|| !g_sharedLibrary.GetFunction("PQsetdbLogin", (PDynaLink::Function &)g_PQsetdbLogin)
+			|| !g_sharedLibrary.GetFunction("PQstatus", (PDynaLink::Function &)g_PQstatus)
+			) {
+#ifdef hasDynaLinkGetLastError
+			PTRACE (1, GetName() << "\tFailed to load shared database library: " << g_sharedLibrary.GetLastError());
+#else
+			PTRACE (1, GetName() << "\tFailed to load shared database library: unknown error");
+#endif
+			g_sharedLibrary.Close();
+			return NULL;
+		}
+	}
+
 	PGconn* conn;
 	const PString portStr(m_port);
 //	const PString optionsStr("connect_timeout=10000");
-	if ((conn = PQsetdbLogin(m_host, 
+	if ((conn = (*g_PQsetdbLogin)(m_host, 
 			m_port ? (const char*)portStr : (const char*)NULL,
 			NULL /*(const char*)optionsStr*/, NULL,
 			m_database, m_username, 
 			m_password.IsEmpty() ? (const char*)NULL : (const char*)m_password
-			)) && PQstatus(conn) == CONNECTION_OK) {
+			)) && (*g_PQstatus)(conn) == CONNECTION_OK) {
 		PTRACE(5, GetName() << "\tPgSQL connection to " << m_username << '@' << m_host 
 			<< '[' << m_database << "] established successfully"
 			);
@@ -379,10 +374,10 @@ GkSQLConnection::SQLConnPtr GkPgSQLConnection::CreateNewConnection(
 	} else {
 		PTRACE(2, GetName() << "\tPgSQL connection to " << m_username << '@' << m_host 
 			<< '[' << m_database << "] failed (PQsetdbLogin failed): " 
-			<< (conn ? PQerrorMessage(conn) : "")
+			<< (conn ? (*g_PQerrorMessage)(conn) : "")
 			);
 		if (conn)
-			PQfinish(conn);
+			(*g_PQfinish)(conn);
 	}
 	return NULL;
 }
@@ -397,26 +392,26 @@ GkSQLResult* GkPgSQLConnection::ExecuteQuery(
 	)
 {
 	PGconn* pgsqlconn = ((PgSQLConnWrapper*)conn)->m_conn;
-	PGresult* result = PQexec(pgsqlconn, queryStr);
+	PGresult* result = (*g_PQexec)(pgsqlconn, queryStr);
 	if (result == NULL) {
-		GkSQLResult * sqlResult = new GkPgSQLResult(PGRES_FATAL_ERROR, PQerrorMessage(pgsqlconn));
+		GkSQLResult * sqlResult = new GkPgSQLResult(PGRES_FATAL_ERROR, (*g_PQerrorMessage)(pgsqlconn));
 		Disconnect();
 		return sqlResult;
 	}
 		
-	ExecStatusType resultInfo = PQresultStatus(result);
+	ExecStatusType resultInfo = (*g_PQresultStatus)(result);
 	switch (resultInfo)
 	{
 	case PGRES_COMMAND_OK:
 		return new GkPgSQLResult(
-			PQcmdTuples(result) ? atoi(PQcmdTuples(result)) : 0
+			(*g_PQcmdTuples)(result) ? atoi((*g_PQcmdTuples)(result)) : 0
 			);
 		
 	case PGRES_TUPLES_OK:
 		return new GkPgSQLResult(result);
 		
 	default:
-		GkSQLResult * sqlResult = new GkPgSQLResult(resultInfo, PQresultErrorMessage(result));
+		GkSQLResult * sqlResult = new GkPgSQLResult(resultInfo, (*g_PQresultErrorMessage)(result));
 		Disconnect();
 		return sqlResult;
 	}
@@ -424,18 +419,21 @@ GkSQLResult* GkPgSQLConnection::ExecuteQuery(
 
 PString GkPgSQLConnection::EscapeString(
 	/// SQL connection to get escaping parameters from
-	SQLConnPtr /* conn */,
+	SQLConnPtr conn,
 	/// string to be escaped
 	const char* str
 	)
 {
 	PString escapedStr;
-	const unsigned long numChars = str ? strlen(str) : 0;
+	const size_t numChars = str ? strlen(str) : 0;
+	int err = 0;
 	
-	if (numChars)
-		escapedStr.SetSize(
-			PQescapeString(escapedStr.GetPointer(numChars*2+1), str, numChars) + 1
-			);
+	if (numChars) {
+		char * buf = (char *)malloc(numChars * 2 + 1);
+		(*g_PQescapeStringConn) (((PgSQLConnWrapper*)conn)->m_conn, buf, str, numChars, &err);
+		escapedStr = buf;
+		free(buf);
+	}
 	return escapedStr;
 }
 
