@@ -3,32 +3,12 @@
  *
  * SQLite driver module for GnuGk
  *
- * Copyright (c) 2007, Jan Willamowius
+ * Copyright (c) 2007-2010, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
- *
- * $Log: gksql_sqlite.cxx,v $
- * Revision 1.6  2009/05/24 20:48:26  willamowius
- * remove hacks for VC6 which isn't supported any more since quite a while
- *
- * Revision 1.5  2009/02/09 13:25:59  willamowius
- * typo in comment
- *
- * Revision 1.4  2008/09/26 14:55:45  willamowius
- * code cleanup
- *
- * Revision 1.3  2008/04/18 14:37:28  willamowius
- * never include gnugkbuildopts.h directly, always include config.h
- *
- * Revision 1.2  2008/04/18 13:14:11  shorne
- * Fixes for auto-configure on windows
- *
- * Revision 1.1  2008/04/18 06:56:13  willamowius
- * database driver for sqlite
- *
+ * with the OpenH323/H323Plus and OpenSSL library.
  *
  */
 
@@ -40,9 +20,15 @@
 #include <sqlite3.h>
 #include "gksql.h"
 
-#ifdef _WIN32
-#pragma comment( lib, SQLITE_LIBRARY )
-#endif
+static PDynaLink g_sharedLibrary;
+static int (*g_sqlite3_changes)(sqlite3*) = NULL;
+static int (*g_sqlite3_close)(sqlite3 *) = NULL;
+static const char * (*g_sqlite3_errmsg)(sqlite3*) = NULL;
+static int (*g_sqlite3_exec)(sqlite3*, const char *sql, int (*callback)(void*,int,char**,char**), void *, char **errmsg) = NULL;
+static void (*g_sqlite3_free)(void*) = NULL;
+static char * (*g_sqlite3_mprintf)(const char*,...) = NULL;
+static int (*g_sqlite3_open)(const char *filename, sqlite3 **ppDb) = NULL;
+
 
 /** Class that encapsulates SQL query result for SQLite backend.
 	It does not provide any multithread safety, so should be accessed
@@ -299,7 +285,7 @@ GkSQLiteConnection::~GkSQLiteConnection()
 
 GkSQLiteConnection::GkSQLiteConnWrapper::~GkSQLiteConnWrapper()
 {
-	sqlite3_close(m_conn);
+	(*g_sqlite3_close)(m_conn);
 }
 
 GkSQLConnection::SQLConnPtr GkSQLiteConnection::CreateNewConnection(
@@ -307,13 +293,45 @@ GkSQLConnection::SQLConnPtr GkSQLiteConnection::CreateNewConnection(
 	int id
 	)
 {
+	if (!g_sharedLibrary.IsLoaded()) {
+		if (m_library.IsEmpty()) {
+#ifdef _WIN32
+			m_library = "sqlite3" + g_sharedLibrary.GetExtension();
+#else
+			m_library = "libsqlite3" + g_sharedLibrary.GetExtension();
+#endif
+		}
+
+		if (!g_sharedLibrary.Open(m_library)) {
+			PTRACE (1, GetName() << "\tCan't load library " << m_library);
+			return NULL;
+		}
+
+		if (!g_sharedLibrary.GetFunction("sqlite3_changes", (PDynaLink::Function &)g_sqlite3_changes)
+			|| !g_sharedLibrary.GetFunction("sqlite3_close", (PDynaLink::Function &)g_sqlite3_close)
+			|| !g_sharedLibrary.GetFunction("sqlite3_errmsg", (PDynaLink::Function &)g_sqlite3_errmsg)
+			|| !g_sharedLibrary.GetFunction("sqlite3_exec", (PDynaLink::Function &)g_sqlite3_exec)
+			|| !g_sharedLibrary.GetFunction("sqlite3_free", (PDynaLink::Function &)g_sqlite3_free)
+			|| !g_sharedLibrary.GetFunction("sqlite3_mprintf", (PDynaLink::Function &)g_sqlite3_mprintf)
+			|| !g_sharedLibrary.GetFunction("sqlite3_open", (PDynaLink::Function &)g_sqlite3_open)
+			) {
+#ifdef hasDynaLinkGetLastError
+			PTRACE (1, GetName() << "\tFailed to load shared database library: " << g_sharedLibrary.GetLastError());
+#else
+			PTRACE (1, GetName() << "\tFailed to load shared database library: unknown error");
+#endif
+			g_sharedLibrary.Close();
+			return NULL;
+		}
+	}
+
 	sqlite3 *conn;
-	int rc = sqlite3_open(m_database, &conn);
+	int rc = (*g_sqlite3_open)(m_database, &conn);
 	if (rc) {
 		PTRACE(2, GetName() << "\tSQLite connection to " <<  m_database 
-			<< " failed (sqlite3_open failed): " << sqlite3_errmsg(conn)
+			<< " failed (sqlite3_open failed): " << (*g_sqlite3_errmsg)(conn)
 			);
-		sqlite3_close(conn);
+		(*g_sqlite3_close)(conn);
 		return NULL;
 	}	
 	
@@ -346,13 +364,13 @@ GkSQLResult* GkSQLiteConnection::ExecuteQuery(
 
 	vector<GkSQLResult::ResultRow*> * resultRows = new vector<GkSQLResult::ResultRow*>();
 	char *errormsg = NULL;
-	int rc = sqlite3_exec(conn, queryStr, sqlite_callback, resultRows, &errormsg);
+	int rc = (*g_sqlite3_exec)(conn, queryStr, sqlite_callback, resultRows, &errormsg);
 	if (rc != SQLITE_OK) {
 		delete resultRows;
 		Disconnect();
 		return new GkSQLiteResult(rc, errormsg);
 	}
-	return new GkSQLiteResult(sqlite3_changes(conn), resultRows);
+	return new GkSQLiteResult((*g_sqlite3_changes)(conn), resultRows);
 }
 
 PString GkSQLiteConnection::EscapeString(
@@ -362,9 +380,9 @@ PString GkSQLiteConnection::EscapeString(
 	const char* str
 	)
 {
-	char * quoted = sqlite3_mprintf("%q",str);
+	char * quoted = (*g_sqlite3_mprintf)("%q",str);
 	PString result(quoted);
-	sqlite3_free(quoted);
+	(*g_sqlite3_free)(quoted);
 	return result;
 }
 
