@@ -1,11 +1,13 @@
 //////////////////////////////////////////////////////////////////
 //
-// H.323 utility functions that should migrate into the OpenH323 library
+// H.323 utility functions
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -36,19 +38,25 @@ PString AsString(const H225_TransportAddress & ta)
 	return stream;
 }
 
-PString AsDotString(const H225_TransportAddress & ip)
+PString AsDotString(const H225_TransportAddress & ip, bool showPort)
 {
 	return (ip.IsValid() && ip.GetTag() == H225_TransportAddress::e_ipAddress) ?
-		AsString((const H225_TransportAddress_ipAddress &)ip) : PString();
+		AsString((const H225_TransportAddress_ipAddress &)ip, showPort) : PString::Empty();
 }
 
-PString AsString(const H225_TransportAddress_ipAddress & ip)
+PString AsString(const H225_TransportAddress_ipAddress & ip, bool showPort)
 {
-	return PString(PString::Printf, "%d.%d.%d.%d:%u",
-		ip.m_ip[0], ip.m_ip[1], ip.m_ip[2], ip.m_ip[3],
-		ip.m_port.GetValue());
+	if (showPort) {
+		return PString(PString::Printf, "%d.%d.%d.%d:%u",
+			ip.m_ip[0], ip.m_ip[1], ip.m_ip[2], ip.m_ip[3],
+			ip.m_port.GetValue());
+	} else {
+		return PString(PString::Printf, "%d.%d.%d.%d",
+			ip.m_ip[0], ip.m_ip[1], ip.m_ip[2], ip.m_ip[3]);
+	}
 }
 
+
 PString AsString(const H225_EndpointType & terminalType)
 {
 	PString terminalTypeString;
@@ -109,9 +117,9 @@ PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasType)
 
 PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAliasName)
 {
-	PString aliasListString = "";
+	PString aliasListString = PString::Empty();
 
-	for(PINDEX cnt = 0; cnt < terminalAlias.GetSize(); cnt ++ )
+	for(PINDEX cnt = 0; cnt < terminalAlias.GetSize(); cnt++ )
 	{
 
 		aliasListString += AsString(terminalAlias[cnt], includeAliasName);
@@ -134,6 +142,16 @@ PString AsString(const PASN_OctetString & Octets)
 	return result;
 }
 
+PString AsString(const PBYTEArray & array)
+{
+	PString result;
+	for (PINDEX i = 0; i < array.GetSize(); i++) {
+		if (isprint(array[i]))
+			result += array[i];
+	}
+	return result;
+}
+
 PString StripAliasType(const PString & alias)
 {
 	const PINDEX nameIndex = alias.FindLast(':');
@@ -244,13 +262,6 @@ bool IsIPAddress(const PString & addr)
 	return ((addr.FindRegEx(ipPattern) != P_MAX_INDEX) || (addr.FindRegEx(ipAndPortPattern) != P_MAX_INDEX));
 }
 
-bool IsRemoteNATSupport(const PString & addr)
-{
-	PString ipAddr = addr.Trim();
-	PINDEX p = ipAddr.Find('*');
-	return (p != P_MAX_INDEX);
-}
-
 bool IsLoopback(const PIPSocket::Address & addr)
 {
 	return addr.IsLoopback() != 0;
@@ -280,6 +291,18 @@ PString GetGUIDString(
 	return idstr;
 }
 
+/** convert a string into a call-id
+ */
+H225_CallIdentifier StringToCallId(PString CallId)
+{
+	H225_CallIdentifier result;
+	CallId.Replace("-", "", true);
+	CallId.Replace(" ", "", true);
+	OpalGloballyUniqueID tmp_guid(CallId);
+	result.m_guid = tmp_guid;
+	return result;
+}
+
 PINDEX GetBestAliasAddressIndex(
 	const H225_ArrayOf_AliasAddress& aliases, /// aliases to be searched
 	bool exactMatch, /// search only specified tags or find any alias
@@ -316,7 +339,7 @@ PString GetBestAliasAddressString(
 	if (i != P_MAX_INDEX)
 		return AsString(aliases[i], FALSE);
 	else
-		return PString();
+		return PString::Empty();
 }
 
 PINDEX FindAlias(
@@ -372,7 +395,7 @@ PString RewriteString(
 	while (lastDstDot != NULL) {
 		lastSrcDot = strchr(lastSrcDot, '.');
 		if (lastSrcDot == NULL) {
-			PTRACE(0, "GK\tInvalid rewrite rule (dots do not match) - "
+			PTRACE(1, "GK\tInvalid rewrite rule (dots do not match) - "
 				<< prefix << " = " << value
 				);
 			break;
