@@ -1,20 +1,24 @@
 //////////////////////////////////////////////////////////////////
 //
-// H.323 utility functions that should migrate into the OpenH323 library
+// H.323 utility functions
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2000-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 
 #ifndef H323UTIL_H
-#define H323UTIL_H "@(#) $Id: h323util.h,v 1.28 2009/10/13 14:55:21 willamowius Exp $"
+#define H323UTIL_H "@(#) $Id: h323util.h,v 1.33 2011/05/23 15:41:34 willamowius Exp $"
 
+#include <ptlib.h>
 #include <ptlib/sockets.h>
 
+class H225_CallIdentifier;
 class H225_GloballyUniqueID;
 class H225_TransportAddress;
 class H225_TransportAddress_ipAddress;
@@ -31,14 +35,16 @@ PString AsString(const H245_UnicastAddress_iPAddress &);
 
 PString AsString(const H225_TransportAddress & ta);
 
-PString AsDotString(const H225_TransportAddress & ip);
+PString AsDotString(const H225_TransportAddress & ip, bool showPort=true);
 
-PString AsString(const H225_TransportAddress_ipAddress & ip);
+PString AsString(const H225_TransportAddress_ipAddress & ip, bool showPort=true);
 
 PString AsString(const H225_EndpointType & terminalType);
 
 PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasName = TRUE);
 
+PString AsString(const PBYTEArray & array);
+
 PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAliasName = TRUE);
 
 PString AsString(const PASN_OctetString & Octets);
@@ -65,8 +71,6 @@ bool GetIPAndPortFromTransportAddr(const H225_TransportAddress & addr, PIPSocket
 
 bool IsIPAddress(const PString & addr);
 
-bool IsRemoteNATSupport(const PString & addr);
-
 bool IsLoopback(const PIPSocket::Address &);
 
 /** Find an alias which tag is of type specified by #primaryTags#.
@@ -117,6 +121,10 @@ PString GetGUIDString(
 	bool fixedLength = false /// skip leading zeros (false) or not (true)
 	);
 
+/** convert a string into a call-id
+ */
+H225_CallIdentifier StringToCallId(PString CallId);
+
 /** Check if the given #alias# is present on the list of #aliases#.
 
     @return
