@@ -3,15 +3,18 @@
  *
  * IP based authentication modules
  *
- * @(#) $Id: ipauth.cxx,v 1.8 2009/02/09 13:25:59 willamowius Exp $
+ * @(#) $Id: ipauth.cxx,v 1.12 2011/07/20 14:06:46 willamowius Exp $
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
+ * Copyright (c) 2006-2010, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
+ * with the OpenH323/H323Plus and OpenSSL library.
+ *
  */
+
 #include <ptlib.h>
 #include <h225.h>
 
@@ -38,8 +41,8 @@ public:
 	IPAuthPrefix& operator=(const IPAuthPrefix&);
 	IPAuthPrefix& operator=(bool);
 
-        typedef std::vector<std::string>::iterator prefix_iterator;
-        typedef std::vector<std::string>::const_iterator const_prefix_iterator;
+	typedef std::vector<std::string>::iterator prefix_iterator;
+	typedef std::vector<std::string>::const_iterator const_prefix_iterator;
 
 	bool auth;
 
@@ -52,7 +55,6 @@ class FileIPAuth : public IPAuthBase {
 public:
 	typedef std::pair<NetworkAddress, IPAuthPrefix> IPAuthEntry;
 
-	
 	/// Create text file based authenticator
 	FileIPAuth( 
 		/// authenticator name from Gatekeeper::Auth section
@@ -106,7 +108,7 @@ int IPAuthBase::Check(
 	unsigned & /*rejectReason*/
 	)
 {
-	return CheckAddress(grqPdu->m_peerAddr, grqPdu->m_peerPort, PString());
+	return CheckAddress(grqPdu->m_peerAddr, grqPdu->m_peerPort, PString::Empty());
 }
 
 int IPAuthBase::Check(
@@ -116,7 +118,7 @@ int IPAuthBase::Check(
 	RRQAuthData & /*authData*/
 	)
 {
-	return CheckAddress(rrqPdu->m_peerAddr, rrqPdu->m_peerPort, PString());
+	return CheckAddress(rrqPdu->m_peerAddr, rrqPdu->m_peerPort, PString::Empty());
 }
 
 int IPAuthBase::Check(
@@ -126,7 +128,7 @@ int IPAuthBase::Check(
 	unsigned & /*rejectReason*/
 	)
 {
-	return CheckAddress(lrqPdu->m_peerAddr, lrqPdu->m_peerPort, PString());
+	return CheckAddress(lrqPdu->m_peerAddr, lrqPdu->m_peerPort, PString::Empty());
 }
 
 int IPAuthBase::Check(
@@ -180,7 +182,7 @@ FileIPAuth::FileIPAuth(
 	if (cfg->HasKey(FileIPAuthSecName, "include")) {
 		const PFilePath fp(cfg->GetString(FileIPAuthSecName, "include", ""));
 		if (!PFile::Exists(fp)) {
-			PTRACE(0, GetName() << "\tCould not read the include file '"
+			PTRACE(1, GetName() << "\tCould not read the include file '"
 				<< fp << "': the file does not exist"
 				);
 			return;
@@ -206,10 +208,10 @@ FileIPAuth::FileIPAuth(
 		PString auth(kv.GetDataAt(i));
 		PString prefix(".");
 
-                if ((position = auth.Find(';', position)) != P_MAX_INDEX) {
-            		position++;
-                        prefix = auth(position, auth.GetLength());
-                        position--;
+		if ((position = auth.Find(';', position)) != P_MAX_INDEX) {
+			position++;
+			prefix = auth(position, auth.GetLength());
+			position--;
 			auth.Delete(position, auth.GetLength() - position);
 		}
 
