@@ -2,19 +2,22 @@
 //
 // pwlib_compat.h PWLib compatibility header
 //
-// This work is published under the GNU Public License (GPL)
+// Copyright (c) 2006-2010, Jan Willamowius
+//
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323/H323Plus library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
 
 #ifndef PWLIB_COMPAT_H
-#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.37 2010/01/19 12:00:21 willamowius Exp $"
+#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.46 2011/07/20 11:26:53 willamowius Exp $"
 
 #include "ptbuildopts.h"
 #include "openh323buildopts.h"
+#include "gnugkbuildopts.h"
 
 // use at least PWLib Pandora
 #if PWLIB_MAJOR == 1
@@ -25,7 +28,7 @@
 
 // check for PConfig support
 #ifndef P_CONFIG_FILE
-	#error "Make sure PWLib has config file support enabled, avoid --disable-configfile or --enable-openh323 or --enable-minsize etc."
+	#error "Make sure PTLib has config file support enabled, avoid --disable-configfile or --enable-openh323 or --enable-minsize etc."
 #endif
 
 // define PTimedMutex for PWLib < 1.9.2
@@ -87,7 +90,7 @@
 	#endif
 #endif
 
-// for Ptlib v2.x
+// for PTlib v2.x
 #ifdef PTLIB_MAJOR 
 	#define hasRDS 1 
 	#define hasSETENUMSERVERS 1    
@@ -99,6 +102,15 @@
 	#if PTLIB_MINOR >= 2
 		#define hasPConfigArray 1
 	#endif
+	// availability of GetLastError in PDynaLink
+	#if PTLIB_MINOR >= 8
+		#define hasDynaLinkGetLastError 1
+	#endif
+	#if PTLIB_MINOR >= 11
+		#define hasLDAPStartTLS 1
+        #define hasNewSTUN      1
+        #define hasPTRACE2      1
+	#endif
 	// bug with no trailing NULL bytes in BMP strings, fixed in PTLib 2.7.1
 	#if ((PTLIB_MINOR == 2) || (PTLIB_MINOR == 4 && PTLIB_BUILD <= 5) || (PTLIB_MINOR == 5 && PTLIB_BUILD <= 2) || (PTLIB_MINOR == 6 && PTLIB_BUILD <= 4))
 		#ifdef _WIN32
@@ -111,14 +123,21 @@
 
 #if !defined(PWLIB_MAJOR) && !defined(PTLIB_MAJOR)
 	#if _WIN32
-		#pragma message ("warning: Can't detect PWLib/PTLib version")
+		#pragma message ("warning: Can't detect PTLib version")
 	#else
-		#warning "Can't detect PWLib/PTLib version"
+		#warning "Can't detect PTLib version"
 	 #endif
 	// be on the safe side and risk a small memleak instead of a crash
 	#define hasDeletingSetStream 1
 #endif
 
+// store version numver it PT macros for dispaly later on
+#if !defined(PTLIB_MAJOR)
+	#define PTLIB_MAJOR	PWLIB_MAJOR
+	#define PTLIB_MINOR	PWLIB_MINOR
+	#define PTLIB_BUILD	PWLIB_BUILD
+#endif
+
 ///////////////////////////////////////////////
 
 // OpenH323/H323Plus version matching
@@ -138,6 +157,19 @@
 	#define HAS_H460P	1  // Presence
 #endif
 
+#if OPENH323_MAJOR == 1
+	#if OPENH323_MINOR == 22
+		#if OPENH323_BUILD >= 1
+		#define HAS_ROUTECALLTOMC 1	// H323Plus endpoints supports RouteCallToMC() - starts with 1.22.1
+		#endif
+	#endif
+#endif
+#if OPENH323_MAJOR == 1
+	#if OPENH323_MINOR > 22
+		#define HAS_ROUTECALLTOMC 1	// H323Plus endpoints supports RouteCallToMC() - starts inside 1.22.1
+	#endif
+#endif
+
 #if OPENH323_MAJOR == 1
 	#if OPENH323_MINOR >= 19
 		#if OPENH323_MINOR == 19
@@ -156,4 +188,10 @@
 	#endif	
 #endif
 
+#if defined(hasPTRACE2)
+   #define PTRACEX(level, args)  PTRACE2(level,NULL,args)
+#else
+   #define PTRACEX(level, args)  PTRACE(level,args)   
+#endif
+
 #endif
