@@ -4,147 +4,12 @@
  * RADIUS protocol accounting logger module for GNU Gatekeeper. 
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
+ * Copyright (c) 2005-2011, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
- *
- * $Log: radacct.cxx,v $
- * Revision 1.37  2009/11/15 15:45:52  willamowius
- * code cleanup
- *
- * Revision 1.36  2009/10/16 11:41:47  willamowius
- * set Radius VSA attribute h323_rtp_proxy in stop event
- *
- * Revision 1.35  2009/09/27 13:30:09  shorne
- * Fix for previous commit
- *
- * Revision 1.34  2009/09/26 02:26:55  shorne
- * Fix compile warning in Windows
- *
- * Revision 1.33  2009/09/11 20:21:51  willamowius
- * new Radius attributes on stop: h323pddtime, h323ringtime, h323routeattempts thanks Yuriy Georgiewskiy
- *
- * Revision 1.32  2009/09/09 07:33:50  willamowius
- * whitespace change
- *
- * Revision 1.31  2009/06/20 10:02:24  willamowius
- * send RTCP statistics to Radius server (thanks to Georgiewskiy Yuriy)
- *
- * Revision 1.30  2009/05/24 22:38:51  willamowius
- * everything that holds a time should be a time_t, not a long
- *
- * Revision 1.29  2009/05/24 20:48:26  willamowius
- * remove hacks for VC6 which isn't supported any more since quite a while
- *
- * Revision 1.28  2009/02/09 13:25:59  willamowius
- * typo in comment
- *
- * Revision 1.27  2008/12/03 18:52:49  willamowius
- * new RadAcct attribute: RewriteE164 (thanks Tusar)
- *
- * Revision 1.26  2008/07/10 08:13:21  willamowius
- * avoid gcc 4.3.x warnings
- *
- * Revision 1.25  2008/05/18 20:40:56  willamowius
- * put codec in RadAcct stop event (Thanks Tusar)
- *
- * Revision 1.24  2008/04/18 14:37:28  willamowius
- * never include gnugkbuildopts.h directly, always include config.h
- *
- * Revision 1.23  2008/04/18 13:14:11  shorne
- * Fixes for auto-configure on windows
- *
- * Revision 1.22  2008/04/03 12:03:35  willamowius
- * set radius release-source attribute in stop accounting packet, like %r in sqlacct
- *
- * Revision 1.21  2007/09/11 01:02:29  willamowius
- * clean up includes
- *
- * Revision 1.20  2006/12/06 16:34:04  zvision
- * Handle RADIUS Class attribute correctly
- *
- * Revision 1.19  2006/07/06 15:25:13  willamowius
- * set all deleted pointers to NULL (most probably more than needed)
- *
- * Revision 1.18  2006/04/14 13:56:19  willamowius
- * call failover code merged
- *
- * Revision 1.1.1.1  2005/11/21 20:19:58  willamowius
- *
- *
- * Revision 1.4  2005/11/15 19:52:56  jan
- * Michal v1 (works, but on in routed, not proxy mode)
- *
- * Revision 1.17  2005/04/24 16:39:44  zvision
- * MSVC6.0 compatibility fixed
- *
- * Revision 1.16  2005/01/05 15:42:41  willamowius
- * new accounting event 'connect', parameter substitution unified in parent class
- *
- * Revision 1.15  2005/01/04 16:47:12  willamowius
- * space in trace msg
- *
- * Revision 1.14  2004/11/15 23:57:42  zvision
- * Ability to choose between the original and the rewritten dialed number
- *
- * Revision 1.13  2004/11/10 18:30:41  zvision
- * Ability to customize timestamp strings
- *
- * Revision 1.12  2004/07/26 12:19:41  zvision
- * New faster Radius implementation, thanks to Pavel Pavlov for ideas!
- *
- * Revision 1.11.2.2  2004/07/07 23:11:07  zvision
- * Faster and more elegant handling of Cisco VSA
- *
- * Revision 1.11.2.1  2004/07/07 20:50:14  zvision
- * New, faster, Radius client implementation. Thanks to Pavel Pavlov for ideas!
- *
- * Revision 1.11  2004/06/25 13:33:18  zvision
- * Better Username, Calling-Station-Id and Called-Station-Id handling.
- * New SetupUnreg option in Gatekeeper::Auth section.
- *
- * Revision 1.10  2004/06/17 10:47:13  zvision
- * New h323-ivr-out=h323-call-id accounting attribute
- *
- * Revision 1.9  2004/04/17 11:43:43  zvision
- * Auth/acct API changes.
- * Header file usage more consistent.
- *
- * Revision 1.8  2004/03/17 00:00:38  zvision
- * Conditional compilation to allow to control RADIUS on Windows just by setting HA_RADIUS macro
- *
- * Revision 1.7  2003/10/31 00:01:24  zvision
- * Improved accounting modules stacking control, optimized radacct/radauth a bit
- *
- * Revision 1.6  2003/10/15 10:16:57  zvision
- * Fixed VC6 compiler warnings. Thanks to Hu Yuxin.
- *
- * Revision 1.5  2003/10/08 12:40:48  zvision
- * Realtime accounting updates added
- *
- * Revision 1.4  2003/09/17 19:23:01  zvision
- * Removed unnecessary setup-time double check.
- * Added h323-connect-time to AcctUpdate packets.
- *
- * Revision 1.3  2003/09/14 21:10:34  zvision
- * Changes due to accounting API redesign.
- *
- * Revision 1.2  2003/09/12 16:31:16  zvision
- * Accounting initially added to the 2.2 branch
- *
- * Revision 1.1.2.5  2003/08/21 15:28:58  zvision
- * Fixed double h323-setup-time sent in Acct-Stop
- *
- * Revision 1.1.2.4  2003/08/17 20:05:39  zvision
- * Added h323-setup-time attribute to Acct-Start packets (Cisco compatibility).
- *
- * Revision 1.1.2.3  2003/07/31 22:58:48  zvision
- * Added Framed-IP-Address attribute and improved h323-disconnect-cause handling
- *
- * Revision 1.1.2.2  2003/07/03 15:30:39  zvision
- * Added cvs Log keyword
+ * with the OpenH323/H323Plus and OpenSSL library.
  *
  */
 
@@ -194,21 +59,13 @@ RadAcct::RadAcct(
 			m_nasIpAddress = interfaces.front();
 		else
 			PTRACE(1, "RADACCT\t" << GetName() << " cannot determine "
-				" NAS IP address"
-				);
+				" NAS IP address");
 	}
 
-	m_appendCiscoAttributes = Toolkit::AsBool(cfg->GetString(
-		cfgSec, "AppendCiscoAttributes", "1"
-		));
+	m_appendCiscoAttributes = Toolkit::AsBool(cfg->GetString(cfgSec, "AppendCiscoAttributes", "1"));
 	m_fixedUsername = cfg->GetString(cfgSec, "FixedUsername", "");
-
 	m_timestampFormat = cfg->GetString(cfgSec, "TimestampFormat", "");
-
-	m_useDialedNumber = Toolkit::AsBool(cfg->GetString(
-		cfgSec, "UseDialedNumber", "0"
-		));
-	
+	m_useDialedNumber = Toolkit::AsBool(cfg->GetString(cfgSec, "UseDialedNumber", "0"));
 	m_attrNasIdentifier = RadiusAttr(RadiusAttr::NasIdentifier, m_nasIdentifier);
 	m_attrH323GwId = RadiusAttr(RadiusAttr::CiscoVSA_h323_gw_id, false, m_nasIdentifier);
 }
@@ -273,12 +130,10 @@ GkAcctLogger::Status RadAcct::Log(
 		const PString username = GetUsername(call);
 		if (username.IsEmpty() && m_fixedUsername.IsEmpty())
 			PTRACE(3, "RADACCT\t" << GetName() << " could not determine User-Name"
-				<< " for the call no. " << call->GetCallNumber()
-				);
+				<< " for the call no. " << call->GetCallNumber());
 		else
 			pdu->AppendAttr(RadiusAttr::UserName, 
-				m_fixedUsername.IsEmpty() ? username : m_fixedUsername
-				);
+				m_fixedUsername.IsEmpty() ? username : m_fixedUsername);
 		
 		if (callerIP.IsValid())
 			pdu->AppendAttr(RadiusAttr::FramedIpAddress, callerIP);
@@ -291,16 +146,14 @@ GkAcctLogger::Status RadAcct::Log(
 			pdu->AppendAttr(RadiusAttr::CallingStationId, stationId);
 		else
 			PTRACE(3, "RADACCT\t" << GetName() << " could not determine"
-				<< " Calling-Station-Id for the call " << call->GetCallNumber()
-				);
+				<< " Calling-Station-Id for the call " << call->GetCallNumber());
 
 		stationId = m_useDialedNumber ? GetDialedNumber(call) : GetCalledStationId(call);
 		if (!stationId)
 			pdu->AppendAttr(RadiusAttr::CalledStationId, stationId);
 		else
 			PTRACE(3, "RADACCT\t" << GetName() << " could not determine"
-				<< " Called-Station-Id for the call no. " << call->GetCallNumber()
-				);
+				<< " Called-Station-Id for the call no. " << call->GetCallNumber());
 		
 		if (m_appendCiscoAttributes) {
 			pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_h323_conf_id,
@@ -311,28 +164,25 @@ GkAcctLogger::Status RadAcct::Log(
 			pdu->AppendAttr(m_attrH323CallOrigin);
 			pdu->AppendAttr(m_attrH323CallType);
 
-			Toolkit* const toolkit = Toolkit::Instance();
+			Toolkit * const toolkit = Toolkit::Instance();
 				
 			time_t tm = call->GetSetupTime();
 			if (tm != 0)
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_h323_setup_time,
-					toolkit->AsString(PTime(tm), m_timestampFormat)
-					);
+					toolkit->AsString(PTime(tm), m_timestampFormat));
 			
 			if (evt & (AcctStop | AcctUpdate)) {
 				tm = call->GetConnectTime();
 				if (tm != 0)
 					pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_h323_connect_time,
-						toolkit->AsString(PTime(tm), m_timestampFormat)
-						);
+						toolkit->AsString(PTime(tm), m_timestampFormat));
 			}
 			
 			if (evt & AcctStop) {
 				tm = call->GetDisconnectTime();
 				if (tm != 0)
 					pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_h323_disconnect_time,
-						toolkit->AsString(PTime(tm), m_timestampFormat)
-						);
+						toolkit->AsString(PTime(tm), m_timestampFormat));
 				
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_h323_disconnect_cause,
 					PString(PString::Unsigned, (long)(call->GetDisconnectCause()), 16)
@@ -343,78 +193,65 @@ GkAcctLogger::Status RadAcct::Log(
 
 				// Post Dial Delay Time
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-					PString("h323pddtime=")+PString(PString::Unsigned,(long)call->GetPostDialDelay()),true
-				);
+					PString("h323pddtime=")+PString(PString::Unsigned,(long)call->GetPostDialDelay()),true);
 
 				// Ring Time
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-					PString("h323ringtime=")+PString(PString::Unsigned,(long)call->GetRingTime()),true
-				);
+					PString("h323ringtime=")+PString(PString::Unsigned,(long)call->GetRingTime()),true);
 
 				// Number of Route Attempts
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-					PString("h323routeattempts=")+PString(PString::Unsigned,call->GetNoCallAttempts()),true
-				);
+					PString("h323routeattempts=")+PString(PString::Unsigned,call->GetNoCallAttempts()),
+					true);
 				
 				// Proxy Mode
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
 					PString("h323_rtp_proxy=")+PString(PString::Unsigned,
-					((call->GetProxyMode() == CallRec::ProxyEnabled) ? 1 : 0)),true
-				);
+					((call->GetProxyMode() == CallRec::ProxyEnabled) ? 1 : 0)), true);
 
-				//RTCP SOURCE REPORT
+				// RTCP SOURCE REPORT
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
 				    PString("RTP_source_IP=")+call->GetSRC_media_IP(),
-    				    true
-				);
+					true);
 				
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
 				    PString("RTP_destination_IP=")+call->GetDST_media_IP(),
-    				    true
-				);
+					true);
 				
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-				    PString("RTCP_source_packet_count=")+PString(PString::Unsigned,call->GetRTCP_SRC_packet_count()),
-    				    true
-				);
+				    PString("RTCP_source_packet_count=")+PString(PString::Unsigned, call->GetRTCP_SRC_packet_count()),
+					true);
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-				    PString("RTCP_source_packet_lost=")+PString(PString::Unsigned,call->GetRTCP_SRC_packet_lost()),
-				    true
-				);
+				    PString("RTCP_source_packet_lost=")+PString(PString::Unsigned, call->GetRTCP_SRC_packet_lost()),
+				    true);
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-				    PString("RTCP_source_jitter=")+PString(PString::Unsigned,call->GetRTCP_SRC_jitter_min())+PString("|")+PString(PString::Unsigned,call->GetRTCP_SRC_jitter_avg())+PString("|")+PString(PString::Unsigned,call->GetRTCP_SRC_jitter_max()),
-				    true
-				);
+				    PString("RTCP_source_jitter=")+PString(PString::Unsigned, call->GetRTCP_SRC_jitter_min())+PString("|")+PString(PString::Unsigned,call->GetRTCP_SRC_jitter_avg())+PString("|")+PString(PString::Unsigned, call->GetRTCP_SRC_jitter_max()),
+				    true);
 				
 				PINDEX i_sdes = 0;
 				PStringList sdes = call->GetRTCP_SRC_sdes();
 				while (i_sdes < sdes.GetSize()) {
 				    pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-					PString("RTCP_source_sdes_")+sdes[i_sdes],
-					true
-				    );
+						PString("RTCP_source_sdes_")+sdes[i_sdes],
+						true);
 				    i_sdes ++;
 				}
-				//RTCP DESTINATION REPORT
+				// RTCP DESTINATION REPORT
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-				    PString("RTCP_destination_packet_count=")+PString(PString::Unsigned,call->GetRTCP_DST_packet_count()),
-				    true
-				);
+				    PString("RTCP_destination_packet_count=")+PString(PString::Unsigned, call->GetRTCP_DST_packet_count()),
+				    true);
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-				    PString("RTCP_destination_packet_lost=")+PString(PString::Unsigned,call->GetRTCP_DST_packet_lost()),
-				    true
-				);
+				    PString("RTCP_destination_packet_lost=")+PString(PString::Unsigned, call->GetRTCP_DST_packet_lost()),
+				    true);
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-				    PString("RTCP_destination_jitter=")+PString(PString::Unsigned,call->GetRTCP_DST_jitter_min())+PString("|")+PString(PString::Unsigned,call->GetRTCP_DST_jitter_avg())+PString("|")+PString(PString::Unsigned,call->GetRTCP_DST_jitter_max()),
-				    true
-				);
+				    PString("RTCP_destination_jitter=")+PString(PString::Unsigned,call->GetRTCP_DST_jitter_min())+PString("|")+PString(PString::Unsigned, call->GetRTCP_DST_jitter_avg())+PString("|")+PString(PString::Unsigned, call->GetRTCP_DST_jitter_max()),
+				    true);
 				i_sdes = 0;
 				sdes = call->GetRTCP_DST_sdes();
 				while (i_sdes < sdes.GetSize()) {
 				    pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-					PString("RTCP_destination_sdes_")+sdes[i_sdes],
-					true
-				    );
+						PString("RTCP_destination_sdes_")+sdes[i_sdes],
+						true);
 				    i_sdes ++;
 				}
 			}					
@@ -422,14 +259,11 @@ GkAcctLogger::Status RadAcct::Log(
 			WORD port;
 			if (call->GetDestSignalAddr(addr, port))
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_h323_remote_address,
-					addr.AsString()
-					);
+					addr.AsString());
 
 			pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
-				PString("h323-ivr-out=h323-call-id:") 
-					+ GetGUIDString(call->GetCallIdentifier().m_guid),
-				true
-				);
+				PString("h323-ivr-out=h323-call-id:") + GetGUIDString(call->GetCallIdentifier().m_guid),
+				true);
 			
 		}
 	
@@ -437,7 +271,7 @@ GkAcctLogger::Status RadAcct::Log(
 	}
 		
 	// send request and wait for response
-	RadiusPDU* response = NULL;
+	RadiusPDU * response = NULL;
 	bool result = OnSendPDU(*pdu, evt, call);
 	
 	// accounting updates must be fast, so we are just sending
@@ -465,8 +299,7 @@ GkAcctLogger::Status RadAcct::Log(
 		else
 			PTRACE(4, "RADACCT\t" << GetName() << " - received response is not "
 				" an AccountingResponse, event " << evt << ", call no. "
-				<< (call ? call->GetCallNumber() : 0)
-				);
+				<< (call ? call->GetCallNumber() : 0));
 		delete response;
 	}
 	return result ? Ok : Fail;
