@@ -4,200 +4,12 @@
  * RADIUS protocol classes.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
+ * Copyright (c) 2003-2010, Jan Willamowius
  *
- * This work is published under the GNU Public License (GPL)
+ * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
  * We also explicitly grant the right to link this code
- * with the OpenH323 library.
- *
- * $Log: radproto.cxx,v $
- * Revision 1.48  2010/04/30 11:45:57  willamowius
- * fix VS2010 warnings
- *
- * Revision 1.47  2009/11/15 15:45:52  willamowius
- * code cleanup
- *
- * Revision 1.46  2009/09/17 15:43:21  willamowius
- * code cleanup
- *
- * Revision 1.45  2009/05/28 10:08:16  willamowius
- * use the more portable ios::fmtflags instead of _Ios_Fmtflags
- *
- * Revision 1.44  2009/05/24 20:48:26  willamowius
- * remove hacks for VC6 which isn't supported any more since quite a while
- *
- * Revision 1.43  2009/02/09 13:25:59  willamowius
- * typo in comment
- *
- * Revision 1.42  2008/12/03 18:52:49  willamowius
- * new RadAcct attribute: RewriteE164 (thanks Tusar)
- *
- * Revision 1.41  2008/07/10 08:18:33  willamowius
- * avoid gcc 4.3.x warnings
- *
- * Revision 1.40  2008/05/18 20:40:56  willamowius
- * put codec in RadAcct stop event (Thanks Tusar)
- *
- * Revision 1.39  2008/04/18 14:37:28  willamowius
- * never include gnugkbuildopts.h directly, always include config.h
- *
- * Revision 1.38  2008/04/18 13:14:11  shorne
- * Fixes for auto-configure on windows
- *
- * Revision 1.37  2008/04/03 12:03:35  willamowius
- * set radius release-source attribute in stop accounting packet, like %r in sqlacct
- *
- * Revision 1.36  2008/03/18 15:51:13  zvision
- * Delete socket when id cannot be generated
- *
- * Revision 1.35  2008/01/09 00:44:07  willamowius
- * remove unused code
- *
- * Revision 1.34  2007/11/25 23:23:40  willamowius
- * cleanup: rename variables that shadow others with the same name
- *
- * Revision 1.33  2007/11/19 20:15:34  willamowius
- * change BOOL to bool where it doesn't affect OpenH323 method signatures
- *
- * Revision 1.32  2007/09/13 11:16:12  willamowius
- * use unique variable names
- *
- * Revision 1.31  2007/09/11 01:02:29  willamowius
- * clean up includes
- *
- * Revision 1.30  2006/07/06 15:25:13  willamowius
- * set all deleted pointers to NULL (most probably more than needed)
- *
- * Revision 1.29  2006/04/14 13:56:19  willamowius
- * call failover code merged
- *
- * Revision 1.2  2005/12/16 14:16:32  zvision
- * Removed incorrect C/C++ language constructs
- *
- * Revision 1.1.1.1  2005/11/21 20:19:57  willamowius
- *
- *
- * Revision 1.4  2005/11/15 19:52:56  jan
- * Michal v1 (works, but on in routed, not proxy mode)
- *
- * Revision 1.27  2005/05/18 20:56:01  zvision
- * Better checking and reporting of network errors
- *
- * Revision 1.26  2005/05/18 14:23:33  zvision
- * Better error reporting on socket allocation
- *
- * Revision 1.25  2005/04/24 16:39:44  zvision
- * MSVC6.0 compatibility fixed
- *
- * Revision 1.24  2005/04/19 07:48:44  zvision
- * Previous fix could cause infinite loops, thanks to kubuqi cn
- *
- * Revision 1.23  2005/04/18 11:24:51  zvision
- * Use list instead of vector in GetSocket to prevent from using invalidated
- * iterators. Thanks to kubuqi cn
- *
- * Revision 1.22  2005/03/15 15:24:28  zvision
- * Removed compiler warning
- *
- * Revision 1.21  2005/01/28 11:19:42  zvision
- * All passwords in the config can be stored in an encrypted form
- *
- * Revision 1.20  2004/12/16 17:11:26  zvision
- * FindVsaAttr ignored vendor identifier during attribute match
- *
- * Revision 1.19  2004/08/09 23:32:11  zvision
- * VC6 compilation errors fixed
- *
- * Revision 1.18  2004/08/09 10:08:28  zvision
- * Fixed missing curly brackets, thanks to Thomas!
- *
- * Revision 1.17  2004/07/26 12:19:42  zvision
- * New faster Radius implementation, thanks to Pavel Pavlov for ideas!
- *
- * Revision 1.16.2.3  2004/07/12 22:27:32  zvision
- * Ability to set a shared secret for each RADIUS server separatelly.
- * More RADIUS code optimizations.
- *
- * Revision 1.16.2.2  2004/07/07 23:11:07  zvision
- * Faster and more elegant handling of Cisco VSA
- *
- * Revision 1.16.2.1  2004/07/07 20:50:14  zvision
- * New, faster, Radius client implementation. Thanks to Pavel Pavlov for ideas!
- *
- * Revision 1.16  2004/06/25 13:33:19  zvision
- * Better Username, Calling-Station-Id and Called-Station-Id handling.
- * New SetupUnreg option in Gatekeeper::Auth section.
- *
- * Revision 1.15  2004/04/21 14:15:26  zvision
- * Default ports are now set to fixes values (1812,1813), because of problems
- * with getservbyname and multiple threads on some systems
- *
- * Revision 1.14  2004/04/20 15:50:07  zvision
- * Descendancy check removed to enable compilation with RTTI enabled PWLib
- *
- * Revision 1.13  2004/04/17 11:43:43  zvision
- * Auth/acct API changes.
- * Header file usage more consistent.
- *
- * Revision 1.12  2004/03/17 00:00:38  zvision
- * Conditional compilation to allow to control RADIUS on Windows just by setting HA_RADIUS macro
- *
- * Revision 1.11  2004/03/11 13:42:48  zvision
- * 64-bit fixes from Klaus Kaempf
- *
- * Revision 1.10  2003/11/11 11:14:21  zvision
- * Fixed invalid signed/unsigned integer conversions for radius attributes.
- * Optimized radius attributes handling.
- *
- * Revision 1.9  2003/10/31 00:02:27  zvision
- * A better tracing/error reporting
- *
- * Revision 1.8  2003/10/21 10:47:16  zvision
- * Fixed minor compilation warnings about precision loss
- *
- * Revision 1.7  2003/10/08 12:40:48  zvision
- * Realtime accounting updates added
- *
- * Revision 1.6  2003/09/28 15:47:23  zvision
- * Better RADIUS client socket handling
- *
- * Revision 1.5  2003/09/24 00:22:03  zvision
- * Removed time_t RadAttr constructors
- *
- * Revision 1.4  2003/09/12 16:31:16  zvision
- * Accounting initially added to the 2.2 branch
- *
- * Revision 1.3  2003/08/20 14:46:19  zvision
- * Avoid PString reference copying. Small code improvements.
- *
- * Revision 1.2  2003/08/19 10:44:18  zvision
- * Initially added to 2.2 branch
- *
- * Revision 1.1.2.8  2003/07/20 23:18:51  zvision
- * Fixed trace output.
- *
- * Revision 1.1.2.7  2003/07/03 15:33:35  zvision
- * Fixed big-endian issues. Fixed small timeout bugs.
- * Fixed strange bug with segfaults on some machines.
- *
- * Revision 1.1.2.6  2003/06/11 12:16:01  zvision
- * Memory usage optimizations.
- * Fixed problems with PSocket::Net2Host call crashing on Solaris.
- *
- * Revision 1.1.2.5  2003/06/05 10:02:20  zvision
- * Bugfixes and small code cleanup.
- *
- * Revision 1.1.2.4  2003/05/26 20:31:09  zvision
- * Better local interface handling in RadiusClient constructor
- *
- * Revision 1.1.2.3  2003/05/13 17:44:40  zvision
- * Added attribute searching functions
- *
- * Revision 1.1.2.2  2003/04/24 11:26:41  willamowius
- * compile fixes for VC 6
- *
- * Revision 1.1.2.1  2003/04/23 20:14:16  zvision
- * Initial revision
+ * with the OpenH323/H323Plus and OpenSSL library.
  *
  */
 
@@ -211,6 +23,11 @@
 #include "Toolkit.h"
 #include "radproto.h"
 
+// ignore overflow warnings
+#if (!_WIN32) && (GCC_VERSION >= 40400)
+#pragma GCC diagnostic ignored "-Wstrict-overflow"
+#endif
+
 namespace {
 #if PTRACING
 /// Human-readable attribute names
@@ -734,14 +551,14 @@ bool RadiusAttr::GetVsaValue(PBYTEArray& buffer, PINDEX offset) const
 PString RadiusAttr::AsString() const
 {
 	if (!IsValid())
-		return PString();
+		return PString::Empty();
 
 	const PINDEX len = m_length;
 	const PINDEX headerLen = (m_type == VendorSpecific) 
 			? VsaFixedHeaderLength : FixedHeaderLength;
 
 	if (len <= headerLen)
-		return PString();
+		return PString::Empty();
 	else
 		return PString((const char*)(m_data + headerLen), len - headerLen);
 }
@@ -772,12 +589,12 @@ PIPSocket::Address RadiusAttr::AsAddress() const
 PString RadiusAttr::AsVsaString() const
 {
 	if (!IsValid() || m_type != VendorSpecific)
-		return PString();
+		return PString::Empty();
 		
 	const PINDEX len = m_length;
 
 	if (len <= VsaRfc2865FixedHeaderLength)
-		return PString();
+		return PString::Empty();
 	else
 		return PString((const char*)m_vendorValue, len - VsaRfc2865FixedHeaderLength);
 }
@@ -786,7 +603,7 @@ PString RadiusAttr::AsCiscoString() const
 {
 	if (!IsValid() || m_type != VendorSpecific 
 			|| GetRadiusInteger(m_vendorId) != CiscoVendorId)
-		return PString();
+		return PString::Empty();
 		
 	const PINDEX len = m_length;
 	PINDEX offset = VsaRfc2865FixedHeaderLength;
@@ -804,7 +621,7 @@ PString RadiusAttr::AsCiscoString() const
 			i++;
 			
 	if (offset >= len)
-		return PString();
+		return PString::Empty();
 	else	
 		return PString((const char*)m_data + offset, len - offset);
 }
