@@ -3,11 +3,12 @@
 // PURPOSE OF THIS FILE: Give version info
 //
 // Copyright (C) 2003 Nils.Bokermann@mediaways.net
+// Copyright (c) 2006-2010, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -21,8 +22,8 @@
 const PString Toolkit::GKVersion()
 {
 	return PString(PString::Printf,
-		       "Gatekeeper(%s) Version(%s) Ext(pthreads=%d,radius=%d,mysql=%d,pgsql=%d,firebird=%d,odbc=%d,sqlite=%d,large_fdset=%d,crypto/ssl=%d,h46018=%d,h46023=%d)"
-		       " Build(%s, %s) Sys(%s %s %s)\r\n",
+		       "Gatekeeper(%s) Version(%s) Ext(pthreads=%d,radius=%d,mysql=%d,pgsql=%d,firebird=%d,odbc=%d,sqlite=%d,large_fdset=%d,crypto/ssl=%d,h46018=%d,h46023=%d,ldap=%d,ssh=%d)"
+		       " H323Plus(%d.%d.%d) PTLib(%d.%d.%d) Build(%s, %s) Sys(%s %s %s)\r\n",
 		       (const unsigned char*)(PProcess::Current().GetManufacturer()),
 		       (const unsigned char*)(PProcess::Current().GetVersion(true)),
 #ifdef P_PTHREADS
@@ -80,6 +81,18 @@ const PString Toolkit::GKVersion()
 #else
 				(int)0,
 #endif
+#if P_LDAP
+				(int)1,
+#else
+				(int)0,
+#endif
+#if HAS_LIBSSH
+				(int)1,
+#else
+				(int)0,
+#endif
+				OPENH323_MAJOR, OPENH323_MINOR, OPENH323_BUILD,
+				PTLIB_MAJOR, PTLIB_MINOR, PTLIB_BUILD,
 		       __DATE__, __TIME__,
 		       (const unsigned char*)(PProcess::GetOSName()),
 		       (const unsigned char*)(PProcess::GetOSHardware()),
