@@ -3,14 +3,12 @@
 // yasocket.cxx
 //
 // Copyright (c) Citron Network Inc. 2002-2003
+// Copyright (c) 2004-2010, Jan Willamowius
 //
-// This work is published under the GNU Public License (GPL)
+// This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
 // We also explicitly grant the right to link this code
-// with the OpenH323 library.
-//
-// initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
-// initial version: 03/14/2003
+// with the OpenH323/H323Plus and OpenSSL library.
 //
 //////////////////////////////////////////////////////////////////
 
@@ -20,6 +18,7 @@
 #include "rwlock.h"
 #include "yasocket.h"
 #include "Toolkit.h"
+#include "gk.h"
 
 using std::mem_fun;
 using std::bind1st;
@@ -63,7 +62,7 @@ bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 		std::vector<YaSocket*>::iterator last = remove_if(
 			fds.begin(), fds.end(),
 			not1(compose1(
-				bind1st(mem_fun(&large_fd_set::has), &fdset), 
+				bind1st(mem_fun(&large_fd_set::has), &fdset),
 				mem_fun(&YaSocket::GetHandle)
 				)));
 		fds.erase(last, fds.end());
@@ -72,14 +71,14 @@ bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 		   another 10-15% of performance gain. As it is not much under normal
 		   conditions, I leave it for thouse who want to squeeze a few more
 		   calls from their proxies;-)
-		   
+
 		   I did some performance tests (Duron 1.1GHz) with simulation of
 		   various fds selected sockets coverage (10%, 33%, 50%, 75%, 90%):
-		   
+
 		   LARGE_FDSET=1024  - 12% performance gain
 		   LARGE_FDSET=4096  - 15% performance gain
 		   LARGE_FDSET=16384 - 13% performance gain
-		   
+
 		   For LARGE_FDSET=4096 it took less than 1ms to manipulate the fdset
 		   and this grows in a linear fashion (LARGE_FDSET=16384 takes a few
 		   milliseconds to perform the same task).
@@ -89,7 +88,7 @@ bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 		const std::vector<YaSocket*>::reverse_iterator rendIter = fds.rend();
 		bool hasfd = false;
 		
-		// start from the end of the list, skip consecutive sockets 
+		// start from the end of the list, skip consecutive sockets
 		// that were not selected (find the first one selected)
 		while (j != rendIter) {
 			k--;
@@ -99,7 +98,7 @@ bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 			} else
 				++j;
 		}
-		// reorder remaining sockets, so non-selected sockets 
+		// reorder remaining sockets, so non-selected sockets
 		// are moved to the end of the vector
 		if (hasfd) {
 			while (++j != rendIter) {
@@ -557,7 +556,7 @@ PSocket *SocketSelectList::operator[](int i) const
 
 
 // class USocket
-USocket::USocket(IPSocket *s, const char *t) 
+USocket::USocket(IPSocket *s, const char *t)
 	: self(s), qsize(0), blocked(false), type(t)
 {
 }
@@ -662,7 +661,7 @@ bool USocket::ErrorHandler(PSocket::ErrorGroup group)
 			}
 			break;
 		default:
-			PTRACE(3, msg << " Error(" << group << "): " 
+			PTRACE(3, msg << " Error(" << group << "): "
 				<< PSocket::GetErrorText(e) << " (" << e << ':'
 				<< self->GetErrorNumber(group) << ')'
 				);
@@ -740,6 +739,8 @@ void SocketsReader::Stop()
 
 void SocketsReader::AddSocket(IPSocket *socket)
 {
+	if (socket == NULL)
+		return;
 	m_listmutex.StartWrite();
 	iterator iter = find(m_sockets.begin(), m_sockets.end(), socket);
 	if (iter == m_sockets.end()) {
@@ -779,8 +780,10 @@ bool SocketsReader::SelectSockets(SocketSelectList & slist)
 	}
 	ConfigReloadMutex.StartRead();
 #if PTRACING
-	PString msg(PString::Printf, "\t%u sockets selected from %u, total %u/%u", slist.GetSize(), ss, m_socksize, m_rmsize);
-	PTRACE(6, GetName() << msg);
+	if (PTrace::CanTrace(6)) {
+		PString msg(PString::Printf, "\t%u sockets selected from %u, total %u/%u", slist.GetSize(), ss, m_socksize, m_rmsize);
+		PTRACE(6, GetName() << msg);
+	}
 #endif
 	return true;
 }
@@ -902,6 +905,18 @@ bool TCPServer::CloseListener(TCPListenSocket *socket)
 
 void TCPServer::ReadSocket(IPSocket *socket)
 {
+	if (ShutdownMutex.WillBlock()) {
+		PTRACE(4, GetName() << "\tShutdown: Rejecting call on " << socket->GetName());
+		int rej = ::accept(socket->GetHandle(), NULL, NULL);
+		::shutdown(rej, 2 /* SHUT_RDWR */ );
+#if defined(_WIN32)
+		::closesocket(rej);
+#else
+		::close(rej);
+#endif
+		return;
+	}
+
 	PTRACE(4, GetName() << "\tAccept request on " << socket->GetName());
 	// rate limiting
 	if (cps_limit > 0) {
@@ -916,7 +931,7 @@ void TCPServer::ReadSocket(IPSocket *socket)
 			int rej = ::accept(socket->GetHandle(), NULL, NULL);
 			::shutdown(rej, 2 /* SHUT_RDWR */ );
 #if defined(_WIN32)
-			::_close(rej);
+			::closesocket(rej);
 #else
 			::close(rej);
 #endif
